import React, { Component } from 'react';
import 'chart.js/auto';
import { Bar } from 'react-chartjs-2';
import { Dropdown } from 'primereact/dropdown';
import { DataTable } from 'primereact/datatable';
import { Column } from 'primereact/column';
import _ from 'lodash';
import moment from 'moment';
import MathUtility from '../../../utils/math.utility';
import UnitConverter from '../../../utils/unit.converter';

/**
 * Failures rate report component for cycle report.
 */
class CycleFailureRate extends Component {

    constructor(props) {
        super(props);
    
        this.state = {}
    }

    componentDidMount() {
        this.setState({
            reportData: this.getReportData()
        });
    }

    componentDidUpdate(prevProps, prevState) {
        if (prevProps.data !== this.props.data) {
            this.setState({
                reportData: this.getReportData()
            });
        }
    }

    /**
     * Function that returns the data required for the chart with labels, datasets, colors for bar, line, barwidth, etc.,
     * @returns Object - with dataset, labels, colors, barwidth, etc.,
     */
    getBarData() {
        const barData = this.state.reportData.chartData ;
        return {
            labels: _.map(barData, 'month'),
            datasets: [
                {
                    type: 'bar',
                    label: 'Hours Lost',
                    borderColor: '#44a3ce',
                    backgroundColor: '#44a3ce',
                    borderWidth: 2,
                    barThickness: 50,
                    // barPercentage: 0.5,
                    data: _.map(barData, 'duration'),
                    borderColor: 'white',
                    borderWidth: 2,
                }
            ],
        };
    }

    /**
     * Function that returns the bar options like axis, scales, max axis value, tooltip, etc.,
     * @returns Object
     */
    getBarOptions() {
        return {
            elements: {
              bar: {
                borderWidth: 1,
              },
            },
            responsive: true,
            plugins: {
              legend: {
                position: 'bottom',
              },
              title: {
                display: true,
                text: 'LOFAR Observing Failures',
              },
            //   tooltip: {
            //     callbacks: {
            //         label: function(tooltipItem, data) {
            //             const itemData = _.find(resourceUtilization, ['type', tooltipItem.label]);
            //             return `${itemData?itemData.percent:""}% (${itemData?itemData.value:""} ${itemData?itemData.unit:""})`;
            //         }
            //     }
            //   }
            }
        };
    }

    /**
     * Function to format data list after calculations and include average if require for table or chart.
     * @param {Boolean} forTable - flag to incluce average value if the data is for table component.
     * @returns Array
     */
    getReportData() {
        const data = this.props.data;
        const timeConversionFactor = UnitConverter.resourceUnitMap.time.conversionFactor;
        // Assuming data structure
        // [{cycle: 'Cycle 15', failures: [{month: "2020-06-01", duration: 1200}, {month: "2020-07-01", duration: 1300},....]}]
        let reportData = {chartData: [], monthTableData: [], cycleTableData: []};
        _.map(data, repData=> {
            const monthlyFailures = repData.failures.months;
            let cycleMonthlyFailures = []
            for (const failureData of monthlyFailures) {
                cycleMonthlyFailures.push({   cycle: repData.cycle,
                                    date: failureData.month,
                                    month: moment(failureData.month).format("YYYY-MM"),
                                    // duration: failureData.duration?(failureData.duration/timeConversionFactor).toFixed(2):null
                                    duration: failureData.total_failed?(failureData.total_failed/timeConversionFactor).toFixed(2):null
                });
            }
            reportData.chartData = reportData.chartData.concat(cycleMonthlyFailures);
            reportData.cycleTableData.push({
                cycle: repData.cycle,
                period: _.minBy(cycleMonthlyFailures, 'date')?
                        `${(_.minBy(cycleMonthlyFailures, 'date')).month} -  ${(_.maxBy(cycleMonthlyFailures, 'date')).month}`:'',
                duration: ((_.sumBy(monthlyFailures, 'total_failed'))/timeConversionFactor).toFixed(2)
            });
        });
        reportData.monthTableData = reportData.chartData;
        if (this.props.setExportData && reportData.monthTableData.length>0) {
            this.props.setExportData('Failures', 
                                        _.map(_.cloneDeep(reportData.monthTableData), data => { 
                                            delete data['date']; return data;
                                        }));
        }
        return reportData;
    }

    renderReportTable() {
        const reportTableData = this.state.reportData.tableData;
        const reportColumns = _.keys(reportTableData[0]).map((col,i) => {
            return <Column key={col} field={col} header={col} />;
        });
        return (
            <DataTable value={reportTableData} resizableColumns columnResizeMode="expand" className="card" style={{paddingLeft: '0em'}}>
                { reportColumns }
            </DataTable>
        );
    }
    
    render() {
        return (
            <>
                {this.state.reportData &&
                <>
                    <Bar data={this.getBarData()} options={this.getBarOptions()} />
                    <DataTable value={this.state.reportData.monthTableData} resizableColumns columnResizeMode="expand" className="card" style={{paddingLeft: '0em'}}>
                        <Column field="cycle" header="Cycle Code" />
                        <Column field="month" header="Period (YYYY-MM)" />
                        <Column field="duration" header="Hours Lost per Month" />
                    </DataTable>
                    <DataTable value={this.state.reportData.cycleTableData} resizableColumns columnResizeMode="expand" className="card" style={{paddingLeft: '0em'}}>
                        <Column field="cycle" header="Cycle Code" />
                        <Column field="period" header="Period (YYYY-MM)" />
                        <Column field="duration" header="Hours Lost per Cycle" />
                    </DataTable>
                </>
                }
            </>
        );
    }
}

export default CycleFailureRate;