import React,{ Component } from "react";
import 'chart.js/auto';
import { Bar } from 'react-chartjs-2';
import _ from 'lodash';
import { DataTable } from 'primereact/datatable';
import { Column } from 'primereact/column';
import MathUtility from "../../../utils/math.utility";
import UnitConverter from "../../../utils/unit.converter";

/** Compoent to show usage in ILT and Local mode */
class CyclesILTLocalUsage extends Component{

    constructor(props) {
        super(props);

        this.state = {
            data : [],
            reportData: []
        };
    }

    componentDidMount() {
        if(this.props.data){
            this.setState({data: this.props.data});
            this.getReportData(this.props.data);
        }
    }

    componentDidUpdate() {
        if(this.props.data !== this.state.data){
            this.setState({data: this.props.data});
            this.getReportData(this.props.data);
        }
    }

    /**
     * Function that returns the data required for the chart with lablels, datasets, colors for bar, line, barwidth, etc.,
     * @returns Object - with dataset, labels, colors, barwidth, etc.,
     */
    getBarData() {
        let reportData = this.state.reportData;
        reportData = reportData.slice(0,-1);
        return {
            labels: _.map(reportData, 'cycle'),
            datasets:[

                {   label: 'ILT-mode observing', 
                    data:_.map(reportData, 'observing'),
                    backgroundColor: '#4472C4',
                    barThickness: 75,
                    fill: 'origin'    
                },
                {   label: 'ILT-mode idle/test', 
                    data:_.map(reportData, 'idle'),
                    backgroundColor: '#ED7D31',
                    barThickness: 75,
                    fill: 'origin'    
                },
                {   label: 'Local-mode', 
                    data:_.map(reportData, 'local_mode'),
                    backgroundColor: '#A5A5A5',
                    barThickness: 75,
                    fill: 'origin'
                },
            ]
        };
    }

    /**
     * Function that returns the bar options like axis, scales, max axis value, tooltip, etc.,
     * @returns Object
     */
    getBarOptions() {
        return {
            indexAxis: 'x',
            // Elements options apply to all of the options unless overridden in a dataset
            // In this case, we are setting the border of each horizontal bar to be 1px wide
            elements: {
              bar: {
                borderWidth: 1
              },
            },
            scales: {
                x: {
                    stacked: true,
                },
                y: {
                    beginAtZero: true,
                    stacked: true,   
                }
            },
            responsive: true,
            plugins: {
              filler: {
                  propagate: true
              },
              legend: {
                position: 'bottom',
              },
              title: {
                display: true,
                text: 'ILT Usage Plot',
              },
            }
        };
    }

    /**Function to format data as required for the reports
     * 
     * @param {Array} data  
     */
    getReportData(data) {
        let reportData = [];
        const timeConversionFactor = UnitConverter.resourceUnitMap["time"].conversionFactor;
        data.map(iltCycleData => {
            let iltReportData = {};
            iltReportData['cycle'] = iltCycleData.cycle;
            iltReportData['observing'] = (iltCycleData.usage_mode['ILT mode'].observing/timeConversionFactor).toFixed(2);
            iltReportData['idle'] = (iltCycleData.usage_mode['ILT mode']['idle/test']/timeConversionFactor).toFixed(2);
            iltReportData['local_mode'] = (iltCycleData.usage_mode['stand-alone mode'].total/timeConversionFactor).toFixed(2);
            iltReportData['ilt_total'] = (iltCycleData.usage_mode['ILT mode'].total/timeConversionFactor).toFixed(2);
            iltReportData['ilt_mode_dutch'] = null;
            reportData.push(iltReportData);
        });
        if (reportData.length>0) {
            reportData.push({
                cycle: 'Average',
                observing: MathUtility.getAverage(_.map(reportData, 'observing')),
                idle: MathUtility.getAverage(_.map(reportData, 'idle')),
                local_mode: MathUtility.getAverage(_.map(reportData, 'local_mode')),
                ilt_total: MathUtility.getAverage(_.map(reportData, 'ilt_total')),
                ilt_mode_dutch: MathUtility.getAverage(_.map(reportData, 'ilt_mode_dutch'))
            });
        }
        if (this.props.setExportData && reportData.length>0) {
            this.props.setExportData('ILT_LocalUsage', reportData);
        }
        this.setState({reportData: reportData});
    }

    render() {
        let reportData = this.state.reportData;
        return(
            <React.Fragment>
                <div  id={`cycles-ilt-usage`}
                    style={{paddingTop: "10px", paddingBottom: "10px"}}>
                    <Bar data={this.getBarData()} options={this.getBarOptions()} width="50%" height="20" />
                </div>
                <div className="ilt-time-details" id={`cycle-ilt-details`}>
                    <DataTable value={reportData} resizableColumns columnResizeMode="expand" className="card" style={{paddingLeft: '0em', textAlign: 'center'}}>
                        <Column field="cycle" header="Cycle Code"></Column>
                        <Column field="observing" header="ILT-mode observing (hrs)"></Column>
                        <Column field="idle" header="ILT-mode idle/test (hrs)"></Column>
                        <Column field="local_mode" header="Local-mode (hrs)"></Column>
                        <Column field="ilt_total" header="ILT-mode (total, in hrs)"></Column>
                        <Column field="ilt_mode_dutch" header="ILT-mode (Dutch array observing, in hrs)"></Column>
                    </DataTable>
                </div>
            </React.Fragment>
        )
    }
}

export default CyclesILTLocalUsage;