import React,{ Component } from "react";
import 'chart.js/auto';
import { Bar } from 'react-chartjs-2';
import _ from 'lodash';
import { DataTable } from 'primereact/datatable';
import { Column } from 'primereact/column';
import MathUtility from "../../../utils/math.utility";


const REGULAR_DDT_COMM = ['REGULAR', 'DDT', 'COMMISSIONING', 'USER_SHARED_SUPPORT','UNASSIGNED']
class CycleTelescopeTime extends Component{

    constructor(props){
        super(props);

        this.state= {
            data:[],
            reportData:[]
        };
    }

    componentDidMount() {
        if(this.props.data){
            this.setState({data: this.props.data})
            this.getReportData(this.props.data)
        }
    }

    componentDidUpdate(){
        if(this.props.data !== this.state.data){
            this.setState({data: this.props.data})
            this.getReportData(this.props.data)
        }
    }

    /**
     * Function that returns the data required for the chart with lablels, datasets, colors for bar, line, barwidth, etc.,
     * @returns Object - with dataset, labels, colors, barwidth, etc.,
     */
    getBarData() {
        let reportData = this.state.reportData;
        reportData = reportData.slice(0,-1);
        return {
            labels: _.map(reportData, 'cycle'),
            datasets:[
                
                {   label: 'Regular_DDT_Comm', 
                    data:_.map(reportData, 'reg_ddt_comm'),
                    backgroundColor: '#4472C4',
                    barThickness: 75,  
                },
                {   label: 'Filler', 
                    data:_.map(reportData, 'filler'),
                    backgroundColor: '#ED7D31',
                    barThickness: 75,       
                },
                {   label: 'Failed', 
                    data:_.map(reportData, 'failed'),
                    backgroundColor: '#A5A5A5',
                    barThickness: 75,       
                },
                {   label: 'System/Idle', 
                    data:_.map(reportData, 'idle'),
                    backgroundColor:  '#FFC000',
                    barThickness: 75,       
                }
            ]
        };
    }

    /**
     * Function that returns the bar options like axis, scales, max axis value, tooltip, etc.,
     * @returns Object
     */
    getBarOptions() {
        return {
            indexAxis: 'x',
            // Elements options apply to all of the options unless overridden in a dataset
            // In this case, we are setting the border of each horizontal bar to be 1px wide
            elements: {
              bar: {
                borderWidth: 1,
              },
            },
            scales: {
                x: {
                    stacked: true,
                    categoryPercentage: 1.0,
                    barPercentage: 1.0   
                },
                y: {
                    stacked: true,
                    max: 100,
                }
            },
            responsive: true,
            plugins: {
              legend: {
                position: 'bottom',
              },
              title: {
                display: true,
                text: 'Telescope time distribution',
              },
            }
        };
    }

    /**
     * Function for calculating the date for the report
     * @param {Object} groupData Formated data required for calculations
     * @param {String} groupType 
     * @param {Float} totalCycle Sum of all the totals for percentage calculations
     * @returns Float
     */
    calculateReportData(groupData, groupType, totalCycle) {
        let groupArr = Object.values(groupData);
        let percent;
        if(groupType === 'reg_ddt_comm' || groupType === 'filler')
            percent =  parseFloat(((_.sumBy(groupArr, cal => cal.durations.succeeded )/totalCycle)*100).toFixed(0));
        if(groupType === 'failed')
            percent =  parseFloat(((_.sumBy(groupArr, cal => cal.durations.failed )/totalCycle)*100).toFixed(0));
        if(groupType === 'idle')
            percent =  parseFloat((((_.sumBy(groupArr, cal => cal.durations.idle ) + groupData['TEST'].durations.succeeded) /totalCycle)*100).toFixed(0));
        
        return percent > 0 ? percent:null;
    }

    /**
     * Function for formating data as required for the Chart and Table
     * @param {Array} data 
     */
    getReportData(data) {
        let reportData =[];
        data.map(cycleData => {
            let cycleReportData = {};
            let cycleTelescopeTime = cycleData.telescope_time_distribution;
            let totalCycleDuration = _.sumBy(Object.values(cycleTelescopeTime), ttdValue => ttdValue.durations.total);
            cycleReportData['cycle'] = cycleData.cycle;
            if(totalCycleDuration > 0){
                cycleReportData['reg_ddt_comm'] = this.calculateReportData(_.pick(cycleTelescopeTime,REGULAR_DDT_COMM), 'reg_ddt_comm', totalCycleDuration);                                       
                cycleReportData['filler'] = this.calculateReportData(_.pick(cycleTelescopeTime,['FILLER']), 'filler', totalCycleDuration);               
                cycleReportData['failed'] = this.calculateReportData(cycleTelescopeTime, 'failed', totalCycleDuration);
                cycleReportData['idle'] = this.calculateReportData(cycleTelescopeTime, 'idle', totalCycleDuration);
            }
            reportData.push(cycleReportData);
        });
        if (reportData.length>0) {
            reportData.push({cycle: 'Average',
                                reg_ddt_comm: MathUtility.getAverage(_.map(reportData, 'reg_ddt_comm')), 
                                filler: MathUtility.getAverage(_.map(reportData, 'filler')),
                                failed: MathUtility.getAverage(_.map(reportData, 'failed')), 
                                idle: MathUtility.getAverage(_.map(reportData, 'idle'))
                            });
        }
        if (this.props.setExportData && reportData.length>0) {
            this.props.setExportData('TelescopeTime', reportData);
        }
        this.setState({reportData : reportData});
    }

    render() {
        const reportData = this.state.reportData;
        return(
            <React.Fragment>
                <div  id={`cycles-telescope-time`}
                    style={{paddingTop: "10px", paddingBottom: "10px"}}>
                    <Bar data={this.getBarData()} options={this.getBarOptions()} width="50%" height="20" />
                </div>
                <div className="ttd-details" id={`cycle-ttd-details`}>
                    <DataTable value={reportData} resizableColumns columnResizeMode="expand" className="card" style={{paddingLeft: '0em', textAlign: 'center'}}>
                        <Column field="cycle" header="Cycle Code"></Column>
                        <Column field="reg_ddt_comm" header="Regular/DDT/Comm (%)"></Column>
                        <Column field="filler" header="Filler (%)"></Column>
                        <Column field="failed" header="Failed (%)"></Column>
                        <Column field="idle" header="System/Idle (%)"></Column>
                    </DataTable>
                </div>
            </React.Fragment>
        )
    }
}

export default CycleTelescopeTime;