import React, { Component } from 'react';
import { Link } from 'react-router-dom';
import moment from 'moment';
import _ from 'lodash';
import Papa from "papaparse";
import jsPDF from 'jspdf';
import html2canvas from 'html2canvas';
import ReactToPrint from "react-to-print";

import { AppLoader } from '../../layout/components/AppLoader';
import { appGrowl } from '../../layout/components/AppGrowl';
import ProjectService from '../../services/project.service';
import { AutoComplete } from 'primereact/autocomplete';
import { Calendar } from 'primereact/calendar';
import { Button } from 'primereact/button';
import { ProgressBar } from 'primereact/progressbar';
import CycleService from '../../services/cycle.service';
import ProjectReport from './project.report';

// Constants for SU details table column property name to be used for identifying the properties in the 
// report data and title for displaying in reports and while exporting them.
const SU_DETAILS_COLUMNS = [{name: "su_name", headerTitle: "SU Name & Link in TMSS", propertyName: "name"},
                            {name: "su_status", headerTitle: "SU Status Failed / Success ", propertyName: "status"},
                            {name: "su_acceptance_status", headerTitle: "SU Acceptance Failed / Success / Pending", propertyName: "acceptance_status"},
                            {name: "su_execDate", headerTitle: "SU Execution Date", propertyName: "execDate"},
                            {name: "observTime", headerTitle: "Time Observed (hr)", propertyName: "observingTime"},
                            {name: "observTimeInc", headerTitle: "Time Observed Incremental (hr)", propertyName: "observingTimeInc"},
                            {name: "observTimeLeft", headerTitle: "Time left for Observing (hr)", propertyName: "observingTimeLeft"},
                            {name: "observTimeIncPercent", headerTitle: "Completed Observing Time(%)", propertyName: "observingTimeIncPercent"},
                            {name: "processTime", headerTitle: "Time Processed (hr)", propertyName: "processTime"},
                            {name: "processTimeInc", headerTitle: "Time Processed Incremental (hr)", propertyName: "processTimeInc"},
                            {name: "processTimeLeft", headerTitle: "Time left for Processing (hr)", propertyName: "processTimeLeft"},
                            {name: "processTimeIncPercent", headerTitle: "Completed Processing Time(%)", propertyName: "processTimeIncPercent"},
                            {name: "ingestDate", headerTitle: "LTA Ingest Date", propertyName: "ingestDate"},
                            {name: "ingestDataSize", headerTitle: "Ingested Data Size(TB)", propertyName: "ingestDataSize"},
                            {name: "ingestDataIncPercent", headerTitle: "Used LTA Allocation (Incremental) (%)", propertyName: "ingestDataIncPercent"},
                            {name: "observationSASId", headerTitle: "SAS ID (Observations)", propertyName: "observationSASId"},
                            {name: "pipelinseSASId", headerTitle: "SAS ID (Pipelines)", propertyName: "pipelinseSASId"},
                            ]

/**
 * Main component for Project Report
 */
class ProjectReportMain extends Component {
    
    constructor(props) {
        super(props);
        this.state = {
            pageLoading: true,
            selectedProjects: [],   // Used by the Autocomplete field
            reportProjects: [],     // Used to generate the report for selected project
            reportPeriod: []        // Period of reporting
        };
        this.searchProjects = this.searchProjects.bind(this);
        this.selectProjects = this.selectProjects.bind(this);
        this.setReportPeriod = this.setReportPeriod.bind(this);
        this.setReportProjects = this.setReportProjects.bind(this);
        this.renderProjectReports = this.renderProjectReports.bind(this);
        this.setReportData = this.setReportData.bind(this);
        this.downloadCSV = this.downloadCSV.bind(this);
        this.downloadPDF = this.downloadPDF.bind(this);
        this.clearAll = this.clearAll.bind(this);
        this.toggleScroll = this.toggleScroll.bind(this);
        this.getPageMargins = this.getPageMargins.bind(this)
    }

    componentDidMount() {
        ProjectService.getResources()
        .then(resourceList => {this.resourceList = resourceList});
        CycleService.getAllCycles()
        .then(cycles => {this.cycles = cycles});
        ProjectService.getProjects()
        .then(projects => { this.setState({projects: projects, pageLoading: false}) });
        window.addEventListener('scroll', this.toggleScroll);
    }

    /**
     * Function passed to AutoComplete component to search and list
     * @param {*} event 
     */
    searchProjects(event) {
        setTimeout(() => {
            let suggestedProjects;
            if (!event.query.trim().length) {
                suggestedProjects = [...this.state.projects];
            }
            else {
                suggestedProjects = this.state.projects.filter((project) => {
                    // For matching the start of the project with the search text
                    // return project.name.toLowerCase().startsWith(event.query.toLowerCase());
                    // For values containing the search text
                    return project.name.toLowerCase().indexOf(event.query.toLowerCase())>=0;
                });
            }
            this.setState({ suggestedProjects });
        }, 250);
    }

    /**
     * Function to set the selected projects from the AutoComplete component
     * @param {Array} projects 
     */
    selectProjects(projects) {
        let reportPeriod = [];
        if (projects.length) {
            let projectCycleDates = [];
            // Get all cycle dates of the project associated with
            for (const project of projects) {
                if (project.cycles_ids.length > 0) {
                    for (const cycleId of project.cycles_ids) {
                        const cycle = _.find(this.cycles, ['name', cycleId]);
                        projectCycleDates.push(moment(cycle.start));
                        projectCycleDates.push(moment(cycle.stop));
                    }
                }
            }
            // Get the minimum and maximum date from the cycle dates and set as the report period
            let minProjectCycleDate = _.min(projectCycleDates);
            let maxProjectCycleDate = _.max(projectCycleDates);
            if (minProjectCycleDate && maxProjectCycleDate) {
                reportPeriod = [minProjectCycleDate.toDate(), maxProjectCycleDate.toDate()];
            }
        }
        this.setState({selectedProjects: projects, reportPeriod: reportPeriod});
    }

    /**
     * Set Report period and close the calendar if both values in the range are selected.
     * @param {Array} value - Array of date objects
     */
     setReportPeriod(value) {
        this.setState({reportPeriod: value?value:[]});
        if (value && value.length===2 && value[1]) {
            document.getElementsByClassName('p-datepicker')[0].style.display = "none";
        }
    }
    
    /**
     * Function to generate the PDF of the report.
     */
    async downloadPDF() {
        this.setState({isDownloading: true});
        let reportDivs = document.getElementsByClassName('report-div');

        // Create a PDF document with landscape orientation, 1st report div width and the max height of all report divs in pixels
        const pdf = new jsPDF('l', 'px',  [ (reportDivs[0].clientWidth+100), (reportDivs[0].clientHeight+100)]);
        let pageIndex = 0;
        // Draw each project report in canvas and create image of the canvas before saving the pdf
        for (const project of this.state.reportProjects) {
            // Create page for each project
            if (pageIndex > 0) {
                pdf.addPage([reportDivs[0].clientWidth+100, reportDivs[pageIndex].clientHeight+100], pageIndex===5?'p':'l');
            }
            const reportElement = document.getElementById(`${project.name}-report-div`);
            const options = {
                foreignObjectRendering: true,
                width: reportElement.clientWidth,
                height: reportElement.clientHeight
            };
            const projectCanvas = await html2canvas(reportElement, options);
            pdf.addImage(projectCanvas.toDataURL('image/jpeg'), 'JPEG', 50, 50, reportDivs[pageIndex].clientWidth, reportDivs[pageIndex].clientHeight);
            pageIndex++;
        }
        // To open the generated PDF in new window
        // pdf.output('dataurlnewwindow');
        const reportPeriod = `${moment(this.state.reportPeriod[0]).format("DDMMMYYYY")}-${moment(this.state.reportPeriod[1]).format("DDMMMYYYY")}`;
        pdf.save(`Project_${reportPeriod}.pdf`);
        this.setState({isDownloading: false});
    }

    /**
     * Function to download the report data in CSV format
     */
    downloadCSV() {
        let csvConfig = {};
        let csvList= [];            //Data list to export
        let colHeaders = [];        //Column header to export. Both no of headers and data values should be same.
        let isColHeaderSet = false; //Flag to identify if colHeader is added
        // For every project of the selected project get report data
        for(const projectName of _.keys(this.projectsReportData)) {
            // Get the whole report data set from ProjectReport component
            const projectReport = this.projectsReportData[projectName];
            // Get Project Data
            const reportData = projectReport.reportData;
            // Get Project Resources Data
            const projectResources = projectReport.projectResources;
            // Add project and SU data to the csv list
            projectReport.suStatsList.map((data, index) => {
                let csvData = {};
                const colKeys = _.keys(data);
                /* Add Project Details to the first row of the project SU data */
                if (!isColHeaderSet) {
                    colHeaders.push("Project");
                    colHeaders.push("Contact Project Friend");
                    colHeaders.push("Awarded Observing Time(hours)");
                    colHeaders.push("Awarded Processing Time(hours)");
                    colHeaders.push("Awarded LTA Storage(TB)");
                }
                if (index === 0) {
                    csvData["Project"] = projectName;
                    csvData["Contact Project Friend"] = reportData.friends?reportData.friends.join(","):"-"
                    csvData["Awarded Observing Time(hours)"] = projectResources["LOFAR Observing Time"]?projectResources["LOFAR Observing Time"].convertedValue:"-";
                    csvData["Awarded Processing Time(hours)"] = projectResources["CEP Processing Time"]?projectResources["CEP Processing Time"].convertedValue:"-";
                    csvData["Awarded LTA Storage(TB)"] = projectResources["LTA Storage"]?projectResources["LTA Storage"].convertedValue:"-";
                }
                // For every column of the data, replace the column name with the column title
                for (const colKey of colKeys) {
                    let colHeader = _.find(SU_DETAILS_COLUMNS, ["propertyName", colKey]);
                    colHeader = colHeader?colHeader.headerTitle:_.upperFirst(colKey);
                    if (!isColHeaderSet) {
                        colHeaders.push(colHeader);
                    }
                    csvData[colHeader] = data[colKey];
                }
                csvList.push(csvData);
                isColHeaderSet = true;
            });
            
        }
        if (csvList.length > 0) {
            // Pass column headers to CSV parser
            if (colHeaders.length > 0) {
                csvConfig.columns = colHeaders;
            }
            // Parse the CSV list and header opject to create CSV string
            const csvString = Papa.unparse(csvList, csvConfig);
            // Create a CSV BLOB from the csvString
            const blob = new Blob([csvString], { type: "text/csv" });

            // Add dummy link to click and download the created CSV
            let a = document.createElement("a");
            document.body.appendChild(a);
            a.style = "display: none";
            let  url = window.URL.createObjectURL(blob);
            a.href = url;
            const reportPeriod = `${moment(this.state.reportPeriod[0]).format("DDMMMYYYY")}-${moment(this.state.reportPeriod[1]).format("DDMMMYYYY")}`;
            a.download = `Project_${reportPeriod}.csv`;
            a.click();
            window.URL.revokeObjectURL(url);
        }   else {
            appGrowl.show({severity: 'info', summary: 'No Data', detail: 'None of the project has Scheduling Unit data to download'});
        }
    }

    /**
     * Function called to set report project from the selected projects.
     */
    setReportProjects() {
        this.setState({ reportProjects: _.cloneDeep(this.state.selectedProjects)});
    }

    /**
     * Callback function passed to the ProjectReport component to get the report data to use in CSV export.
     * @param {String} projectName 
     * @param {Object} projectReportData - All required data from the ProjectReport component
     */
    setReportData(projectName, projectReportData) {
        let projectsReportData = this.projectsReportData || {};
        projectsReportData[projectName] = projectReportData;
        this.projectsReportData = projectsReportData;
    }

    /**
     * Clears the report filters and report divs
     */
    clearAll(){
        this.projectsReportData = {};
        this.setState({selectedProjects:[], reportPeriod:[], reportProjects:[]});
    }

    /**
     * To render multiple project reports from a list of projects.
     * @returns Component
     */
    renderProjectReports() {
        return (
            <>
                {
                    this.state.reportProjects.map((project, index) => {
                        return <div style={{marginBottom: "20px"}}>
                                <style>{this.getPageMargins()}</style>
                                    <ProjectReport project={project} resourceList={this.resourceList} 
                                        reportPeriod={this.state.reportPeriod} SU_DETAILS_COLUMNS={SU_DETAILS_COLUMNS} 
                                        passReportData={this.setReportData} />
                                </div>
                    })
                }
            </>
        );
    }
    /**
     * Callback function to window scroll event listener to set or unset flag to show or hide ScrollToTop button
     */
     toggleScroll() {
        const scrolled = document.documentElement.scrollTop;
        if (scrolled > 300){
            this.setState({scrollTopVisible: true});
        } else if (scrolled <= 300){
            this.setState({scrollTopVisible: false});
        }
    };

    /**
     * Function to scroll to top of the window on clicking ScrollToTop button.
     */
     scrollToTop() {
        window.scrollTo({
            top: 75, 
            behavior: 'smooth'});
    }
    getPageMargins() {
        return `@page { margin: 138px 72px 138px 72px !important; }`;
      };

    render() {
        return(
            <React.Fragment>
                {this.state.pageLoading &&
                    <AppLoader />}
                {!this.state.pageLoading &&
                <div className="report-toolbar p-grid" style={{marginTop: "10px", paddingBottom: "10px", borderBottom: "1px solid lightgrey"}}>
                    <label htmlFor="project" className="col-lg-1 col-md-2 col-sm-12">Project </label>
                    <AutoComplete className="col-lg-3 col-md-3 col-sm-12" multiple forceSelection autoFocus autoHighlight
                        tooltip="Type few letters of the project and select the project suggested" tooltipOptions={{position: "top"}}
                        value={this.state.selectedProjects} suggestions={this.state.suggestedProjects}
                        completeMethod={this.searchProjects} field="name" onChange={(e) => this.selectProjects(e.value)} />
                    <label htmlFor="period" className="col-lg-2 col-md-2 col-sm-12" style={{paddingRight: "0px"}}>For Period </label>
                    <div className="col-lg-3 col-md-3 col-sm-12 report-calendar">
                        <Calendar value={this.state.reportPeriod} selectionMode="range" dateFormat="yy-mm-dd"
                                tooltip="Select the date range for the report"
                                showButtonBar onChange={e => this.setReportPeriod(e.value)}></Calendar>
                    </div>
                    <div className="col-lg-2 col-md-1 col-sm-12">
                        <Button label="" className="p-button-primary" icon="pi pi-check" tooltip="Generate Report"
                                onClick={this.setReportProjects} disabled={this.state.selectedProjects.length===0 || this.state.reportPeriod.length!==2} />
                        <Button label="" className="p-button-primary" icon="fas fa-sync-alt" tooltip="Clear All"
                                onClick={this.clearAll} style={{marginLeft:"10px"}} />
                    </div>
                </div>
                }
                {this.state.isDownloading &&
                <div><ProgressBar mode="indeterminate" style={{ height: '5px' }}/></div> }
                {this.state.reportProjects.length>0 &&
                <>
                    <div ref={(el) => (this.reportDownloadBarRef = el)} >   
                        <Link to={{}} className="report-download-bar" style={{color: "#148048"}} title="Download Report Data in CSV format" 
                                onClick={this.downloadCSV}>
                                <i className="fas fa-file-csv"></i>
                        </Link> 
                        <Link to={{}} className="report-download-bar" style={{color: "#f20f00"}} title="Download Report as PDF" 
                                onClick={(e) => { // Scroll to the bottom of the report and back to the top before exporting
                                                    this.reportEndRef.scrollIntoView({behavior: "smooth", block: "end"});
                                                    this.reportDownloadBarRef.scrollIntoView({behavior: "smooth", block: "end"});
                                                    setTimeout(this.downloadPDF, 500)}}>
                                <i className="fas fa-file-pdf"></i>
                        </Link> 
                        <ReactToPrint
                            trigger={() => <i className="pi pi-print report-download-bar print-btn" title="Print Report"></i>}
                            content={() => this.reportPrintRef}
                        />
                    </div>
                    <div ref={(el) => (this.reportPrintRef = el)}>
                        { this.renderProjectReports()}
                    </div>
                    {/* Dummy reference div to scroll down before exporting to PDF so that 
                        all components will be rendered properly before exporting */}
                    <div ref={(el) => {this.reportEndRef = el}}></div>
                    {/* Scroll To Top button */}
                    {this.state.scrollTopVisible &&
                        <Button className="p-button-info p-button-raised float-btn-right-bottom"
                            onClick={this.scrollToTop}
                            icon="pi pi-arrow-up" tooltip="Move to top"></Button>
                    }
                </>
                }
            </React.Fragment>
        );
    }

}

export default ProjectReportMain;