import React from 'react';
import { cleanup, waitFor } from '@testing-library/react';
import '@testing-library/jest-dom';
import UtilService from "../../services/util.service";
import SchedulingSetCreate from './excelview.schedulingset';
import ScheduleService from '../../services/schedule.service';
import ProjectService from '../../services/project.service';
import TaskService from '../../services/task.service';
import AuthUtil from '../../utils/auth.util';
import SUServiceMock from '../../__mocks__/scheduleunit.service.data';
import ProjectServiceMock from '../../__mocks__/project.service.data';
import TaskServiceMock from '../../__mocks__/task.service.data';
import AuthServiceMock from '../../__mocks__/auth.service.data';
import { clickItem, removeReact18ConsoleErrors, renderPage, setMultiSelectValue } from "../../utils/test.helper";
import { setSchedulingUnitStrategy } from "./schedulingunit.test.helper";

let projectListSpy, observStrategiesSpy, taskTemplatesSpy, userPermissionSpy, rolesSpy, suConstraintTemplateSpy,
    templatePurposesSpy, templateStatesSpy,utcSpy;

//Jest issue: https://github.com/jestjs/jest/issues/9709: Cannot be called in beforeXX test method
const OBSERVATION_STRATEGY_TEMPLATES = SUServiceMock.getObservStrategies()

jest.setTimeout(120000);
removeReact18ConsoleErrors();

/**
 * To set mock spy for Services that have API calls to the back end to fetch data
 */
function createStandardPageMocks() {
    projectListSpy = jest.spyOn(ProjectService, 'getProjectList').mockImplementation(() => {
        return Promise.resolve(ProjectServiceMock.projectList)
    });

    observStrategiesSpy = jest.spyOn(ScheduleService, 'getObservationStrategies').mockResolvedValue(OBSERVATION_STRATEGY_TEMPLATES);

    suConstraintTemplateSpy = jest.spyOn(ScheduleService, 'getSchedulingConstraintTemplates').mockImplementation(() => {
        return Promise.resolve(SUServiceMock.getSUCTemplates());
    });

    taskTemplatesSpy = jest.spyOn(TaskService, 'getTaskTemplates').mockImplementation(() => {
        return Promise.resolve(TaskServiceMock.getTaskTemplates())
    });

    templatePurposesSpy = jest.spyOn(ScheduleService, 'getStrategyPurpose').mockImplementation(() => {
        return Promise.resolve(SUServiceMock.templatePurposes)
    });
    templateStatesSpy = jest.spyOn(ScheduleService, 'getStrategyState').mockImplementation(() => {
        return Promise.resolve(SUServiceMock.templateStates)
    });

    userPermissionSpy = jest.spyOn(AuthUtil, 'getUserPermissionByModule').mockImplementation(() => {
        return Promise.resolve(AuthServiceMock.scheduleunit_draft)
    });

    rolesSpy = jest.spyOn(ProjectService, 'getMyRoles').mockImplementation(() => {
        return Promise.resolve(SUServiceMock.getMyRoles)
    });
    const utcTime = '2023-08-11 06:20:45';
    utcSpy = jest.spyOn(UtilService, 'getUTC').mockResolvedValue(utcTime);
}

function getColumnTypes(columnDefs) {
    let columnTypes = columnDefs.map(col => {
        if (col.children) {
            return getColumnTypes(col.children)
        }
        return col.cellEditor
    });
    columnTypes = columnTypes.flat().filter(colType => colType !== undefined)
    return columnTypes;
}

describe('Scheduling Units Excel View create page default', () => {
    let pageContent;

    beforeEach(async () => {
        createStandardPageMocks();
        pageContent = await renderPage(<SchedulingSetCreate location={{ pathname: '/schedulingunit/' }} />);
    })

    afterEach(() => {
        jest.clearAllMocks();
        cleanup();
    });

    test("Renders with standard service mocks", async () => {
        waitFor(() => expect(pageContent.queryByText('Scheduling Unit(s) Add Multiple')).not.toBe(null));;
        waitFor(() => expect(projectListSpy).toHaveBeenCalled());
        waitFor(() => expect(observStrategiesSpy).toHaveBeenCalled());
        waitFor(() => expect(suConstraintTemplateSpy).toHaveBeenCalled());
        waitFor(() => expect(taskTemplatesSpy).toHaveBeenCalled());
        waitFor(() => expect(templatePurposesSpy).toHaveBeenCalled());
        waitFor(() => expect(templateStatesSpy).toHaveBeenCalled());
        waitFor(() => expect(userPermissionSpy).toHaveBeenCalled());
        waitFor(() => expect(rolesSpy).toHaveBeenCalled());
    });

    test("Selects a project, loads the project's sets, selects the test scheduling set and changes number of SUs", async () => {
        const buttonElement = pageContent.queryByTestId('add-schedulingset-button');
        expect(buttonElement).toHaveAttribute('disabled');
        let scheduleSetListSpy = jest.spyOn(ScheduleService, 'getSchedulingSets').mockImplementation((filter) => {
            let allSUSets = SUServiceMock.scheduleSetList;
            let project = filter.replace("project=", "");
            return Promise.resolve(allSUSets.filter(set => set.project_id === project));
        });
        await setMultiSelectValue(pageContent, "project", "high")
        await setMultiSelectValue(pageContent, "schedSet", "Test Scheduling Set")
        await waitFor( ()=> expect(scheduleSetListSpy).toHaveBeenCalled()); 
        expect(pageContent.queryByTestId("project").textContent).toContain("high");
        expect(pageContent.queryByTestId("schedSet").textContent).toContain("Test Scheduling Set");
        expect(buttonElement.hasAttribute("disabled")).toBeFalsy();

        const noOfSUelement = pageContent.queryByTestId("noOfSU");
        expect(noOfSUelement.textContent).toBe("Enter No. of SU (1 to 500)")
        expect(pageContent.queryByDisplayValue('10')).toBeDefined();
        await clickItem(noOfSUelement)
        await clickItem(pageContent.getByText("50"))
        expect(pageContent.queryByDisplayValue('10')).toBeNull()
        expect(pageContent.queryAllByDisplayValue('50').length==2);
    });
});





describe('Scheduling Units Excel View create page with an observation strategy template', () => {
   
    let pageContent;
    let scheduleSetListSpy, scheduleSetByIdSpy, saveSUFromStrategySpy, utcSpy, validatorSpy, stationGroupSpy;

    beforeEach(async () => {
        createStandardPageMocks();
        pageContent = await renderPage(<SchedulingSetCreate location={{pathname: '/schedulingunit/'}}/>);
        scheduleSetListSpy = jest.spyOn(ScheduleService, 'getSchedulingSets').mockImplementation((filter) => {
            let allSUSets = SUServiceMock.scheduleSetList;
            let project = filter.replace("project=", "");
            return Promise.resolve(allSUSets.filter(set => set.project_id === project));
        });
        scheduleSetByIdSpy = jest.spyOn(ScheduleService, 'getSchedulingBySet').mockImplementation(() => {
            return Promise.resolve(SUServiceMock.getSchedulingBySet())
        });
        await setMultiSelectValue(pageContent, "project", "high")
        await setMultiSelectValue(pageContent, "schedSet", "Test Scheduling Set")
    })

    afterEach(() => {
        jest.clearAllMocks();
        cleanup()
    });
    test("test predondition test Scheduling Units Excel View",  () => {
        expect(pageContent).not.toBeNull();
    });


    function skipper(observationStrategy) {
        if (observationStrategy.name=="IM LBA LoDSS - 5 Beams" && observationStrategy.state=="legacy" && observationStrategy.version===55 ) return true
        if (observationStrategy.name=="IM LBA Survey - 3 Beams" && observationStrategy.state=="legacy" && observationStrategy.version===28 ) return true
        if (observationStrategy.name=="IM LBA LoDSS - 5 Beams" && observationStrategy.state=="legacy" && observationStrategy.version===45 ) return true
        return observationStrategy.name=="IM LBA Survey - 3 Beams" && observationStrategy.state=="legacy" && observationStrategy.version===28;
    }

    test.each(OBSERVATION_STRATEGY_TEMPLATES)(`Correctly creates and checks headers for template '$name: $purpose, $state, v$version'`, async (observationStrategy) => {
        
        /* Temporary Tweak until these are removed from production*/
        if (skipper(observationStrategy)) return true;


        expect(scheduleSetListSpy).toHaveBeenCalled();
        expect(scheduleSetByIdSpy).toHaveBeenCalled();
        await setSchedulingUnitStrategy(observationStrategy, pageContent);

        const gridElement = document.querySelector(".ag-root-wrapper").__agComponent.gridOptions;
        //Min_distance section in the grid equals the scheduling constraints section in the template
        let columnHeaderNames = gridElement.columnDefs.map(columnDef =>
            columnDef.headerName === "Min_distance" ? "Scheduling Constraints".toLowerCase() : columnDef.headerName.toLowerCase()
        );
        const strategyParameterNames = observationStrategy.template.parameters.map(param => param.name.toLowerCase())
        expect(columnHeaderNames).toEqual(expect.arrayContaining(strategyParameterNames))

        expect(gridElement.rowData.length).toBe(10)

        let saveSUFromStrategySpy = jest.spyOn(ScheduleService, 'saveSchedulingUnit').mockImplementation(() => {
            let schedulingUnitDraft = SUServiceMock.getSchedulingUnitFromObservStrategy(observationStrategy);
            schedulingUnitDraft["isSUUpdated"] = true;
            return Promise.resolve(schedulingUnitDraft);
        });
        await waitFor(() => expect(pageContent.queryByTestId('save-btn')).toBeInTheDocument());
        await clickItem(pageContent.queryByTestId('save-btn'))
        expect(saveSUFromStrategySpy).not.toHaveBeenCalled();
        expect(pageContent.getByRole('dialog')).toBeInTheDocument(); //pop-up indicating errored SU
    });


    test.each(OBSERVATION_STRATEGY_TEMPLATES)(`Checks column types for template '$name: $purpose, $state, v$version'`, async (observationStrategy) => {
        await setSchedulingUnitStrategy(observationStrategy, pageContent);

        await waitFor(() => expect(pageContent.queryByTestId('select-multiple-su-input')).toBeInTheDocument());
        await clickItem(pageContent.queryByTestId('select-multiple-su-input'))

        const multipleInputValuesGridElement = document.querySelector(".ag-root-wrapper").__agComponent.gridOptions;
        const columnDefs = multipleInputValuesGridElement.columnDefs;

        const uniqueColumnTypes = new Set(getColumnTypes(columnDefs));
        const knownColumnTypes = ['numericEditor', 'agSelectCellEditor', 'agDateInput', 'betweenEditor', 'multiselector',
            'offsetTimeInputmask', 'timeInputMask', 'degreeInputMask', 'station', 'beamformer']
        for (const type of uniqueColumnTypes) {
            expect(knownColumnTypes, `A template in the back-end defined a column type that is not implemented in the front-end: ${type}`).toContain(type);
        }
    });
    
/*
    test.skip.each(OBSERVATION_STRATEGY_TEMPLATES)(`Correctly creates, alters via multi input grid and saves for template '$name: $purpose, $state, v$version'`, async (observationStrategy) => {
        await setSchedulingUnitStrategy(observationStrategy, pageContent);

        await clickItem(pageContent.queryByTestId('select-multiple-su-input'))
        expect(pageContent.queryByRole('region')).not.toBeNull()

        const multipleInputValuesGridElement = document.querySelector(".ag-root-wrapper").__agComponent.gridOptions;

        //TODO: finish test for actual input. Refactor Spreadsheet cell editor components
     */
    });
   

