import moment from "moment";
import {
    cleanSkyPropertiesOfConstraintTemplate,
    cleanTimePropertiesOfConstraintTemplate,
    deleteInvalidProperties, editConstraintSettings, formatNestedTimeObject, updateSUConstraintTemplate
} from "./summary.helper";
import ScheduleService from "../../services/schedule.service";
import {appGrowl} from "../../layout/components/AppGrowl";

describe('cleanTimePropertiesOfConstraintTemplate', () => {
    const time = moment("2023-08-29T19:00:01")
    const timeString = time.format()
    const formattedTimeNow = `${time.format("YYYY-MM-DDTHH:mm:ss")}Z`;

    test.each([null, undefined, false])('should remove properties (after, at, before) if they exist but are invalid: %s', (invalidValue) => {
        const constraintTemplate = {
            time: {
                after: invalidValue,
                at: invalidValue,
                before: invalidValue,
            },
        };

        const result = cleanTimePropertiesOfConstraintTemplate(constraintTemplate);

        expect(result.time.at).toBeUndefined();
        expect(result.time.after).toBeUndefined();
        expect(result.time.before).toBeUndefined();
    });

    it('should format properties (after, at, before) correctly', () => {
        const constraintTemplate = {
            time: {
                after: timeString,
                at: timeString,
                before: timeString,
            },
        };

        const result = cleanTimePropertiesOfConstraintTemplate(constraintTemplate);

        expect(result.time.before).toBe(formattedTimeNow);
        expect(result.time.after).toBe(formattedTimeNow);
        expect(result.time.at).toBe(formattedTimeNow);

    });

    test.each([null, undefined, false])('should remove nested properties (between, not_between) if they exist but are invalid: %s', (invalidValue) => {
        const constraintTemplate = {
            time: {
                between: [{from: invalidValue}, {until: invalidValue}],
                not_between: [{from: invalidValue}, {until: invalidValue}],
            }
        };

        const result = cleanTimePropertiesOfConstraintTemplate(constraintTemplate);
        expect(result.time.between[0].from).toBeUndefined();
        expect(result.time.between[1].until).toBeUndefined();
        expect(result.time.not_between[0].from).toBeUndefined();
        expect(result.time.not_between[1].until).toBeUndefined();
    });

    it('should format nested properties (between, not_between) correctly', () => {
        const constraintTemplate = {
            time: {
                between: [{from: timeString}, {until: timeString}],
                not_between: [{from: timeString}, {until: timeString}],
            }
        };

        const result = cleanTimePropertiesOfConstraintTemplate(constraintTemplate);
        expect(result.time.between[0].from).toBe(formattedTimeNow);
        expect(result.time.between[1].until).toBe(formattedTimeNow);
        expect(result.time.not_between[0].from).toBe(formattedTimeNow);
        expect(result.time.not_between[1].until).toBe(formattedTimeNow);

    });

    it('should handle empty time properties', () => {
        const constraintTemplate = {
            time: {},
        };

        const result = cleanTimePropertiesOfConstraintTemplate(constraintTemplate);

        expect(result.time).toEqual({});
    });

    it('should not modify constraintTemplate if no time properties', () => {
        const constraintTemplate = {
            otherProperty: 'value',
        };

        const result = cleanTimePropertiesOfConstraintTemplate(constraintTemplate);

        expect(result).toEqual(constraintTemplate);
    });
});


describe('cleanSkyPropertiesOfConstraintTemplate', () => {
    it('should convert transit offset properties to seconds', () => {
        const constraintTemplate = {
            sky: {
                transit_offset: {
                    from: '01:00:00',
                    to: '02:30:00',
                },
            },
        };

        const expectedResult = {
            sky: {
                transit_offset: {
                    from: 3600,
                    to: 9000,
                },
            },
        };

        const result = cleanSkyPropertiesOfConstraintTemplate(constraintTemplate);

        expect(result).toEqual(expectedResult);
    });

    it('should convert other sky properties (so not transit offset or reference pointings) to radians', () => {
        const constraintTemplate = {
            sky: {
                min_distance: {sun: 180},
                min_elevation: {target: 45},
                reference_pointing: {angle1: 12},
            },
        };

        const minDistanceSun = Math.PI;
        const minElevationTarget = 0.785; // Approximately 45 degrees in radians

        const result = cleanSkyPropertiesOfConstraintTemplate(constraintTemplate);

        expect(result.sky.min_distance.sun).toBeCloseTo(minDistanceSun);
        expect(result.sky.min_elevation.target).toBeCloseTo(minElevationTarget);
        expect(result.sky.reference_pointing.angle1).toBe(12); //unchanged by design
    });

    it('should not modify constraintTemplate if no sky properties', () => {
        const constraintTemplate = {
            otherProperty: 'value',
        };

        const result = cleanSkyPropertiesOfConstraintTemplate(constraintTemplate);

        expect(result).toEqual(constraintTemplate);
    });

    it('should handle missing transit_offset properties', () => {
        const constraintTemplate = {
            sky: {
                transit_offset: {},
            },
        };

        const result = cleanSkyPropertiesOfConstraintTemplate(constraintTemplate);

        expect(result).toEqual(constraintTemplate);
    });

    it('should handle missing sky properties', () => {
        const constraintTemplate = {};

        const result = cleanSkyPropertiesOfConstraintTemplate(constraintTemplate);

        expect(result).toEqual({});
    });

    it('should handle decimal values', () => {
        const constraintTemplate = {
            sky: {
                marvel: 45.5,
                batman: 30.25,
            },
        };

        const expectedAzimuth = 0.794; // Approximately 45.5 degrees in radians
        const expectedElevation = 0.528; // Approximately 30.25 degrees in radians

        const result = cleanSkyPropertiesOfConstraintTemplate(constraintTemplate);

        expect(result.sky.marvel).toBeCloseTo(expectedAzimuth);
        expect(result.sky.batman).toBeCloseTo(expectedElevation);
    });
});

describe('deleteInvalidProperties', () => {
    it('should delete invalid properties', () => {
        const propertyNames = ['at', 'before', 'after'];
        const constraintTemplate = {
            time: {
                at: null,
                before: '2023-08-29T12:00:00Z',
                after: undefined,
                otherProperty: 'value',
            },
        };

        const expectedResult = {
            time: {
                before: '2023-08-29T12:00:00Z',
                otherProperty: 'value',
            },
        };

        const result = deleteInvalidProperties(propertyNames, constraintTemplate);

        expect(result).toEqual(expectedResult);
    });

    it('should not modify constraintTemplate if no invalid properties', () => {
        const propertyNames = ['at', 'before', 'after'];
        const constraintTemplate = {
            time: {
                at: '2023-08-29T12:00:00Z',
                before: '2023-08-30T12:00:00Z',
                after: '2023-08-28T12:00:00Z',
            },
        };

        const result = deleteInvalidProperties(propertyNames, constraintTemplate);

        expect(result).toEqual(constraintTemplate);
    });

    test.each([undefined, null, {}])('should handle invalid constraint template: %s', (invalidConstraintTemplate) => {
        const propertyNames = ['at', 'before', 'after'];

        const result = deleteInvalidProperties(propertyNames, invalidConstraintTemplate);

        expect(result).toEqual(invalidConstraintTemplate);
    });
});

describe('formatNestedTimeObject', () => {
    it('should format valid time properties', () => {
        const timeArray = {
            from: '2023-08-29T12:00:00Z',
            until: '2023-08-29T15:00:00Z',
        };

        const expectedResult = {
            from: moment('2023-08-29T12:00:00Z').utc(true).format(),
            until: moment('2023-08-29T15:00:00Z').utc(true).format(),
        };

        const result = formatNestedTimeObject(timeArray);

        expect(result).toEqual(expectedResult);
    });

    it('should ignore invalid time properties', () => {
        const timeArray = {
            from: null,
            until: '2023-08-29T15:00:00Z',
        };

        const expectedResult = {
            until: moment('2023-08-29T15:00:00Z').utc(true).format(),
        };

        const result = formatNestedTimeObject(timeArray);

        expect(result).toEqual(expectedResult);
    });

    it('should handle empty time object', () => {
        const timeArray = {};

        const result = formatNestedTimeObject(timeArray);

        expect(result).toEqual({});
    });

    it('should handle missing time object', () => {
        const result = formatNestedTimeObject();

        expect(result).toEqual({});
    });
});

describe('editConstraintSettings', () => {
  it('should set properties when value is true', () => {
    const setOnClose = jest.fn();
    const setOnSaveDisabled = jest.fn();
    const setCancelStatus = jest.fn();

    editConstraintSettings(true, setOnClose, setOnSaveDisabled, setCancelStatus);

    expect(setOnClose).toHaveBeenCalledWith(true);
    expect(setOnSaveDisabled).toHaveBeenCalledWith(false);
    expect(setCancelStatus).toHaveBeenCalledWith(false);
  });

  it('should set properties when value is false', () => {
    const setOnClose = jest.fn();
    const setOnSaveDisabled = jest.fn();
    const setCancelStatus = jest.fn();

    editConstraintSettings(false, setOnClose, setOnSaveDisabled, setCancelStatus);

    expect(setOnClose).toHaveBeenCalledWith(false);
    expect(setOnSaveDisabled).toHaveBeenCalledWith(true);
    expect(setCancelStatus).toHaveBeenCalledWith(false);
  });
});