import React, { Component} from 'react';
import { Link, Redirect } from 'react-router-dom';
import _ from 'lodash';
import { Toast } from 'primereact/toast';
import { appGrowl, setAppGrowl } from '../../layout/components/AppGrowl';
import { InputText } from 'primereact/inputtext';
import { InputTextarea } from 'primereact/inputtextarea';
import { Chips } from 'primereact/chips';
import { Dropdown } from 'primereact/dropdown';
import { CustomDialog } from '../../layout/components/CustomDialog';
import Jeditor from '../../components/JSONEditor/JEditor';
import { publish } from '../../App';

import TaskService from '../../services/task.service';
import AppLoader from "./../../layout/components/AppLoader";
import PageHeader from '../../layout/components/PageHeader';
import AuthUtil from '../../utils/auth.util';
import AccessDenied from '../../layout/components/AccessDenied';
import UIConstants from '../../utils/ui.constants';
import FormActionbar from '../../components/FormActionbar'; 

export class TaskEdit extends Component {
    templateOutput = {};        // id: selectedTemplateId, output: values enetered in the editor form
    
    constructor(props) {
        super(props);
        this.state = {
            showDialog: false,
            isDirty: false,
            task: {
                name: "",
                created_at: null,
                updated_at: null,
                tags:[],
                do_cancel: false
            },
            redirect: null,
            taskTemplates:[],
            validEditor: false,
            validForm: false,
            validDocument: false,
            errors: {},
            isLoading: true,
            userrole: {},
            taskId: '',
            fieldsToDisable: []
        };
        this.formRules = {
            name: {required: true, message: "Name can not be empty"},
            description: {required: true, message: "Description can not be empty"}
        };
        this.readOnlyProperties = ['duration', 'relative_start_time', 'relative_stop_time'];
        this.setEditorOutput = this.setEditorOutput.bind(this);
        this.setTaskParams = this.setTaskParams.bind(this);
        this.changeTaskTemplate = this.changeTaskTemplate.bind(this);
        this.setEditorFunction = this.setEditorFunction.bind(this);
        this.validateForm = this.validateForm.bind(this);
        this.saveTask = this.saveTask.bind(this);
        this.cancelEdit = this.cancelEdit.bind(this);
        this.checkIsDirty = this.checkIsDirty.bind(this);
        this.close = this.close.bind(this);
    }

    /**
     * This is the callback method to be passed to the JSON editor. 
     * JEditor will call this function when there is change in the editor.
     * @param {Object} jsonOutput 
     * @param {Array} errors 
     */
    async setEditorOutput(jsonOutput, errors) {
        // the json editor only yields the user-properties, not the properties like '$schema'
        // so, copy back the '$id' from the schema to make it comparable to the original specifications_doc
        jsonOutput['$schema'] = this.state.taskSchema['$id']

        if(!_.isEqual(this.templateOutput[this.state.task.specifications_template_id],jsonOutput)) {
            if(!this.state.isDirty) {
                publish('edit-dirty', true);
            }
            this.setState({isDirty: true});
        }
        this.templateOutput[this.state.task.specifications_template_id] = jsonOutput;
        await this.validateDocumentInBackend();
        if (errors.length === 0 && !this.state.validEditor && jsonOutput) {
            this.setState({validEditor: true, validForm: this.validateForm() });
        }   else if (errors.length > 0 && this.state.validEditor) {
            this.setState({validEditor: false, validForm: this.validateForm() });
        }
    }

    /**
     * Function called when there is change in the task parameters.
     * @param {String} key 
     * @param {*} value 
     */
    setTaskParams(key, value) {
        let task = this.state.task;
        const taskValue = this.state.task[key];
        task[key] = value;
        if  ( !this.state.isDirty && taskValue && !_.isEqual(taskValue, value) ) {
            if(!this.state.isDirty) {
                publish('edit-dirty', true);
            }
            this.setState({task: task, validForm: this.validateForm(), isDirty: true});
        }   else {
            this.setState({task: task, validForm: this.validateForm() });
        }

        this.validateDocumentInBackend();
    }

    /**
     * JEditor's function that to be called when parent wants to trigger change in the JSON Editor
     * @param {Function} editorFunction 
     */
    setEditorFunction(editorFunction) {
        this.setState({editorFunction: editorFunction});
    }

    /**
     * Function to be called when the template schema is changed
     * @param {Number} templateId 
     */
    changeTaskTemplate(templateId) {
		
        const template = _.find(this.state.taskTemplates, {'id': templateId});
        let task = this.state.task;
        task.specifications_template_id = templateId;
		
        task.specifications_template = template.url;
        if(!this.state.isDirty) {
            publish('edit-dirty', true);
        }
        this.setState({taskSchema: null});
        this.setState({task: task, taskSchema: template.ref_resolved_schema, isDirty: true});
        this.state.editorFunction();
    }

    /**
     * Function to validate the form excluding the JSON Editor values
     */
    validateForm() {
        let validForm = false;
        let errors = {};
        for (const fieldName in this.formRules) {
            const rule = this.formRules[fieldName];
            const fieldValue = this.state.task[fieldName];
            if (rule.required) {
                if (!fieldValue) {
                    errors[fieldName] = rule.message?rule.message:`${fieldName} is required`;
                }
            }
        }
        this.setState({errors: errors});
        if (this.state.task.name && this.state.task.description) {
            validForm = true;
        }
        return validForm;
    }

    /**
     * Function to validate the json task specifications_doc in the backend against the schema and consistency/sanity rules.
     */
    async validateDocumentInBackend() {
        if (this.state.task.name && this.state.task.specifications_template_id) {
            let specifications_doc = this.templateOutput[this.state.task.specifications_template_id];

            const validation_result = await TaskService.validateSpecificationsDoc(this.state.task, specifications_doc)
            if (validation_result) {
                this.setState({validDocument: validation_result['valid'],
                               validation_message: validation_result['message']});
            }
        }
    }

    /**
     * Function to call the servie and pass the values to save
     */
    async saveTask() {
        this.setState({isDirty: false});
        publish('edit-dirty', false);
        let task = this.state.task;
        task.specifications_doc = this.templateOutput[task.specifications_template_id];
        if (task.specifications_doc.station_configuration && task.specifications_doc.station_configuration.station_groups) {
            for( const stationGrops of task.specifications_doc.station_configuration.station_groups) {
                delete stationGrops['selected_station_list'];
            }
        }        
        // Remove read only properties from the object before sending to API
        this.readOnlyProperties.forEach(property => { delete task[property]});
        TaskService.updateTask("draft", task)
        .then( (taskDraft) => {
            if (taskDraft) {
                if (appGrowl === null) {
                    setAppGrowl(this.growl)
                }
                appGrowl.show({severity: 'success', summary: 'Success', detail: 'Tasks updated successfully.'});
                this.props.history.length>2 && this.props.history.go(-2);
                this.setState({redirect: '/task/view/draft/' + task.id});
            }
        });
    }

    /**
     * warn before cancel the page if any changes detected 
     */
    checkIsDirty() {
        if( this.state.isDirty ){
            this.setState({showDialog: true});
        } else {
            this.cancelEdit();
        }
    }
    
    close() {
        this.setState({showDialog: false});
    }

    cancelEdit() {
        publish('edit-dirty', false);
        this.props.history.length>1?this.state.isDirty?this.props.history.go(-2):this.props.history.go(-1):this.props.history.push(`/task/view/draft/${this.state.taskId}`);
        this.setState({showDialog: false});
        
    }

    async componentDidMount() {
        this.setState({ isLoading: true });
        TaskService.getTaskTemplates()
        .then((templates) => {            
            this.setState({taskTemplates: templates});
        });
        let taskId = this.props.match.params?this.props.match.params.id:null;
        taskId = taskId?taskId:(this.props.taskId?this.props.taskId:this.props.location.state.taskId);
        const permission = await AuthUtil.getUserPermissionByModuleId('task_draft',taskId);
        this.setState({userrole: permission, taskId:taskId})
        TaskService.getTaskDetails("draft", taskId)
        .then(async(task) => {
            if (task) {
                const taskTemplate = await TaskService.getTaskTemplate(task.specifications_template_id);
                const schedulingUnit = await TaskService.getSchedulingUnit("draft", task.scheduling_unit_draft_id, true);
                let targetTask = _.find(schedulingUnit[`task_drafts`], suTask => (
                    _.includes(UIConstants.TARGET_OBSERVATION_NAMES,suTask.specifications_template.name)
                ));                
                let fieldsToDisable = [];
                //To disable input fields in JSON editor
                task.specifications_doc.station_configuration?.SAPs.map((sap, index) => {
                        fieldsToDisable.push('SAPs.'+ index +'.name');
                        return sap;
                    })
                this.templateOutput[task.specifications_template_id] = task.specifications_doc;
                this.setState({schedulingUnit: schedulingUnit,targetTask: targetTask, task: task, taskSchema: taskTemplate.ref_resolved_schema, isLoading: false, fieldsToDisable: fieldsToDisable});
            }   else {
                this.setState({redirect: "/not-found"});
            }
        });
    }

    render() {
       
        if (this.state.redirect) {
            return <Redirect to={ {pathname: this.state.redirect, 
                                    state: {id: this.state.task.id}} }></Redirect>
        }
        const { isLoading } = this.state;
        
        const taskSchema = this.state.taskSchema;
        const template = _.find(this.state.taskTemplates, {'id': this.state.task.specifications_template_id});
        
        let jeditor = null;
        if (this.state.taskSchema) {         
		          jeditor = React.createElement(Jeditor, {title: "Specification", 
                                                        schema: taskSchema,
                                                        //initValue: this.state.templateOutput[this.state.task.specifications_template_id], 
                                                        initValue: this.templateOutput[this.state.task.specifications_template_id], 
                                                        fieldsToDisable: this.state.fieldsToDisable,    // Array of field names to disable eg. [SAPs.0.name, specification_doc.name,...]
                                                        callback: this.setEditorOutput,
                                                        parentFunction: this.setEditorFunction,
                                                        observationType: template.name,
                                                        targetObservation: this.state.targetTask,
                                                    });
        }
        
        return (
            <React.Fragment>
                {/*} <div className="p-grid">
                    <div className="p-col-10 p-lg-10 p-md-10">
                        <h2>Task - Edit</h2>
                    </div>
                    <div className="p-col-2 p-lg-2 p-md-2">
                        <Link to={{ pathname: `/task/view/draft/${this.state.task?this.state.task.id:''}`}} title="Close Edit"
                                style={{float: "right"}} >
                            <i className="fa fa-window-close" style={{marginTop: "10px"}}></i>
                        </Link>
                    </div>
                    </div> */}
                {this.state.userrole[this.state.taskId] && this.state.userrole[this.state.taskId].edit ?
                <>
                     <Toast ref={(el) => this.growl = el} />
                    <PageHeader location={this.props.location} title={'Task - Edit'} actions={[{icon: 'fa-window-close',
                    title:'Click to Close Task Edit Page',type: 'button',  actOn: 'click',props : { pathname:  `/task/view/draft/${this.state.task?this.state.task.id:''}`,callback: this.checkIsDirty}}]}/>
                    {isLoading ? <AppLoader/> :
                    <div>
                        <div className="p-fluid">
                        <div className="p-field p-grid">
                        <label htmlFor="taskName" className="col-lg-2 col-md-2 col-sm-12">Name <span style={{color:'red'}}>*</span></label>
                        <div className="col-lg-4 col-md-4 col-sm-12">
                            <InputText className={this.state.errors.name ?'input-error':''} id="taskName" type="text" value={this.state.task.name} disabled ={true}
                            data-testid="taskName" onChange={(e) => this.setTaskParams('name', e.target.value)}/>
                            <label className="error">
                                {this.state.errors.name ? this.state.errors.name : ""}
                            </label>
                        </div>
                        <label htmlFor="description" className="col-lg-2 col-md-2 col-sm-12">Description <span style={{color:'red'}}>*</span></label>
                        <div className="col-lg-4 col-md-4 col-sm-12">
                            <InputTextarea className={this.state.errors.description ?'input-error':''} rows={3} cols={30}
                            data-testid="taskDescription" value={this.state.task.description} onChange={(e) => this.setTaskParams('description', e.target.value)}/>
                            <label className="error">
                                {this.state.errors.description ? this.state.errors.description : ""}
                            </label>
                        </div>
                        </div>
                        <div className="p-field p-grid">
                        <label htmlFor="tags" className="col-lg-2 col-md-2 col-sm-12">Tags</label>
                        <div className="col-lg-4 col-md-4 col-sm-12">
                            <Chips value={this.state.task.tags?this.state.task.tags:[]} onChange={(e) => this.setTaskParams('tags', e.value)}></Chips>
                        </div>
                        {/* <label htmlFor="doCancel" className="col-lg-2 col-md-2 col-sm-12">Do Cancel</label>
                        <div className="col-lg-4 col-md-4 col-sm-12">
                            <Checkbox onChange={e => this.setTaskParams('do_cancel', e.checked)} checked={this.state.task.do_cancel}></Checkbox>
                        </div> */}
                        {this.state.schedulingUnit &&
                        <>
                            <label className="col-lg-2 col-md-2 col-sm-12">Scheduling Unit</label>
                            <Link className="col-lg-4 col-md-4 col-sm-12" to={ { pathname:`/schedulingunit/view/draft/${this.state.schedulingUnit.id}`, state: {id: this.state.schedulingUnit.id}}}>{this.state.schedulingUnit?this.state.schedulingUnit.name:''}</Link>
                        </>
                        }                        
                        </div>
                        <div className="p-field p-grid">
                            <label htmlFor="tags" className="col-lg-2 col-md-2 col-sm-12">Template</label>
                            <div className="col-lg-4 col-md-4 col-sm-12">
                                <Dropdown optionLabel="name" optionValue="id" 
                                value={this.state.task.specifications_template_id} 
                                options={this.state.taskTemplates} disabled
                                onChange={(e) => {this.changeTaskTemplate(e.value)}} 
                                placeholder="Select Task Template"/>
                            </div>
                            <label htmlFor="short_description" className="col-lg-2 col-md-2 col-sm-12">Short Description</label>
                            <div className="col-lg-4 col-md-4 col-sm-12">
                                <InputText className={this.state.errors.short_description ?'input-error':''} id="taskShortDescription" maxlength={32} type="text" value={this.state.task.short_description} 
                                  data-testid="taskShortDescription"   onChange={(e) => this.setTaskParams('short_description', e.target.value)}/>
                                <label className="info">Max 32 characters</label>
                            </div>
                        </div>                        
                        </div>
                        </div>
                    }
                    <div className="p-fluid">
                    <div className="p-grid"><div className="p-col-12" style={{width: "100%"}}>
                        {this.state.taskSchema?jeditor:""}
                    </div>
                    </div>
                    </div>
                
                    <div className="p-grid p-justify-start act-btn-grp" style={{marginBottom: "2em"}}>
                        <div className="col-lg-12 col-md-12 col-sm-12">
                            <label className={this.state.validDocument ? "info" : "error"}>
                                {this.state.validation_message ? this.state.validation_message : ""}
                            </label>
                        </div>
                    </div>
                    <div className="p-breadcrumb footer-long-div actions">
                        <FormActionbar onSubmit={this.saveTask} 
                        enableSaveBtn={!this.state.validEditor || !this.state.validForm || !this.state.validDocument} onCancel={this.checkIsDirty} />
                    </div>
                    <div className="p-grid" data-testid="confirm_dialog">
                        <CustomDialog type="confirmation" visible={this.state.showDialog} width="40vw"
                            header={'Edit Task'} message={'Do you want to discard your changes? Your changes to the Task will not be saved.'} 
                            content={''} onClose={this.close} 
                            actions={ [ {id:"yes", title: 'Discard', callback: this.cancelEdit, style:{"background": "#e91224","border": "1px solid #e91224"} },
                                    {id:"no", title: 'Cancel', callback: this.close} ]}                            
                                > 
                        </CustomDialog>
                    </div>
                </>: <AccessDenied/>}
            </React.Fragment>
        );
    }
}
