import React from 'react';
import { BrowserRouter as Router } from 'react-router-dom';
import { act } from "react-dom/test-utils";
import { render,  cleanup, fireEvent ,waitFor} from '@testing-library/react';
import '@testing-library/jest-dom';
import mockConsole from 'jest-mock-console'
import _ from 'lodash';
import { TaskEdit } from './edit';
import TaskService from '../../services/task.service';
import TaskServiceMock from '../../__mocks__/task.service.data';
import AuthService from '../../services/auth.service';
import AuthServiceMock from '../../__mocks__/auth.service.data';

let taskPermissionSpy, taskFilterDefSpy, subtaskTypesSpy, taskDetailsSpy, taskTemplateSpy, taskTemplatesSpy,
schedulingUnitSpy, updateTaskSpy, validatorSpy;
jest.setTimeout(240000);
let restoreConsole;
beforeEach(() => {
    restoreConsole = mockConsole();
    setMockSpy();
});

afterEach(() => {
    restoreConsole();
    // cleanup on exiting
    clearMockSpy();
    cleanup();
});

/**
 * To set mock spy for Services that have API calls to the back end to fetch data
 */
const setMockSpy = (() => {
    taskPermissionSpy = jest.spyOn(AuthService, 'getAccessControlMethod');
    taskPermissionSpy.mockImplementation((url) => {
        if (_.startsWith(url, "task_draft")) {
            return Promise.resolve(AuthServiceMock.task["1"]);
        }   else if (_.startsWith(url, "task_blueprint")) {
            return Promise.resolve(AuthServiceMock.task_blueprint["9"]);
        }   else if (_.startsWith(url, "subtask")) {
            return Promise.resolve(AuthServiceMock.subtask["2000011"]);
        }
    });
    taskFilterDefSpy = jest.spyOn(TaskService, 'getTaskFilterDefinition');
    taskFilterDefSpy.mockImplementation(() => { return Promise.resolve(TaskServiceMock.getTaskFilterDefinition)});
    subtaskTypesSpy = jest.spyOn(TaskService, 'getSubtaskType');
    subtaskTypesSpy.mockImplementation(() => {return Promise.resolve(TaskServiceMock.subTaskTypes)});
    taskDetailsSpy = jest.spyOn(TaskService, 'getTaskDetails');
    taskDetailsSpy.mockImplementation((taskType, taskId, fetchSubtask) => { 
        if (taskType === "draft") {
            const taskDraftslist = TaskServiceMock.taskDrafts;
            return Promise.resolve(_.find(taskDraftslist, {"id": parseInt(taskId)}));
        }   else {
            return Promise.resolve(_.find(TaskServiceMock.taskBlueprints, {"id": parseInt(taskId)}));
        }
    });
    updateTaskSpy = jest.spyOn(TaskService, 'updateTask');
    updateTaskSpy.mockImplementation((type, task) => {
        const taskDraftslist = TaskServiceMock.taskDrafts;
        return Promise.resolve(_.find(taskDraftslist, {id: task.id}));
    });

    taskTemplateSpy = jest.spyOn(TaskService, 'getTaskTemplate');
    taskTemplateSpy.mockImplementation((templateId) => {
        return Promise.resolve(_.find(TaskServiceMock.getTaskTemplates(), {"id":templateId}));
    });

    taskTemplatesSpy = jest.spyOn(TaskService, 'getTaskTemplates');
    taskTemplatesSpy.mockImplementation(() => {
        return Promise.resolve(TaskServiceMock.getTaskTemplates());;
    });

    schedulingUnitSpy = jest.spyOn(TaskService, "getSchedulingUnit");
    schedulingUnitSpy.mockImplementation((type, id, loadTasks) => {
        if (type === "draft") {
            return Promise.resolve(_.find(TaskServiceMock.schedulingUnitDrafts, {"id": id}));
        }   else {
            return Promise.resolve(_.find(TaskServiceMock.schedulingUnitBlueprints, {"id": id}));
        }
    });
    validatorSpy = jest.spyOn(TaskService, 'validateSpecificationsDoc');
    validatorSpy.mockImplementation(() =>{
        return Promise.resolve({valid: true, message: "validation_result['message']"})
    })
});

const clearMockSpy = (() => {
    taskPermissionSpy.mockRestore();
    taskFilterDefSpy.mockRestore();
    subtaskTypesSpy.mockRestore();
    taskDetailsSpy.mockRestore();
    taskTemplateSpy.mockRestore(); 
    taskTemplatesSpy.mockRestore(); 
    schedulingUnitSpy.mockRestore();
    updateTaskSpy.mockRestore();
    validatorSpy.mockRestore();
}); 

it("renders Task details Edit page", async () => { 
    let content;
    await act(async () => {
        content = render(<Router key="router"><TaskEdit match={{params:{id: "9"}}} history={{}} location="/task/edit/draft/9" /></Router>);
    });
    expect(content.getByText("Task - Edit")).toBeInTheDocument();
});

it("test Combined Observation Task Draft view of IM LBA Survey - 3 Beams edit", async () => {
    let content;
    await act(async () => {
        content =  render(<Router><TaskEdit match={{params:{id: "9"}}} history={{}} location="/task/edit/draft/9" /></Router>);
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.getByText("Task - Edit")).toBeInTheDocument();    
    // checking Task details and specification 
    expect(content.queryByTestId('taskDescription').value).toEqual('Combined parallel Calibrator & Target Observation for LBA with 3 target beams');
    expect(content.queryByTestId('taskShortDescription').value).toEqual('OOO.O Paaa+01 Paaa+02 Paaa+03');
    content.queryByTestId('taskShortDescription').value = 'Testing';
    expect(content.getByText("IM LBA Survey - 3 Beams")).toBeInTheDocument()
    expect(content.getAllByText("parallel calibrator target observation").length).toBe(2);
    await waitFor(()=>expect(content.getByText("calibrator")).toBeInTheDocument());
    expect(content.getByText("calibrator")).toBeInTheDocument()
    await waitFor(()=>expect(content.getByText("Specification")).toBeInTheDocument());
    expect(content.getByText("Specification")).toBeInTheDocument();
    await waitFor(()=>expect(content.getByText("Auto-select")).toBeInTheDocument());
    expect(content.getByText("Auto-select")).toBeInTheDocument();  
    expect(content.getByLabelText("specification[calibrator][name]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[calibrator][pointing][angle1]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[calibrator][pointing][angle2]")).toBeInTheDocument();
    expect(content.getAllByText("Reference frame").length).toBe(5);
    expect(content.getByLabelText("specification[calibrator][pointing][target]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[duration]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[correlator][integration_time]")).toBeInTheDocument();
    expect(content.getByText("Storage cluster")).toBeInTheDocument();
    expect(content.getByText("Topocentric Frequency Correction")).toBeInTheDocument();
    expect(content.getByText("Channels/subband")).toBeInTheDocument();
    expect(content.getByLabelText("specification[correlator][frequency_resolution]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[correlator][time_resolution]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][0][digital_pointing][angle1]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][0][digital_pointing][angle2]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][0][digital_pointing][target]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][0][name]")).toBeInTheDocument();

    expect(content.getByText("Antenna set")).toBeInTheDocument();
    expect(content.getByText("Band-pass filter")).toBeInTheDocument();
    expect(content.getByText("Station groups")).toBeInTheDocument();
    expect(content.getByText("Station group 1")).toBeInTheDocument();
    expect(content.getByText("Station group 2")).toBeInTheDocument();
    expect(content.getByText("Station group 3")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][tile_beam][angle1]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][tile_beam][angle2]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][tile_beam][target]")).toBeInTheDocument();
    expect(content.getByText("Inspection Plots")).toBeInTheDocument();
    expect(content.getByText("autocorrelation")).toBeInTheDocument();
    expect(content.getByText("crosscorrelation")).toBeInTheDocument();
    expect(content.getByText("Do/Don't create plots from the QA file from the observation")).toBeInTheDocument();



    await waitFor(()=>expect(content.getByLabelText("specification[station_configuration][SAPs][0][subbands]")).toBeInTheDocument());
    expect(content.getByLabelText("specification[station_configuration][SAPs][0][subbands]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][0][frequency]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][1][digital_pointing][angle1]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][1][digital_pointing][angle2]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][1][digital_pointing][target]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][1][name]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][1][subbands]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][1][frequency]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][2][digital_pointing][angle1]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][2][digital_pointing][angle2]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][2][digital_pointing][target]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][2][name]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][2][subbands]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][2][frequency]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[calibrator][name]")).toBeInTheDocument();
  
    // check for save button enable/disable
    expect(content.queryByTestId('save-btn')).not.toBeDisabled();
    expect(content.queryByTestId('taskName').value).toBe('Combined Observation');
    const nameInputEl = content.queryByTestId('taskName');
    await act( async() => {
        fireEvent.change(nameInputEl, { target: { value: '' } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(nameInputEl.value).toBe('');
    expect(content.queryByTestId('save-btn')).toBeDisabled();
    const angel1El = content.getByLabelText("specification[station_configuration][SAPs][0][digital_pointing][angle1]");
    expect(angel1El.value).not.toBe('');
    const angle1Value = angel1El.value;
    await act( async() => {
        fireEvent.change(angel1El, { target: { value: '' } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.queryByTestId('save-btn')).toBeDisabled();
    
    await act( async() => {
        fireEvent.change(angel1El, { target: { value: '72h31m49.09s' } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.getByText("Not a valid input. Mimimum: 00:00:00.0000hours or 0 or 00:00:00.0000degrees, Maximum:23:59:59.9999hours or 6.2831 or 359:59:59.9999degrees.")).toBeVisible();
    expect(content.queryByTestId('save-btn')).toBeDisabled();

    await act( async() => {
        fireEvent.change(nameInputEl, { target: { value: 'Combined Observation' }});
        fireEvent.change(angel1El, { target: { value: angle1Value } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.getByText("Not a valid input. Mimimum: 00:00:00.0000hours or 0 or 00:00:00.0000degrees, Maximum:23:59:59.9999hours or 6.2831 or 359:59:59.9999degrees.")).not.toBeVisible();
    expect(content.queryByTestId('save-btn')).not.toBeDisabled();

    await act( async() => {
        fireEvent.change(nameInputEl, { target: { value: 'Combined Observation' }});
        fireEvent.change(angel1El, { target: { value: angle1Value } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.queryByTestId('save-btn')).not.toBeDisabled();
    // Save task
    await act(async () => {
        fireEvent.click(content.queryByTestId('save-btn'));
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(updateTaskSpy).toHaveBeenCalled();
}); 


it("test Combined Observation Task Draft view of IM LBA Survey - 3 Beams edit" , async () => {
    let content;
    await act(async () => {
        content = render(<Router><TaskEdit match={{params:{id: "1"}}} history={{}} location="/task/edit/draft/1" /></Router>);
        });
    expect(content.getByText("Task - Edit")).toBeInTheDocument();

    // checking Task details and specification 
    expect(content.queryByTestId('taskName').value).toEqual('Combined Observation');
    expect(content.queryByTestId('taskDescription').value).toEqual('Combined parallel Calibrator & Target Observation for LBA with 5 target beams');
    expect(content.queryByTestId('taskShortDescription').value).toEqual('oXXX Paaa+01 3Cabc');     
    expect(content.getAllByText("parallel calibrator target observation").length).toBe(2);
    await waitFor(()=>expect(content.getByText("Specification")).toBeInTheDocument());
    expect(content.getByText("Specification")).toBeInTheDocument();
    expect(content.getByText("Auto-select")).toBeInTheDocument();
    expect(content.getByLabelText("specification[calibrator][name]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[calibrator][pointing][angle1]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[calibrator][pointing][angle2]")).toBeInTheDocument();
    expect(content.getAllByText("Reference frame").length).toBe(7);
    expect(content.getByLabelText("specification[calibrator][pointing][target]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[duration]")).toBeInTheDocument();

    expect(content.getByLabelText("specification[correlator][integration_time]")).toBeInTheDocument();
    expect(content.getByText("Storage cluster")).toBeInTheDocument();
    expect(content.getByText("Topocentric Frequency Correction")).toBeInTheDocument();
    expect(content.getByText("Channels/subband")).toBeInTheDocument();
    expect(content.getByLabelText("specification[correlator][frequency_resolution]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[correlator][time_resolution]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][0][digital_pointing][angle1]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][0][digital_pointing][angle2]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][0][digital_pointing][target]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][0][name]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][0][subbands]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][0][frequency]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][1][digital_pointing][angle1]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][1][digital_pointing][angle2]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][1][digital_pointing][target]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][1][name]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][1][subbands]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][1][frequency]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][2][digital_pointing][angle1]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][2][digital_pointing][angle2]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][2][digital_pointing][target]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][2][name]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][2][subbands]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][2][frequency]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][3][digital_pointing][angle1]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][3][digital_pointing][angle2]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][3][digital_pointing][target]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][3][name]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][3][subbands]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][3][frequency]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][4][digital_pointing][angle1]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][4][digital_pointing][angle2]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][4][digital_pointing][target]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][4][name]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][4][subbands]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][4][frequency]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[calibrator][name]")).toBeInTheDocument();
    expect(content.getByText("Antenna set")).toBeInTheDocument();
    expect(content.getByText("Band-pass filter")).toBeInTheDocument();
    expect(content.getByText("Station groups")).toBeInTheDocument();
    expect(content.getByText("Station group 1")).toBeInTheDocument();
    expect(content.getByText("Station group 2")).toBeInTheDocument();
    expect(content.getByText("Station group 3")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][tile_beam][angle1]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][tile_beam][angle2]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][tile_beam][target]")).toBeInTheDocument();
    expect(content.getByText("Inspection Plots")).toBeInTheDocument();
    expect(content.getByText("autocorrelation")).toBeInTheDocument();
    expect(content.getByText("crosscorrelation")).toBeInTheDocument();
    expect(content.getByText("Do/Don't create plots from the QA file from the observation")).toBeInTheDocument();
    
    // check for save button enable/disable
    expect(content.queryByTestId('save-btn')).not.toBeDisabled();
    expect(content.queryByTestId('taskName').value).toBe('Combined Observation');
    const nameInputEl = content.queryByTestId('taskName');
    await act( async() => {
        fireEvent.change(nameInputEl, { target: { value: '' } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(nameInputEl.value).toBe('');
    expect(content.queryByTestId('save-btn')).toBeDisabled();
    const angel1El = content.getByLabelText("specification[station_configuration][SAPs][0][digital_pointing][angle1]");
    expect(angel1El.value).not.toBe('');
    const angle1Value = angel1El.value;
    await act( async() => {
        fireEvent.change(angel1El, { target: { value: '' } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.queryByTestId('save-btn')).toBeDisabled();

    await act( async() => {
        fireEvent.change(angel1El, { target: { value: '72h31m49.09s' } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.getByText("Not a valid input. Mimimum: 00:00:00.0000hours or 0 or 00:00:00.0000degrees, Maximum:23:59:59.9999hours or 6.2831 or 359:59:59.9999degrees.")).toBeVisible();
    expect(content.queryByTestId('save-btn')).toBeDisabled();

    await act( async() => {
        fireEvent.change(nameInputEl, { target: { value: 'Combined Observation' }});
        fireEvent.change(angel1El, { target: { value: angle1Value } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.getByText("Not a valid input. Mimimum: 00:00:00.0000hours or 0 or 00:00:00.0000degrees, Maximum:23:59:59.9999hours or 6.2831 or 359:59:59.9999degrees.")).not.toBeVisible();
    expect(content.queryByTestId('save-btn')).not.toBeDisabled();
    //Save task
    await act(async () => {
        fireEvent.click(content.queryByTestId('save-btn'));
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(updateTaskSpy).toHaveBeenCalled();
});


it("test Calibrator Pipeline Task Draft view of IM LBA LoDSS - 5 Beams edit", async () => {
    let content;
    await act(async () => {
        content = render(<Router><TaskEdit match={{params:{id: "2"}}} history={{}} location="/task/edit/draft/2" /></Router>);
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.getByText("Task - Edit")).toBeInTheDocument();
    // checking Task details and specification 
    expect(content.queryByTestId('taskName').value).toEqual('Calibrator Pipeline');
    expect(content.queryByTestId('taskDescription').value).toEqual('Preprocessing Pipeline for Calibrator Observation');
    expect(content.queryByTestId('taskShortDescription').value).toEqual('3Cabc/1.0/CP');     
    expect(content.getAllByText("preprocessing pipeline").length).toBe(2);
    await waitFor(()=>expect(content.getByText("Specification")).toBeInTheDocument());
    expect(content.getByText("Specification")).toBeInTheDocument();
    expect(content.getByText("Task - Edit")).toBeInTheDocument();
    expect(content.getByLabelText("specification[average][time_steps]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[average][frequency_steps]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[average][time_resolution]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[average][frequency_resolution]")).toBeInTheDocument();
    expect(content.getByText("CasA")).toBeInTheDocument();
    expect(content.getByText("CygA")).toBeInTheDocument();
    expect(content.getByText("HerA")).toBeInTheDocument();
    expect(content.getByText("HydraA")).toBeInTheDocument();
    expect(content.getByText("TauA")).toBeInTheDocument();
    expect(content.getByText("VirA")).toBeInTheDocument();
    expect(content.getByLabelText("specification[demix][time_steps]")).toBeInTheDocument();
    expect(content.getByText("Ignore target")).toBeInTheDocument();
    expect(content.getByLabelText("specification[demix][frequency_steps]")).toBeInTheDocument();
    expect(content.getByText("RFI flagging strategy")).toBeInTheDocument();
    expect(content.getByText("Flag outer channels")).toBeInTheDocument();
    expect(content.getByText("Flag auto correlations")).toBeInTheDocument();
    expect(content.getByText("Storage Manager")).toBeInTheDocument();
    expect(content.getByLabelText("specification[cluster_resources][cores_per_task]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[cluster_resources][parallel_tasks]")).toBeInTheDocument();
    expect(content.getByText("Cluster")).toBeInTheDocument();
    expect(content.getByText("Partition")).toBeInTheDocument();

    // check for save button enable/disable
    expect(content.queryByTestId('save-btn')).not.toBeDisabled();
    expect(content.queryByTestId('taskName').value).toBe('Calibrator Pipeline');
    const nameInputEl = content.queryByTestId('taskName');
    await act( async() => {
        fireEvent.change(nameInputEl, { target: { value: '' } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(nameInputEl.value).toBe('');
    expect(content.queryByTestId('save-btn')).toBeDisabled();
    const averageTSEl = content.getByLabelText("specification[average][time_steps]");
    expect(averageTSEl.value).not.toBe('');
    const averageTSValue = averageTSEl.value;
    await act( async() => {
        fireEvent.change(averageTSEl, { target: { value: '' } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.queryByTestId('save-btn')).toBeDisabled();

    await act( async() => {
        fireEvent.change(averageTSEl, { target: { value: 0 } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.getByText("Value must be at least 1.")).toBeVisible();
    expect(content.queryByTestId('save-btn')).toBeDisabled();

    await act( async() => {
        fireEvent.change(nameInputEl, { target: { value: 'Calibrator Pipeline' }});
        fireEvent.change(averageTSEl, { target: { value: averageTSValue } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.getByText("Value must be at least 1.")).not.toBeVisible();
    expect(content.queryByTestId('save-btn')).not.toBeDisabled();

    //Save task
    await act(async () => {
        fireEvent.click(content.queryByTestId('save-btn'));
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(updateTaskSpy).toHaveBeenCalled();
});  

it("test Target Pipeline Task Draft view of IM LBA LoDSS - 5 Beams edit", async () => {
    let content;
    await act(async () => {
        content = render(<Router><TaskEdit match={{params:{id: "7", type:"draft"}}} history={{}} location="/task/edit/draft/7" /></Router>);
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.getByText("Task - Edit")).toBeInTheDocument(); 
    // checking Task details and specification
    expect(content.queryByTestId('taskName').value).toEqual('Pipeline target1');
    expect(content.queryByTestId('taskDescription').value).toEqual('Preprocessing Pipeline for Target Observation target1');
    expect(content.queryByTestId('taskShortDescription').value).toEqual('Paaa+01/1.0/PP');     
    expect(content.getAllByText("preprocessing pipeline").length).toBe(2); 
    await waitFor(()=>expect(content.getByText("Specification")).toBeInTheDocument());
    expect(content.getByText("Specification")).toBeInTheDocument();
    expect(content.getByLabelText("specification[average][time_steps]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[average][frequency_steps]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[average][time_resolution]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[average][frequency_resolution]")).toBeInTheDocument();
    expect(content.getByText("CasA")).toBeInTheDocument();
    expect(content.getByText("CygA")).toBeInTheDocument();
    expect(content.getByText("HerA")).toBeInTheDocument();
    expect(content.getByText("HydraA")).toBeInTheDocument();
    expect(content.getByText("TauA")).toBeInTheDocument();
    expect(content.getByText("VirA")).toBeInTheDocument();
    expect(content.getByLabelText("specification[demix][time_steps]")).toBeInTheDocument();
    expect(content.getByText("Ignore target")).toBeInTheDocument();
    expect(content.getByLabelText("specification[demix][frequency_steps]")).toBeInTheDocument();
    expect(content.getByText("RFI flagging strategy")).toBeInTheDocument();
    expect(content.getByText("Flag outer channels")).toBeInTheDocument();
    expect(content.getByText("Flag auto correlations")).toBeInTheDocument();
    expect(content.getByText("Storage Manager")).toBeInTheDocument();
    expect(content.getByLabelText("specification[cluster_resources][cores_per_task]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[cluster_resources][parallel_tasks]")).toBeInTheDocument();
    expect(content.getByText("Cluster")).toBeInTheDocument();
    expect(content.getByText("Partition")).toBeInTheDocument();

    // premery check for save button enable/disable
    expect(content.queryByTestId('save-btn')).not.toBeDisabled();
    expect(content.queryByTestId('taskName').value).toBe('Pipeline target1');
    const nameInputEl = content.queryByTestId('taskName');
    await act( async() => {
        fireEvent.change(nameInputEl, { target: { value: '' } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(nameInputEl.value).toBe('');
    expect(content.queryByTestId('save-btn')).toBeDisabled();
    const averageTSEl = content.getByLabelText("specification[average][time_steps]");
    expect(averageTSEl.value).not.toBe('');
    const averageTSValue = averageTSEl.value;
    await act( async() => {
        fireEvent.change(averageTSEl, { target: { value: '' } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.queryByTestId('save-btn')).toBeDisabled();

    await act( async() => {
        fireEvent.change(averageTSEl, { target: { value: 0 } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.getByText("Value must be at least 1.")).toBeVisible();
    expect(content.queryByTestId('save-btn')).toBeDisabled();

    await act( async() => {
        fireEvent.change(nameInputEl, { target: { value: 'Pipeline target1' }});
        fireEvent.change(averageTSEl, { target: { value: averageTSValue } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.getByText("Value must be at least 1.")).not.toBeVisible();
    expect(content.queryByTestId('save-btn')).not.toBeDisabled();

    //Save task
    await act(async () => {
        fireEvent.click(content.queryByTestId('save-btn'));
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(updateTaskSpy).toHaveBeenCalled();
}); 

//1741

it("test IM BF Cal&Tgt Task Draft edit", async () => {
    let content;
    await act(async () => {
        content = render(<Router><TaskEdit match={{params:{id: "68"}}} history={{}} location="/task/edit/draft/68" /></Router>);
        await new Promise((r) => setTimeout(r, 1000));
    });
    expect(content.getByText("Task - Edit")).toBeInTheDocument();
    // checking Task details and specification 
    expect(content.queryByTestId('taskName').value).toEqual('Observation');
    expect(content.queryByTestId('taskDescription').value).toEqual('IM BF Cal&Tgt');
    expect(content.queryByTestId('taskShortDescription').value).toEqual('');
    expect(content.getAllByText("parallel calibrator target and beamforming observation").length).toBeGreaterThan(0);
    await waitFor(()=>expect(content.getByText("Specification")).toBeInTheDocument());
    expect(content.getByText("Specification")).toBeInTheDocument();
    expect(content.getByText("Auto-select")).toBeInTheDocument();
    expect(content.getByLabelText("specification[calibrator][name]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[calibrator][pointing][angle1]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[calibrator][pointing][angle2]")).toBeInTheDocument();
    expect(content.getAllByText("Reference frame").length).toBe(3);
    expect(content.getByLabelText("specification[calibrator][pointing][target]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[duration]")).toBeInTheDocument();
    
    expect(content.getByText("Beamformer")).toBeInTheDocument();
    expect(content.getByText("Pipelines")).toBeInTheDocument();
    
    expect(content.getByText("Coherent Tied-Array Beams")).toBeInTheDocument();
    expect(content.getAllByText("SAPs").length).toBe(3);
    expect(content.getAllByText("Output quantisation settings").length).toBe(3);
    expect(content.getAllByText("Bits/sample").length).toBe(3);
    expect(content.getAllByText("Output quantisation into integers").length).toBe(3);
    expect(content.getByLabelText("specification[beamformer][pipelines][0][coherent][settings][quantisation][scale_max]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[beamformer][pipelines][0][coherent][settings][quantisation][scale_min]")).toBeInTheDocument();
    expect(content.getAllByText("Stokes").length).toBe(3);
    expect(content.getByLabelText("specification[beamformer][pipelines][0][coherent][settings][subbands_per_file]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[beamformer][pipelines][0][coherent][settings][time_integration_factor]")).toBeInTheDocument();
    expect(content.getAllByText("Channels/subband").length).toBe(4);
    expect(content.getByLabelText("specification[beamformer][pipelines][0][coherent][settings][frequency_resolution]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[beamformer][pipelines][0][coherent][settings][time_resolution]")).toBeInTheDocument();
    
    expect(content.getByText("Enable Fly's Eye")).toBeInTheDocument();
    expect(content.getByLabelText("specification[beamformer][pipelines][0][flys eye][settings][quantisation][scale_max]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[beamformer][pipelines][0][flys eye][settings][quantisation][scale_min]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[beamformer][pipelines][0][flys eye][settings][subbands_per_file]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[beamformer][pipelines][0][flys eye][settings][time_integration_factor]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[beamformer][pipelines][0][flys eye][settings][frequency_resolution]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[beamformer][pipelines][0][flys eye][settings][time_resolution]")).toBeInTheDocument();
    
    expect(content.getByText("Incoherent Tied-Array Beams")).toBeInTheDocument();
    expect(content.getByLabelText("specification[beamformer][pipelines][0][incoherent][settings][quantisation][scale_max]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[beamformer][pipelines][0][incoherent][settings][quantisation][scale_min]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[beamformer][pipelines][0][incoherent][settings][subbands_per_file]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[beamformer][pipelines][0][incoherent][settings][time_integration_factor]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[beamformer][pipelines][0][incoherent][settings][frequency_resolution]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[beamformer][pipelines][0][incoherent][settings][time_resolution]")).toBeInTheDocument();
    
    expect(content.getByLabelText("specification[beamformer][pipelines][0][name]")).toBeInTheDocument();
    expect(content.getAllByText("Station groups").length).toBe(2);
    expect(content.getAllByRole("selected_stations").length).toBe(1);
    expect(content.getAllByRole("selected_stations")[0].children.length).toBe(21);
    expect(content.getByLabelText("specification[beamformer][pipelines][0][station_groups][0][max_nr_missing]")).toBeInTheDocument();
    expect(content.getByText("Use PPF when beamforming")).toBeInTheDocument();
    
    expect(content.getByLabelText("specification[correlator][integration_time]")).toBeInTheDocument();
    expect(content.getByText("Storage cluster")).toBeInTheDocument();
    expect(content.getByText("Topocentric Frequency Correction")).toBeInTheDocument();
    expect(content.getByLabelText("specification[correlator][frequency_resolution]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[correlator][time_resolution]")).toBeInTheDocument();
    
    expect(content.getByLabelText("specification[station_configuration][SAPs][0][digital_pointing][angle1]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][0][digital_pointing][angle2]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][0][digital_pointing][target]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][0][name]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][0][subbands]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][0][frequency]")).toBeInTheDocument();
    
    expect(content.getByText("Antenna set")).toBeInTheDocument();
    expect(content.getByText("Band-pass filter")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][station_groups][0][max_nr_missing]")).toBeInTheDocument();
    
    expect(content.getByLabelText("specification[station_configuration][tile_beam][angle1]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][tile_beam][angle2]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][tile_beam][target]")).toBeInTheDocument();

    expect(content.getByText("File Conversion")).toBeInTheDocument();
    expect(content.getByText("Do/Don't create a QA file for the observation")).toBeInTheDocument();
    expect(content.getByLabelText("specification[QA][file_conversion][nr_of_subbands]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[QA][file_conversion][nr_of_timestamps]")).toBeInTheDocument();

    expect(content.getByText("Inspection Plots")).toBeInTheDocument();
    expect(content.getByText("autocorrelation")).toBeInTheDocument();
    expect(content.getByText("crosscorrelation")).toBeInTheDocument();
    expect(content.getByText("Do/Don't create plots from the QA file from the observation")).toBeInTheDocument();

    // premery check for save button enable/disable
    expect(content.queryByTestId('save-btn')).not.toBeDisabled();
    expect(content.queryByTestId('taskName').value).not.toBeNull();
    const nameInputEl = content.queryByTestId('taskName');
    const nameInputElValue = nameInputEl.value;
    await act( async() => {
        fireEvent.change(nameInputEl, { target: { value: '' } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(nameInputEl.value).toBe('');
    expect(content.queryByTestId('save-btn')).toBeDisabled();

    const taskDescInputEl = content.queryByTestId('taskDescription');
    const taskDescInputEllValue = taskDescInputEl.value;
    await act( async() => {
        fireEvent.change(taskDescInputEl, { target: { value: '' } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(taskDescInputEl.value).toBe('');
    expect(content.queryByTestId('save-btn')).toBeDisabled();
    await act( async() => {
        fireEvent.change(taskDescInputEl, { target: { value: taskDescInputEllValue } });
        await new Promise((r) => setTimeout(r, 500));
    });

    const angle1El = content.getByLabelText("specification[calibrator][pointing][angle1]");
    expect(angle1El.value).not.toBe('');
    const averageTSValue = angle1El.value;
    await act( async() => {
        fireEvent.change(angle1El, { target: { value: '' } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.queryByTestId('save-btn')).toBeDisabled();

    await act( async() => {
        fireEvent.change(angle1El, { target: { value: '82h31m49.09s' } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.getByText("Not a valid input. Mimimum: 00:00:00.0000hours or 0 or 00:00:00.0000degrees, Maximum:23:59:59.9999hours or 6.2831 or 359:59:59.9999degrees.")).toBeVisible();
    expect(content.queryByTestId('save-btn')).toBeDisabled();

    await act( async() => {
        fireEvent.change(nameInputEl, { target: { value: nameInputElValue }});
        fireEvent.change(angle1El, { target: { value: averageTSValue } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.getByText("Not a valid input. Mimimum: 00:00:00.0000hours or 0 or 00:00:00.0000degrees, Maximum:23:59:59.9999hours or 6.2831 or 359:59:59.9999degrees.")).not.toBeVisible();
    expect(content.queryByTestId('save-btn')).not.toBeDisabled();
 
    const angle2El = content.getByLabelText("specification[station_configuration][SAPs][0][digital_pointing][angle2]");
    expect(angle2El.value).not.toBe('');
    const angle2ElValue = angle2El.value;
    await act( async() => {
        fireEvent.change(angle2El, { target: { value: '' } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.queryByTestId('save-btn')).toBeDisabled();

    await act( async() => {
        fireEvent.change(angle2El, { target: { value: '91d15m50.8s' } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.getByText("Not a valid input. Mimimum: -90:00:00.0000degrees 0r -1.57079632679489661923, Maximum:90:00:00.0000degrees or 1.57079632679489661923.")).toBeVisible();
    expect(content.queryByTestId('save-btn')).toBeDisabled();

    await act( async() => {
        fireEvent.change(angle2El, { target: { value: angle2ElValue } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.getByText("Not a valid input. Mimimum: -90:00:00.0000degrees 0r -1.57079632679489661923, Maximum:90:00:00.0000degrees or 1.57079632679489661923.")).not.toBeVisible();
    expect(content.queryByTestId('save-btn')).not.toBeDisabled();
 
    //Save task
     await act(async () => {
        fireEvent.click(content.queryByTestId('save-btn'));
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(updateTaskSpy).toHaveBeenCalled();
});

it("test IM BF Cal&Tgt Task Draft edit", async () => {
    let content;
    await act(async () => {
        content = render(<Router><TaskEdit match={{params:{id: "53"}}} history={{}} location="/task/edit/draft/53" /></Router>);
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.getByText("Task - Edit")).toBeInTheDocument();
    // checking Task details and specification 
    expect(content.queryByTestId('taskName').value).toEqual('Observation');
    expect(content.queryByTestId('taskDescription').value).toEqual('A simple short test beamforming observation');
    expect(content.queryByTestId('taskShortDescription').value).toEqual('');

    expect(content.getByText("Simple Beamforming Observation")).toBeInTheDocument();
    await waitFor(()=>expect(content.getByText("Specification")).toBeInTheDocument());
    expect(content.getByText("Specification")).toBeInTheDocument();
    expect(content.getByLabelText("specification[duration]")).toBeInTheDocument();
    
    expect(content.getByText("Beamformer")).toBeInTheDocument();
    expect(content.getByText("Pipelines")).toBeInTheDocument();
    
    expect(content.getByText("Coherent Tied-Array Beams")).toBeInTheDocument();
    expect(content.getAllByText("SAPs").length).toBe(3);
    expect(content.getByLabelText("specification[beamformer][pipelines][0][coherent][SAPs][0][name]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[beamformer][pipelines][0][coherent][SAPs][0][subbands][list]")).toBeInTheDocument();
    expect(content.getAllByText("Method").length).toBe(1);
    expect(content.getByLabelText("specification[beamformer][pipelines][0][coherent][SAPs][0][tab_rings][count]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[beamformer][pipelines][0][coherent][SAPs][0][tab_rings][width]")).toBeInTheDocument();
    expect(content.getByText("Allow after previous")).toBeInTheDocument();
    expect(content.getByLabelText("specification[beamformer][pipelines][0][coherent][SAPs][0][tabs][0][duration]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[beamformer][pipelines][0][coherent][SAPs][0][tabs][0][pointing][angle1]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[beamformer][pipelines][0][coherent][SAPs][0][tabs][0][pointing][angle2]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[beamformer][pipelines][0][coherent][SAPs][0][tabs][0][pointing][target]")).toBeInTheDocument();
    expect(content.getAllByText("Reference frame").length).toBe(3);
    expect(content.getByText("Relative to SAP")).toBeInTheDocument();
    expect(content.getAllByText("Output quantisation settings").length).toBe(3);
    expect(content.getAllByText("Bits/sample").length).toBe(3);
    expect(content.getAllByText("Output quantisation into integers").length).toBe(3);
    expect(content.getByLabelText("specification[beamformer][pipelines][0][coherent][settings][quantisation][scale_max]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[beamformer][pipelines][0][coherent][settings][quantisation][scale_min]")).toBeInTheDocument();
    expect(content.getAllByText("Stokes").length).toBe(3);
    
    expect(content.getByLabelText("specification[beamformer][pipelines][0][coherent][settings][subbands_per_file]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[beamformer][pipelines][0][coherent][settings][time_integration_factor]")).toBeInTheDocument();
    expect(content.getAllByText("Channels/subband").length).toBe(3);
    expect(content.getByLabelText("specification[beamformer][pipelines][0][coherent][settings][frequency_resolution]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[beamformer][pipelines][0][coherent][settings][time_resolution]")).toBeInTheDocument();
    
    expect(content.getByText("Enable Fly's Eye")).toBeInTheDocument();
    expect(content.getByLabelText("specification[beamformer][pipelines][0][flys eye][settings][quantisation][scale_max]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[beamformer][pipelines][0][flys eye][settings][quantisation][scale_min]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[beamformer][pipelines][0][flys eye][settings][subbands_per_file]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[beamformer][pipelines][0][flys eye][settings][time_integration_factor]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[beamformer][pipelines][0][flys eye][settings][frequency_resolution]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[beamformer][pipelines][0][flys eye][settings][time_resolution]")).toBeInTheDocument();
    
    expect(content.getByText("Incoherent Tied-Array Beams")).toBeInTheDocument();
    expect(content.getByLabelText("specification[beamformer][pipelines][0][incoherent][settings][quantisation][scale_max]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[beamformer][pipelines][0][incoherent][settings][quantisation][scale_min]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[beamformer][pipelines][0][incoherent][settings][subbands_per_file]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[beamformer][pipelines][0][incoherent][settings][time_integration_factor]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[beamformer][pipelines][0][incoherent][settings][frequency_resolution]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[beamformer][pipelines][0][incoherent][settings][time_resolution]")).toBeInTheDocument();
    
    expect(content.getByLabelText("specification[beamformer][pipelines][0][name]")).toBeInTheDocument();
    expect(content.getAllByText("Station groups").length).toBe(2);
    expect(content.getByLabelText("specification[beamformer][pipelines][0][station_groups][0][max_nr_missing]")).toBeInTheDocument();
    expect(content.getByText("Use PPF when beamforming")).toBeInTheDocument();
    
    expect(content.getByLabelText("specification[station_configuration][SAPs][0][digital_pointing][angle1]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][0][digital_pointing][angle2]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][0][digital_pointing][target]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][0][name]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][0][subbands]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][0][frequency]")).toBeInTheDocument();
    
    expect(content.getByText("Antenna set")).toBeInTheDocument();
    expect(content.getByText("Band-pass filter")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][station_groups][0][max_nr_missing]")).toBeInTheDocument();
    
    expect(content.getByLabelText("specification[station_configuration][tile_beam][angle1]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][tile_beam][angle2]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][tile_beam][target]")).toBeInTheDocument();

    
    // premery check for save button enable/disable
    expect(content.queryByTestId('save-btn')).not.toBeDisabled();
    expect(content.queryByTestId('taskName').value).not.toBeNull();
    const nameInputEl = content.queryByTestId('taskName');
    const nameInputElValue = nameInputEl.value;
    await act( async() => {
        fireEvent.change(nameInputEl, { target: { value: '' } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(nameInputEl.value).toBe('');
    expect(content.queryByTestId('save-btn')).toBeDisabled();
    const angle1El = content.getByLabelText("specification[beamformer][pipelines][0][coherent][SAPs][0][tabs][0][pointing][angle1]");
    expect(angle1El.value).not.toBe('');
    const averageTSValue = angle1El.value;
    await act( async() => {
        fireEvent.change(angle1El, { target: { value: '' } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.queryByTestId('save-btn')).toBeDisabled();

    await act( async() => {
        fireEvent.change(angle1El, { target: { value: '82h31m49.09s' } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.getByText("Not a valid input. Mimimum: 00:00:00.0000hours or 0 or 00:00:00.0000degrees, Maximum:23:59:59.9999hours or 6.2831 or 359:59:59.9999degrees.")).toBeVisible();
    expect(content.queryByTestId('save-btn')).toBeDisabled();

    await act( async() => {
        fireEvent.change(nameInputEl, { target: { value: nameInputElValue }});
        fireEvent.change(angle1El, { target: { value: averageTSValue } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.getByText("Not a valid input. Mimimum: 00:00:00.0000hours or 0 or 00:00:00.0000degrees, Maximum:23:59:59.9999hours or 6.2831 or 359:59:59.9999degrees.")).not.toBeVisible();
    expect(content.queryByTestId('save-btn')).not.toBeDisabled();

    //Save task
    await act(async () => {
        fireEvent.click(content.queryByTestId('save-btn'));
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(updateTaskSpy).toHaveBeenCalled();
}); 

it("test Pulsar Pipeline Task Draft edit", async () => {
    let content;
    await act(async () => {
        content = render(<Router><TaskEdit match={{params:{id: "50"}}} history={{}} location="/task/edit/draft/50" /></Router>);
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.getByText("Task - Edit")).toBeInTheDocument();
    // checking Task details and specification 
    expect(content.queryByTestId('taskName').value).toEqual('Pipeline');
    expect(content.queryByTestId('taskDescription').value).toEqual('Pulsar Pipeline for default HBA pulsar timing observation');
    expect(content.queryByTestId('taskShortDescription').value).toEqual('');
 
    expect(content.getByText("Specification")).toBeInTheDocument();
    expect(content.getByLabelText("specification[dspsr][digifil][dm]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[dspsr][digifil][channels_per_part]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[dspsr][digifil][integration_time_factor]")).toBeInTheDocument();
    expect(content.getAllByText("Coherent Dedispersion").length).toBe(2);
    expect(content.getAllByText("Enabled").length).toBe(5);
    expect(content.getByLabelText("specification[dspsr][filterbank][channels_per_part]")).toBeInTheDocument();
    expect(content.getByText("RFI excision")).toBeInTheDocument();
    expect(content.getByText("Optimise period & DM")).toBeInTheDocument();
    expect(content.getByLabelText("specification[dspsr][subintegration_length]")).toBeInTheDocument();
    expect(content.getByText("Single pulse subintegration")).toBeInTheDocument();
    expect(content.getByLabelText("specification[output][quantisation][scale]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[output][dynamic_spectrum][time_average]")).toBeInTheDocument();
    
    expect(content.getByLabelText("specification[presto][input][nr_blocks]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[presto][input][decode_sigma]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[presto][input][samples_per_block]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[presto][rrats][dm_range]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[presto][rfifind][blocks]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[presto][prepdata][dm]")).toBeInTheDocument();
    expect(content.getByText("Enable prepfold")).toBeInTheDocument();
    expect(content.getByText("Fold")).toBeInTheDocument();
    
    expect(content.getByLabelText("specification[pulsar][name]")).toBeInTheDocument();
    expect(content.getByText("Strategy")).toBeInTheDocument();
    
    expect(content.getByLabelText("specification[cluster_resources][cores_per_task]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[cluster_resources][parallel_tasks]")).toBeInTheDocument();
    expect(content.getByText("Cluster")).toBeInTheDocument();
    expect(content.getByText("Partition")).toBeInTheDocument();
    expect(content.getByText("Single-pulse search")).toBeInTheDocument();
    
    // premery check for save button enable/disable
    expect(content.queryByTestId('save-btn')).not.toBeDisabled();
    expect(content.queryByTestId('taskName').value).not.toBeNull();
    const nameInputEl = content.queryByTestId('taskName');
    const nameInputElValue = nameInputEl.value;
    await act( async() => {
        fireEvent.change(nameInputEl, { target: { value: '' } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(nameInputEl.value).toBe('');
    expect(content.queryByTestId('save-btn')).toBeDisabled();
    const channelPartEl = content.getByLabelText("specification[dspsr][digifil][channels_per_part]");
    expect(channelPartEl.value).not.toBe('');
    const channelPartValue = channelPartEl.value;
    await act( async() => {
        fireEvent.change(channelPartEl, { target: { value: '' } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.queryByTestId('save-btn')).toBeDisabled();

    await act( async() => {
        fireEvent.change(channelPartEl, { target: { value: '513' } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.getByText("Value must be at most 512.")).toBeVisible();
    expect(content.queryByTestId('save-btn')).toBeDisabled();

    await act( async() => {
        fireEvent.change(nameInputEl, { target: { value: nameInputElValue }});
        fireEvent.change(channelPartEl, { target: { value: channelPartValue } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.getByText("Value must be at most 512.")).not.toBeVisible();
    expect(content.queryByTestId('save-btn')).not.toBeDisabled();

    //Save task
    await act(async () => {
        fireEvent.click(content.queryByTestId('save-btn'));
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(updateTaskSpy).toHaveBeenCalled();
});

it("test Target Observation (IM HBA LoTSS strategy) edit", async () => {
    let content;
    await act(async () => {
        content = render(<Router><TaskEdit match={{params:{id: "23"}}} history={{}} location="/task/edit/draft/23" /></Router>);
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.getByText("Task - Edit")).toBeInTheDocument();
    // checking Task details and specification 
    expect(content.queryByTestId('taskName').value).toEqual('Target Observation');
    expect(content.queryByTestId('taskDescription').value).toEqual('Target Observation for LoTSS scheduling unit. HBA_DUAL_INNER, 120-168 MHz, 1s, 64ch/sb');
    expect(content.queryByTestId('taskShortDescription').value).toEqual('Paaa+01 & Paaa+02');
    expect(content.getAllByText("target observation").length).toBe(2);
    await waitFor(()=>expect(content.getByText("Specification")).toBeInTheDocument());
    expect(content.getByText("Specification")).toBeInTheDocument();
    expect(content.getByLabelText("specification[duration]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[correlator][integration_time]")).toBeInTheDocument();
    expect(content.getByText("Storage cluster")).toBeInTheDocument();
    expect(content.getByText("Topocentric Frequency Correction")).toBeInTheDocument();
    expect(content.getByText("Channels/subband")).toBeInTheDocument();
    expect(content.getByLabelText("specification[correlator][time_resolution]")).toBeInTheDocument();

    expect(content.getByLabelText("specification[station_configuration][SAPs][0][digital_pointing][angle1]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][0][digital_pointing][angle2]")).toBeInTheDocument();
    expect(content.getAllByText("Reference frame").length).toBe(3);
    expect(content.getByLabelText("specification[station_configuration][SAPs][0][digital_pointing][target]")).toBeInTheDocument();

    expect(content.getByLabelText("specification[station_configuration][SAPs][0][name]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][0][subbands]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][0][frequency]")).toBeInTheDocument();

    expect(content.getByLabelText("specification[station_configuration][SAPs][1][digital_pointing][angle1]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][1][digital_pointing][angle2]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][1][digital_pointing][target]")).toBeInTheDocument();

    expect(content.getByLabelText("specification[station_configuration][SAPs][1][name]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][1][subbands]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][1][frequency]")).toBeInTheDocument();

    expect(content.getByText("Antenna set")).toBeInTheDocument();
    expect(content.getByText("Band-pass filter")).toBeInTheDocument();
    expect(content.getByText("Station group 1")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][station_groups][0][max_nr_missing]")).toBeInTheDocument();
    expect(content.getByText("Station group 2")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][station_groups][1][max_nr_missing]")).toBeInTheDocument();
    expect(content.getByText("Station group 3")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][station_groups][2][max_nr_missing]")).toBeInTheDocument();

    expect(content.getByLabelText("specification[station_configuration][tile_beam][angle1]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][tile_beam][angle2]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][tile_beam][target]")).toBeInTheDocument();

    expect(content.getAllByText("enabled").length).toBe(2);
    expect(content.getByLabelText("specification[QA][file_conversion][nr_of_subbands]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[QA][file_conversion][nr_of_timestamps]")).toBeInTheDocument();
    expect(content.getByText("Inspection Plots")).toBeInTheDocument();
    expect(content.getByText("autocorrelation")).toBeInTheDocument();
    expect(content.getByText("crosscorrelation")).toBeInTheDocument();

     // premery check for save button enable/disable     
    expect(content.queryByTestId('save-btn')).not.toBeDisabled();

    expect(content.queryByTestId('taskName').value).not.toBeNull();
    const nameInputEl = content.queryByTestId('taskName');
    const nameInputElValue = nameInputEl.value;
    await act( async() => {
        fireEvent.change(nameInputEl, { target: { value: '' } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(nameInputEl.value).toBe('');
    expect(content.queryByTestId('save-btn')).toBeDisabled();
    const angle1El = content.getByLabelText("specification[station_configuration][SAPs][0][digital_pointing][angle1]");
    expect(angle1El.value).not.toBe('');
    const angle1Value = angle1El.value;
    await act( async() => {
        fireEvent.change(angle1El, { target: { value: '' } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.getByText("Not a valid input. Mimimum: 00:00:00.0000hours or 0 or 00:00:00.0000degrees, Maximum:23:59:59.9999hours or 6.2831 or 359:59:59.9999degrees.")).toBeVisible();
    expect(content.queryByTestId('save-btn')).toBeDisabled();

    await act( async() => {
        fireEvent.change(angle1El, { target: { value: '72h31m49.09s' } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.getByText("Not a valid input. Mimimum: 00:00:00.0000hours or 0 or 00:00:00.0000degrees, Maximum:23:59:59.9999hours or 6.2831 or 359:59:59.9999degrees.")).toBeVisible();
    expect(content.queryByTestId('save-btn')).toBeDisabled();

    await act( async() => {
        fireEvent.change(nameInputEl, { target: { value: nameInputElValue }});
        fireEvent.change(angle1El, { target: { value: angle1Value } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.getByText("Not a valid input. Mimimum: 00:00:00.0000hours or 0 or 00:00:00.0000degrees, Maximum:23:59:59.9999hours or 6.2831 or 359:59:59.9999degrees.")).not.toBeVisible();
    expect(content.queryByTestId('save-btn')).not.toBeDisabled();

    //angle2
    const angle2El = content.getByLabelText("specification[station_configuration][SAPs][0][digital_pointing][angle2]");
    expect(angle2El.value).not.toBe('');
    const angle2Value = angle2El.value;
    await act( async() => {
        fireEvent.change(angle2El, { target: { value: '' } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.getByText("Not a valid input. Mimimum: -90:00:00.0000degrees 0r -1.57079632679489661923, Maximum:90:00:00.0000degrees or 1.57079632679489661923.")).toBeVisible();
    expect(content.queryByTestId('save-btn')).toBeDisabled();

    await act( async() => {
        fireEvent.change(angle2El, { target: { value: '72h31m49.09s' } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.getByText("Not a valid input. Mimimum: -90:00:00.0000degrees 0r -1.57079632679489661923, Maximum:90:00:00.0000degrees or 1.57079632679489661923.")).toBeVisible();
    expect(content.queryByTestId('save-btn')).toBeDisabled();

    await act( async() => {
        fireEvent.change(nameInputEl, { target: { value: nameInputElValue }});
        fireEvent.change(angle2El, { target: { value: angle2Value } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.getByText("Not a valid input. Mimimum: -90:00:00.0000degrees 0r -1.57079632679489661923, Maximum:90:00:00.0000degrees or 1.57079632679489661923.")).not.toBeVisible();
    expect(content.queryByTestId('save-btn')).not.toBeDisabled();

    const integrationTimeEl = content.getByLabelText("specification[correlator][integration_time]");
    const time_resolutionEl = content.getByLabelText("specification[correlator][time_resolution]");
    const frequencyResolutionEl = content.getByLabelText("specification[correlator][frequency_resolution]");
    expect(integrationTimeEl.value).not.toBe('');
    expect(time_resolutionEl.value).not.toBe('');
    expect(frequencyResolutionEl.value).not.toBe('');
    await act( async() => {
        fireEvent.change(integrationTimeEl, { target: { value: 'NaN Value' } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.getByText("Value must be of type number.")).toBeVisible();
    expect(content.queryByTestId('save-btn')).toBeDisabled();
    await act( async() => {
        fireEvent.change(integrationTimeEl, { target: { value: 3 } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.getByText("Value must be of type number.")).not.toBeVisible();
    expect(content.queryByTestId('save-btn')).not.toBeDisabled();

    //Subband list
    const subbandListEl = content.getByLabelText("specification[station_configuration][SAPs][0][subbands]");
    const subbandlistFrequencyEl = content.getByLabelText("specification[station_configuration][SAPs][0][frequency]");
    expect(subbandListEl.value).not.toBe('');
    expect(subbandlistFrequencyEl.value).not.toBe('');
    await act( async() => {
        fireEvent.change(subbandListEl, { target: { value: '' } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.getByText("Not a valid input for Subband List.")).toBeVisible();
    expect(content.queryByTestId('save-btn')).toBeDisabled();
    await act( async() => {
        fireEvent.change(subbandListEl, { target: { value: '3..7' } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.getByText("Not a valid input for Subband List.")).not.toBeVisible();
    expect(subbandlistFrequencyEl.value).toBe('100.5-101.5');
    expect(content.queryByTestId('save-btn')).not.toBeDisabled();

    await act( async() => {
        fireEvent.change(subbandListEl, { target: { value: '-124' } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.getByText("Not a valid input for Subband List.")).toBeVisible();
    expect(content.queryByTestId('save-btn')).toBeDisabled();
    await act( async() => {
        fireEvent.change(subbandListEl, { target: { value: '3..7,267..399' } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.getByText("Not a valid input for Subband List.")).not.toBeVisible();
    expect(subbandlistFrequencyEl.value).toBe('100.5-101.5,152.1-178.0');
    expect(content.queryByTestId('save-btn')).not.toBeDisabled();

    //Save task
    await act(async () => {
        fireEvent.click(content.queryByTestId('save-btn'));
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(updateTaskSpy).toHaveBeenCalled();
}); 
 

it("test Calibrator Observation (IM HBA LoTSS strategy) edit", async () => {
    let content;
    await act(async () => {
        content = render(<Router><TaskEdit match={{params:{id: "21"}}} history={{}} location="/task/edit/draft/21" /></Router>);
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.getByText("Task - Edit")).toBeInTheDocument();
    // checking Task details and specification 
    expect(content.queryByTestId('taskName').value).toEqual('Calibrator Observation 1');
    expect(content.queryByTestId('taskDescription').value).toEqual('Calibrator Observation 1. Before the Target Observation');
    expect(content.getAllByText("calibrator observation").length).toBe(2);
    expect(content.queryByTestId('taskShortDescription').value).toEqual('Cal1 3Cabc');
    expect(content.getByText("IM HBA LoTSS - 2 Beams")).toBeInTheDocument();
    await waitFor(()=>expect(content.getByText("Specification")).toBeInTheDocument());
    expect(content.getByText("Specification")).toBeInTheDocument();
    expect(content.getByText("Auto-select")).toBeInTheDocument(); 
    expect(content.getByLabelText("specification[calibrator][name]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[calibrator][pointing][angle1]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[calibrator][pointing][angle2]")).toBeInTheDocument();
    expect(content.getAllByText("Reference frame").length).toBe(1);
    expect(content.getByLabelText("specification[calibrator][pointing][target]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[duration]")).toBeInTheDocument();

     // premery check for save button enable/disable
    expect(content.queryByTestId('save-btn')).not.toBeDisabled();
    expect(content.queryByTestId('taskName').value).not.toBeNull();
    const nameInputEl = content.queryByTestId('taskName');
    const nameInputElValue = nameInputEl.value;
    await act( async() => {
        fireEvent.change(nameInputEl, { target: { value: '' } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(nameInputEl.value).toBe('');
    expect(content.queryByTestId('save-btn')).toBeDisabled();
    const angle1El = content.getByLabelText("specification[calibrator][pointing][angle1]");
    expect(angle1El.value).not.toBe('');
    const averageTSValue = angle1El.value;
    await act( async() => {
        fireEvent.change(angle1El, { target: { value: '' } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.queryByTestId('save-btn')).toBeDisabled();

    await act( async() => {
        fireEvent.change(angle1El, { target: { value: '72h31m49.09s' } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.queryByTestId('save-btn')).toBeDisabled();
   expect(content.getByText("Not a valid input. Mimimum: 00:00:00.0000hours or 0 or 00:00:00.0000degrees, Maximum:23:59:59.9999hours or 6.2831 or 359:59:59.9999degrees.")).toBeVisible();
   
    await act( async() => {
        fireEvent.change(nameInputEl, { target: { value: nameInputElValue }});
        fireEvent.change(angle1El, { target: { value: averageTSValue } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.getByText("Not a valid input. Mimimum: 00:00:00.0000hours or 0 or 00:00:00.0000degrees, Maximum:23:59:59.9999hours or 6.2831 or 359:59:59.9999degrees.")).not.toBeVisible();
    expect(content.queryByTestId('save-btn')).not.toBeDisabled();

    //angle2
    const angle2El = content.getByLabelText("specification[calibrator][pointing][angle2]");
    expect(angle2El.value).not.toBe('');
    const angle2Value = angle2El.value;
    await act( async() => {
        fireEvent.change(angle2El, { target: { value: '' } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.getByText("Not a valid input. Mimimum: -90:00:00.0000degrees 0r -1.57079632679489661923, Maximum:90:00:00.0000degrees or 1.57079632679489661923.")).toBeVisible();
    expect(content.queryByTestId('save-btn')).toBeDisabled();

    await act( async() => {
        fireEvent.change(angle2El, { target: { value: '72h31m49.09s' } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.getByText("Not a valid input. Mimimum: -90:00:00.0000degrees 0r -1.57079632679489661923, Maximum:90:00:00.0000degrees or 1.57079632679489661923.")).toBeVisible();
    expect(content.queryByTestId('save-btn')).toBeDisabled();

    await act( async() => {
        fireEvent.change(nameInputEl, { target: { value: nameInputElValue }});
        fireEvent.change(angle2El, { target: { value: angle2Value } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.getByText("Not a valid input. Mimimum: -90:00:00.0000degrees 0r -1.57079632679489661923, Maximum:90:00:00.0000degrees or 1.57079632679489661923.")).not.toBeVisible();
    expect(content.queryByTestId('save-btn')).not.toBeDisabled();

    //Target
    const targetEl = content.getByLabelText("specification[calibrator][pointing][target]");
    expect(targetEl.value).not.toBe('');
    const targetValue = targetEl.value;
    await act( async() => {
        fireEvent.change(targetEl, { target: { value: '' } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.getByText("Value required.")).toBeVisible();
    expect(content.queryByTestId('save-btn')).toBeDisabled();

    await act( async() => {
        fireEvent.change(targetEl, { target: { value: targetValue } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.getByText("Value required.")).not.toBeVisible();
    expect(content.queryByTestId('save-btn')).not.toBeDisabled();

    //Save task
    await act(async () => {
        fireEvent.click(content.queryByTestId('save-btn'));
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(updateTaskSpy).toHaveBeenCalled();
}); 
 

it("test Preprocessing pipeline for calibrator (IM HBA LoTSS strategy) edit", async () => {
    let content;
    await act(async () => {
        content = render(<Router><TaskEdit match={{params:{id: "22"}}} history={{}} location="/task/edit/draft/22" /></Router>);
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.getByText("Task - Edit")).toBeInTheDocument();
    // checking Task details and specification 
    expect(content.queryByTestId('taskName').value).toEqual('Calibrator Pipeline 1');
    expect(content.queryByTestId('taskDescription').value).toEqual('Preprocessing Pipeline for Calibrator Observation 1');
    expect(content.queryByTestId('taskShortDescription').value).toEqual('Cal1 3Cabc/PP');
    expect(content.getAllByText("preprocessing pipeline").length).toBe(2);
    await waitFor(()=>expect(content.getByText("Specification")).toBeInTheDocument());
    expect(content.getByText("Specification")).toBeInTheDocument();

    expect(content.getByText("Specification")).toBeInTheDocument();
    expect(content.getByLabelText("specification[average][time_steps]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[average][frequency_steps]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[average][time_resolution]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[average][frequency_resolution]")).toBeInTheDocument();

    expect(content.getByText("CasA")).toBeInTheDocument();
    expect(content.getByText("CygA")).toBeInTheDocument();
    expect(content.getByText("HerA")).toBeInTheDocument();
    expect(content.getByText("HydraA")).toBeInTheDocument();
    expect(content.getByText("TauA")).toBeInTheDocument();
    expect(content.getByText("VirA")).toBeInTheDocument();

    expect(content.getByLabelText("specification[demix][time_steps]")).toBeInTheDocument();
    expect(content.getByText("Ignore target")).toBeInTheDocument();
    expect(content.getByLabelText("specification[demix][frequency_steps]")).toBeInTheDocument();

    expect(content.getByText("RFI flagging strategy")).toBeInTheDocument();
    expect(content.getByText("Flag outer channels")).toBeInTheDocument();
    expect(content.getByText("Flag auto correlations")).toBeInTheDocument();
    expect(content.getByText("Storage Manager")).toBeInTheDocument();

    expect(content.getByLabelText("specification[cluster_resources][cores_per_task]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[cluster_resources][parallel_tasks]")).toBeInTheDocument();
    expect(content.getByText("Cluster")).toBeInTheDocument();
    expect(content.getByText("Partition")).toBeInTheDocument();

     // premery check for save button enable/disable
    expect(content.queryByTestId('save-btn')).not.toBeDisabled();
    expect(content.queryByTestId('taskName').value).not.toBeNull();
    const nameInputEl = content.queryByTestId('taskName');
    const nameInputElValue = nameInputEl.value;
    await act( async() => {
        fireEvent.change(nameInputEl, { target: { value: '' } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(nameInputEl.value).toBe('');
    expect(content.queryByTestId('save-btn')).toBeDisabled();

    await act( async() => {
        fireEvent.change(nameInputEl, { target: { value: nameInputElValue }});
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.queryByTestId('save-btn')).not.toBeDisabled();

    const timeStepsEl = content.getByLabelText("specification[average][time_steps]");
    const timeResolutionEl = content.getByLabelText("specification[average][time_resolution]");
    expect(timeStepsEl.value).not.toBe('');
    expect(timeResolutionEl.value).not.toBe('');
    const timeStepsValue = timeStepsEl.value;
    await act( async() => {
        fireEvent.change(timeStepsEl, { target: { value: '' } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.getByText("Value must be of type integer.")).toBeVisible();
    expect(content.queryByTestId('save-btn')).toBeDisabled();

    await act( async() => {
        fireEvent.change(timeStepsEl, { target: { value: '-1' } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.queryByTestId('save-btn')).toBeDisabled();
    expect(content.getByText("Value must be at least 1.")).toBeVisible();
   
    await act( async() => {
        fireEvent.change(timeStepsEl, { target: { value: 'ABC' } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.queryByTestId('save-btn')).toBeDisabled();
    expect(content.getByText("Value must be of type integer.")).toBeVisible();

    await act( async() => {
        fireEvent.change(nameInputEl, { target: { value: nameInputElValue }});
        fireEvent.change(timeStepsEl, { target: { value: timeStepsValue } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.getByText("Value must be of type integer.")).not.toBeVisible();
    expect(timeResolutionEl.value).toBe(timeStepsValue)
    expect(content.queryByTestId('save-btn')).not.toBeDisabled();

    const frequencyStepsEl = content.getByLabelText("specification[average][frequency_steps]");
    const frequencyResolutionEl = content.getByLabelText("specification[average][frequency_resolution]");
    expect(frequencyStepsEl.value).not.toBe('');
    expect(frequencyResolutionEl.value).not.toBe('');
    await act( async() => {
        fireEvent.change(frequencyStepsEl, { target: { value: '' } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.queryByTestId('save-btn')).toBeDisabled();

    await act( async() => {
        fireEvent.change(frequencyStepsEl, { target: { value: '-1' } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.queryByTestId('save-btn')).toBeDisabled();
   
    await act( async() => {
        fireEvent.change(frequencyStepsEl, { target: { value: 'ABC' } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.queryByTestId('save-btn')).toBeDisabled();

    await act( async() => {
        fireEvent.change(frequencyStepsEl, { target: { value: 2 } });
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(frequencyResolutionEl.value).toBe('12.21')
    expect(content.queryByTestId('save-btn')).not.toBeDisabled();
    
    //Save task
    await act(async () => {
        fireEvent.click(content.queryByTestId('save-btn'));
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(updateTaskSpy).toHaveBeenCalled();
}); 

it("test IM+BF Observation (Solar Campaign strategy) edit", async () => {
    let content;
    await act(async () => {
        content = render(<Router><TaskEdit match={{params:{id: "69"}}} history={{}} location="/task/edit/draft/69" /></Router>);
        await new Promise((r) => setTimeout(r, 300));
    });
    expect(content.getByText("Task - Edit")).toBeInTheDocument();
    //checking Task details and specification 
    expect(content.queryByTestId('taskName').value).toEqual('IM+BF');
    expect(content.queryByTestId('taskDescription').value).toEqual('IM+BF Observation');
    expect(content.queryByTestId('taskShortDescription').value).toEqual('IM+BF');
    expect(content.getByLabelText("specification[calibrator][name]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[calibrator][pointing][angle1]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[calibrator][pointing][angle2]")).toBeInTheDocument();
    expect(content.getAllByText("Reference frame").length).toBe(3);
    expect(content.getByLabelText("specification[calibrator][pointing][target]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[duration]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[beamformer][pipelines][0][coherent][SAPs][0][subbands][list]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[beamformer][pipelines][0][coherent][SAPs][0][tab_rings][count]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[beamformer][pipelines][0][coherent][SAPs][0][tab_rings][width]")).toBeInTheDocument();
    expect(content.getAllByText("Bits/sample").length).toBe(3);
    expect(content.getAllByText("Output quantisation into integers").length).toBe(3);
    expect(content.getByLabelText("specification[beamformer][pipelines][0][coherent][settings][quantisation][scale_max]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[beamformer][pipelines][0][coherent][settings][quantisation][scale_min]")).toBeInTheDocument();
    expect(content.getAllByText("Stokes").length).toBe(3);
    expect(content.getByLabelText("specification[beamformer][pipelines][0][coherent][settings][subbands_per_file]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[beamformer][pipelines][0][coherent][settings][time_integration_factor]")).toBeInTheDocument();
    expect(content.getAllByText("Channels/subband").length).toBe(4);
    expect(content.getByLabelText("specification[beamformer][pipelines][0][coherent][settings][frequency_resolution]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[beamformer][pipelines][0][coherent][settings][time_resolution]")).toBeInTheDocument();
    expect(content.getAllByText("settings").length).toBe(3);
    expect(content.getByLabelText("specification[beamformer][pipelines][0][incoherent][settings][quantisation][scale_max]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[beamformer][pipelines][0][incoherent][settings][quantisation][scale_min]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[beamformer][pipelines][0][incoherent][settings][subbands_per_file]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[beamformer][pipelines][0][incoherent][settings][frequency_resolution]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[beamformer][pipelines][0][incoherent][settings][time_resolution]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[beamformer][pipelines][0][name]")).toBeInTheDocument();
    expect(content.getAllByText("Station group 1").length).toBe(2);
    expect(content.getByLabelText("specification[beamformer][pipelines][0][station_groups][0][max_nr_missing]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[correlator][integration_time]")).toBeInTheDocument();
    expect(content.getByText("Topocentric Frequency Correction")).toBeInTheDocument();
    expect(content.getAllByText("Channels/subband").length).toBe(4);
    expect(content.getByLabelText("specification[correlator][frequency_resolution]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[correlator][time_resolution]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][0][digital_pointing][angle1]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][0][digital_pointing][angle2]")).toBeInTheDocument();
    expect(content.getAllByText("Reference frame").length).toBe(3);
    expect(content.getByLabelText("specification[station_configuration][SAPs][0][digital_pointing][target]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][0][name]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][0][subbands]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][SAPs][0][frequency]")).toBeInTheDocument();
    expect(content.getAllByText("Station group 1").length).toBe(2);
    expect(content.getByLabelText("specification[station_configuration][station_groups][0][max_nr_missing]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][station_groups][1][max_nr_missing]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][tile_beam][angle1]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][tile_beam][angle2]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[station_configuration][tile_beam][target]")).toBeInTheDocument();
    expect(content.getAllByText("enabled").length).toBe(2);
    expect(content.getByLabelText("specification[QA][file_conversion][nr_of_subbands]")).toBeInTheDocument();
    expect(content.getByLabelText("specification[QA][file_conversion][nr_of_timestamps]")).toBeInTheDocument();
    expect(content.getByText("Inspection Plots")).toBeInTheDocument();
    expect(content.getByText("autocorrelation")).toBeInTheDocument();
    expect(content.getByText("crosscorrelation")).toBeInTheDocument();

     // premery check for save button enable/disable     
    expect(content.queryByTestId('save-btn')).not.toBeDisabled();
    expect(content.queryByTestId('taskName').value).not.toBeNull();
    const nameInputEl = content.queryByTestId('taskName');
    const nameInputElValue = nameInputEl.value;
    await act( async() => {
        fireEvent.change(nameInputEl, { target: { value: 'IM+BF Observation' } });
        await new Promise((r) => setTimeout(r, 150));
    });
    expect(nameInputEl.value).toBe('IM+BF Observation');
    const angle1El = content.getByLabelText("specification[station_configuration][SAPs][0][digital_pointing][angle1]");
    expect(angle1El.value).not.toBe('');
    await act( async() => {
        fireEvent.change(angle1El, { target: { value: '' } });
        await new Promise((r) => setTimeout(r, 150));
    });
    expect(content.getByText("Not a valid input. Mimimum: 00:00:00.0000hours or 0 or 00:00:00.0000degrees, Maximum:23:59:59.9999hours or 6.2831 or 359:59:59.9999degrees.")).toBeVisible();
    expect(content.queryByTestId('save-btn')).toBeDisabled();
    await act( async() => {
        fireEvent.change(angle1El, { target: { value: '72h31m49.09s' } });
        await new Promise((r) => setTimeout(r, 150));
        'how to make this work'
    });
    expect(content.getByText("Not a valid input. Mimimum: 00:00:00.0000hours or 0 or 00:00:00.0000degrees, Maximum:23:59:59.9999hours or 6.2831 or 359:59:59.9999degrees.")).toBeVisible();
    expect(content.queryByTestId('save-btn')).toBeDisabled();
    await act( async() => {
        fireEvent.change(angle1El, { target: { value: '2h31m49.09s' } });
        await new Promise((r) => setTimeout(r, 250));
    });
    expect(content.getByText("Not a valid input. Mimimum: 00:00:00.0000hours or 0 or 00:00:00.0000degrees, Maximum:23:59:59.9999hours or 6.2831 or 359:59:59.9999degrees.")).not.toBeVisible();
    expect(content.queryByTestId('save-btn')).not.toBeDisabled();

    //angle2
    const angle2El = content.getByLabelText("specification[station_configuration][SAPs][0][digital_pointing][angle2]");
    expect(angle2El.value).not.toBe('');
    const angle2Value = angle2El.value;
    await act( async() => {
        fireEvent.change(angle2El, { target: { value: '' } });
        await new Promise((r) => setTimeout(r, 150));
    });
    expect(content.getByText("Not a valid input. Mimimum: -90:00:00.0000degrees 0r -1.57079632679489661923, Maximum:90:00:00.0000degrees or 1.57079632679489661923.")).toBeVisible();
    expect(content.queryByTestId('save-btn')).toBeDisabled();
    await act( async() => {
        fireEvent.change(angle2El, { target: { value: '72h31m49.09s' } });
        await new Promise((r) => setTimeout(r, 150));
    });
    expect(content.getByText("Not a valid input. Mimimum: -90:00:00.0000degrees 0r -1.57079632679489661923, Maximum:90:00:00.0000degrees or 1.57079632679489661923.")).toBeVisible();
    expect(content.queryByTestId('save-btn')).toBeDisabled();

    await act( async() => {
        fireEvent.change(nameInputEl, { target: { value: nameInputElValue }});
        fireEvent.change(angle2El, { target: { value: angle2Value } });
        await new Promise((r) => setTimeout(r, 150));
    });
    expect(content.getByText("Not a valid input. Mimimum: -90:00:00.0000degrees 0r -1.57079632679489661923, Maximum:90:00:00.0000degrees or 1.57079632679489661923.")).not.toBeVisible();
    expect(content.queryByTestId('save-btn')).not.toBeDisabled();
    //Save task
    await act(async () => {
        fireEvent.click(content.queryByTestId('save-btn'));
        await new Promise((r) => setTimeout(r, 150));
    });
    expect(updateTaskSpy).toHaveBeenCalled();
}); 

it("test IM+BF Observation (Solar Campaign strategy) - validate customized fields", async () => {
    let content;
    await act(async () => {
        content = render(<Router><TaskEdit match={{params:{id: "69"}}} history={{}} location="/task/edit/draft/69" /></Router>);
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.getByText("Task - Edit")).toBeInTheDocument();
    const integrationTimeEl = content.getByLabelText("specification[correlator][integration_time]");
    const time_resolutionEl = content.getByLabelText("specification[correlator][time_resolution]");
    const frequencyResolutionEl = content.getByLabelText("specification[correlator][frequency_resolution]");
    expect(integrationTimeEl.value).not.toBe('');
    expect(time_resolutionEl.value).not.toBe('');
    expect(frequencyResolutionEl.value).not.toBe('');
    await act( async() => {
        fireEvent.change(integrationTimeEl, { target: { value: 'NaN Value' } });
        await new Promise((r) => setTimeout(r, 150));
    });
    expect(content.getByText("Value must be of type number.")).toBeVisible();
    expect(content.queryByTestId('save-btn')).toBeDisabled();
    await act( async() => {
        fireEvent.change(integrationTimeEl, { target: { value: 3 } });
        await new Promise((r) => setTimeout(r, 150));
    });
    expect(content.queryByTestId('save-btn')).not.toBeDisabled();
    //Subband list
    const subbandListEl = content.getByLabelText("specification[station_configuration][SAPs][0][subbands]");
    const subbandlistFrequencyEl = content.getByLabelText("specification[station_configuration][SAPs][0][frequency]");
    expect(subbandListEl.value).not.toBe('');
    expect(subbandlistFrequencyEl.value).not.toBe('');
    await act( async() => {
        fireEvent.change(subbandListEl, { target: { value: '' } });
        await new Promise((r) => setTimeout(r, 150));
    });
    expect(content.getByText("Not a valid input for Subband List.")).toBeVisible();
    expect(content.queryByTestId('save-btn')).toBeDisabled();
    await act( async() => {
        fireEvent.change(subbandListEl, { target: { value: '3..7' } });
        await new Promise((r) => setTimeout(r, 150));
    });
    expect(content.getByText("Not a valid input for Subband List.")).not.toBeVisible();
    expect(subbandlistFrequencyEl.value).toBe('100.5-101.5');
    expect(content.queryByTestId('save-btn')).not.toBeDisabled();
    await act( async() => {
        fireEvent.change(subbandListEl, { target: { value: '-124' } });
        await new Promise((r) => setTimeout(r, 150));
    });
    expect(content.getByText("Not a valid input for Subband List.")).toBeVisible();
    expect(content.queryByTestId('save-btn')).toBeDisabled();
    await act( async() => {
        fireEvent.change(subbandListEl, { target: { value: '3..7,267..399' } });
        await new Promise((r) => setTimeout(r, 150));
    });
    expect(content.getByText("Not a valid input for Subband List.")).not.toBeVisible();
    expect(subbandlistFrequencyEl.value).toBe('100.5-101.5,152.1-178.0');
    expect(content.queryByTestId('save-btn')).not.toBeDisabled();
    //Save task
    await act(async () => {
        fireEvent.click(content.queryByTestId('save-btn'));
        await new Promise((r) => setTimeout(r, 150));
    });
    expect(updateTaskSpy).toHaveBeenCalled();
});

it("test IM+BF Observation (Solar Campaign strategy) - validate other fields", async () => {
    let content;
    await act(async () => {
        content = render(<Router><TaskEdit match={{params:{id: "69"}}} history={{}} location="/task/edit/draft/69" /></Router>);
        await new Promise((r) => setTimeout(r, 500));
    });
    expect(content.getByText("Task - Edit")).toBeInTheDocument();
    //Beamformer
    const timeIntegrationFactorCoherent = content.getByLabelText("specification[beamformer][pipelines][0][coherent][settings][time_integration_factor]");
    const noRings = content.getByLabelText("specification[beamformer][pipelines][0][coherent][SAPs][0][tab_rings][count]");
    const maxFlyEye = content.getByLabelText("specification[beamformer][pipelines][0][flys eye][settings][quantisation][scale_max]");
    const timeIntegrationFactorCoherentE1 = timeIntegrationFactorCoherent.value;
    const noRingsE1 = noRings.value;
    
    await act( async() => {
        fireEvent.change(timeIntegrationFactorCoherent, { target: { value: '' } });
        await new Promise((r) => setTimeout(r, 150));
    });
    expect(content.getByText("Value must be of type integer.")).toBeVisible();
    expect(content.queryByTestId('save-btn')).toBeDisabled();
    await act( async() => {
        fireEvent.change(timeIntegrationFactorCoherent, { target: { value: 60 } });
        fireEvent.change(noRings, { target: { value: 6 } });
        await new Promise((r) => setTimeout(r, 150));
    });
    expect(timeIntegrationFactorCoherent.value).toBe("60");
    expect(noRings.value).toBe("6");
    expect(content.getByText("Value must be of type integer.")).not.toBeVisible();
    expect(content.queryByTestId('save-btn')).not.toBeDisabled();
    await act( async() => {
        fireEvent.change(noRings, { target: { value: noRingsE1 } });
        fireEvent.change(timeIntegrationFactorCoherent, { target: { value: timeIntegrationFactorCoherentE1 } });
        await new Promise((r) => setTimeout(r, 150));
    });
    expect(content.getByText("Value must be of type integer.")).not.toBeVisible();
    expect(content.queryByTestId('save-btn')).not.toBeDisabled();
    await act( async() => {
        fireEvent.change(maxFlyEye, { target: { value: '' } });
        await new Promise((r) => setTimeout(r, 150));
    });
    expect(content.getByText("Value must be of type number.")).toBeVisible();
    expect(content.queryByTestId('save-btn')).toBeDisabled();
    await act( async() => {
        fireEvent.change(maxFlyEye, { target: { value: 10 } });
        await new Promise((r) => setTimeout(r, 150));
    });
    expect(maxFlyEye.value).toBe("10");
    expect(content.getByText("Value must be of type number.")).not.toBeVisible();;
    expect(content.queryByTestId('save-btn')).not.toBeDisabled();
    //Save task
    await act(async () => {
        fireEvent.click(content.queryByTestId('save-btn'));
        await new Promise((r) => setTimeout(r, 150));
    });
    expect(updateTaskSpy).toHaveBeenCalled();
});

it("Checking Correctness of Custom fields in JEditor using Simple Beamforming Observation draft", async () => {
    let content;
    await act(async () => {
        content = render(<Router><TaskEdit match={{params:{id: "53"}}} history={{}} location="/task/edit/draft/53" /></Router>);
        await new Promise((r) => setTimeout(r, 300));
    });
    expect(content.getByText("Task - Edit")).toBeInTheDocument(); 
    expect(content.queryByTestId('save-btn')).not.toBeDisabled();
    const angle1El = content.getByLabelText("specification[station_configuration][SAPs][0][digital_pointing][angle1]");
    expect(angle1El.value).not.toBe('');
    await act( async() => {
        fireEvent.change(angle1El, { target: { value: '' } });
        await new Promise((r) => setTimeout(r, 150));
    });
    expect(content.getByText("Not a valid input. Mimimum: 00:00:00.0000hours or 0 or 00:00:00.0000degrees, Maximum:23:59:59.9999hours or 6.2831 or 359:59:59.9999degrees.")).toBeVisible();
    expect(content.queryByTestId('save-btn')).toBeDisabled();
    await act( async() => {
        fireEvent.change(angle1El, { target: { value: '72h31m49.09s' } });
        await new Promise((r) => setTimeout(r, 150));
        'how to make this work'
    });
    expect(content.getByText("Not a valid input. Mimimum: 00:00:00.0000hours or 0 or 00:00:00.0000degrees, Maximum:23:59:59.9999hours or 6.2831 or 359:59:59.9999degrees.")).toBeVisible();
    expect(content.queryByTestId('save-btn')).toBeDisabled();
    await act( async() => {
        fireEvent.change(angle1El, { target: { value: '20d31m49.09s' } });
        await new Promise((r) => setTimeout(r, 250));
    });
    expect(content.getByText("Not a valid input. Mimimum: 00:00:00.0000hours or 0 or 00:00:00.0000degrees, Maximum:23:59:59.9999hours or 6.2831 or 359:59:59.9999degrees.")).not.toBeVisible();
    expect(content.queryByTestId('save-btn')).not.toBeDisabled();

    //angle2
    const angle2El = content.getByLabelText("specification[station_configuration][SAPs][0][digital_pointing][angle2]");
    expect(angle2El.value).not.toBe('');
    const angle2Value = angle2El.value;
    await act( async() => {
        fireEvent.change(angle2El, { target: { value: '' } });
        await new Promise((r) => setTimeout(r, 150));
    });
    expect(content.getByText("Not a valid input. Mimimum: -90:00:00.0000degrees 0r -1.57079632679489661923, Maximum:90:00:00.0000degrees or 1.57079632679489661923.")).toBeVisible();
    expect(content.queryByTestId('save-btn')).toBeDisabled();
    await act( async() => {
        fireEvent.change(angle2El, { target: { value: '72h31m49.09s' } });
        await new Promise((r) => setTimeout(r, 150));
    });
    expect(content.getByText("Not a valid input. Mimimum: -90:00:00.0000degrees 0r -1.57079632679489661923, Maximum:90:00:00.0000degrees or 1.57079632679489661923.")).toBeVisible();
    expect(content.queryByTestId('save-btn')).toBeDisabled();

    await act( async() => {
        fireEvent.change(angle2El, { target: { value: angle2Value } });
        await new Promise((r) => setTimeout(r, 150));
    });
    expect(content.getByText("Not a valid input. Mimimum: -90:00:00.0000degrees 0r -1.57079632679489661923, Maximum:90:00:00.0000degrees or 1.57079632679489661923.")).not.toBeVisible();

    const subTask = content.getByLabelText("specification[station_configuration][SAPs][0][subbands]");
    const freqRes = content.getByLabelText("specification[station_configuration][SAPs][0][frequency]");
    expect(subTask.value).toBe('0..243');
    expect(freqRes.value).toBe('99.9-147.6');
    await act( async() => {
        fireEvent.change(subTask, { target: { value: '74..154' } });
        await new Promise((r) => setTimeout(r, 150));
    });
    expect(subTask.value).toBe('74..154');
    expect(freqRes.value).toBe('114.4-130.2');
    await act( async() => {
        fireEvent.change(subTask, { target: { value: '174..554' } });
        await new Promise((r) => setTimeout(r, 150));
    });
    expect(content.getByText("Not a valid input for Subband List.")).toBeVisible();
    await act( async() => {
        fireEvent.change(subTask, { target: { value: '0..243' } });
        await new Promise((r) => setTimeout(r, 150));
    });
    expect(content.getByText("Not a valid input for Subband List.")).not.toBeVisible();
    expect(content.queryByTestId('save-btn')).not.toBeDisabled();
    //Save task
    await act(async () => {
        fireEvent.click(content.queryByTestId('save-btn'));
        await new Promise((r) => setTimeout(r, 150));
    });
    expect(updateTaskSpy).toHaveBeenCalled();
});