import Filters from "./components/toolbar/Filters";
import React, {useEffect, useRef, useState} from "react";
//TODO: this import does something magical for the timeline; find out what
import CalendarTimeline from './CalendarTimeline';
import Timeline, {
    CursorMarker,
    CustomMarker,
    DateHeader,
    SidebarHeader,
    TimelineHeaders,
    TimelineMarkers
} from "react-calendar-timeline";
import moment from "moment";
import {getIntervalRendererLST} from "./helpers/timeline.headers.helper";
import UIConstants from "../../utils/ui.constants";
import {groupRenderer, itemRenderer} from "./helpers/timeline.renderer.helper";
import containerResizeDetector from "react-calendar-timeline/lib/resize-detector/container";
import TimelineItemPopover from "./components/TimelineItemPopover";
import {
    calculateSizeAndSetPanelClassName,
    createGroups,
    getCursorDateTimes,
    getHeaderActions,
    getPageHeaderOptionsMenuItems,
    getSchedulerActions,
    updateSchedulerStatuses,
} from "./helpers/week.view.helper";
import {getStore} from "../../services/store.helper";
import DateTimeNavigator from "./components/toolbar/DateTimeNavigator";
import ZoomAndMove from "./components/toolbar/ZoomAndMove";
import {ProgressBar} from "primereact/progressbar";
import Legendbar from "./components/Legendbar";
import PanelButtons from "./components/PanelButtons";
import SchedulingUnitSummary from "../Scheduling/summary";
import {useLocation} from "react-router-dom";
import {editConstraintSettings, updateSUConstraintTemplate} from "../Scheduling/summary.helper";
import PageHeader from "../../layout/components/PageHeader";
import {CustomDialog} from "../../layout/components/CustomDialog";
import DynamicScheduler from "../../components/DynamicScheduler";
import {TieredMenu} from "primereact/tieredmenu";

import { ProgressSpinner } from 'primereact/progressspinner';
import ReservationSummary from "../Reservation/reservation.summary";
import {
    fetchLSTShift, fetchReservationSummaryInformation,
    fetchSchedulerStatuses,
    fetchSunTimings, fetchSUSummaryInformation,
    fetchTimelineData,
    fetchUserPermissions
} from "./data/week.view.data";

function getTimelineHeaders(headerSettings) {
    return <TimelineHeaders className="sticky">
        <SidebarHeader>
            {({getRootProps}) => { //getRootProps is mandatory for the library
                return <div {...getRootProps()} className="sidebar-header">
                    <div className="sidebar-header-row">UTC</div>
                    <div className="sidebar-header-row" title="Approximate LST">~ LST</div>
                </div>
            }}
        </SidebarHeader>
        <DateHeader unit={headerSettings.unit} labelFormat="HH:mm"/>
        <DateHeader unit={headerSettings.unit} intervalRenderer={({getIntervalProps, intervalContext}) => {
            return getIntervalRendererLST({getIntervalProps, intervalContext}, headerSettings.lstShiftInSeconds)
        }}/>
    </TimelineHeaders>;
}

function getTimelineMarkers(setCursorDate) {
    const now = moment(moment().utc(false).format(UIConstants.UTC_DATE_TIME_FORMAT))
    return <TimelineMarkers>
        <CustomMarker date={now.valueOf()}>
            {({styles}) => {
                const customStyles = {
                    ...styles,
                    backgroundColor: 'var(--green)',
                    zIndex: 998
                }
                return <div style={customStyles}/>
            }}
        </CustomMarker>
        <CursorMarker>
            {({styles, date}) => {
                setCursorDate(date) //TODO: don't do this but find a better solution
                const customStyles = {
                    ...styles,
                    backgroundColor: 'var(--blue-400)',
                    zIndex: 998,
                    height: "100%"
                }
                return <div style={customStyles}/>
            }}
        </CursorMarker>
    </TimelineMarkers>;
}

/**
 * Parent component. Check data flow and actions here: https://drive.google.com/file/d/11iZOwfD4fG7TC-24POEId_TVkNczE0O3/view?pli=1
 * Also, please keep the diagram up-to-date
 * @return {JSX.Element}
 * @constructor
 */
export default function WeekView() {
    const timelineStore = getStore(UIConstants.STORE_KEY_TIMELINE)
    const location = useLocation()
    let actionsMenuRef = useRef()

    //general page info
    const [permissions, setPermissions] = useState()
    const [schedulerSettings, setSchedulerSettings] = useState({
        isDynamicSchedulerOn: undefined,
        isFixedTimeSchedulerOn: undefined,
        isDynamicSchedulerActive: undefined,
    })
    const [showSchedulerSettingsDialog, setShowSchedulerSettingsDialog] = useState(false)

    //data info
    const [isLoading, setIsLoading] = useState(false)
    const [refetchToggle, setRefetchToggle] = useState(false)
    const [data, setData] = useState({
        schedulingUnits: [],
        reservations: [],
    })

    //timeline component
    const [startTime, setStartTime] = useState(moment().startOf('day'))
    const [endTime, setEndTime] = useState(moment().endOf('day').add(UIConstants.WEEKVIEW_SHOW_NUMBER_OF_DAYS, 'days'))
    const [groups, setGroups] = useState(createGroups(moment().startOf('day')))
    const [items, setItems] = useState({
        schedulingUnitItems: [],
        reservationItems: [],
        sunTimingsItems: []
    })
    const [headerSettings, setHeaderSettings] = useState({
        timeSteps: timelineStore.timeSteps ?? UIConstants.ALL_TIMESTEPS[0],
        unit: timelineStore.headerUnit ?? "hour",
        lstShiftInSeconds: 0
    })
    const [visibleTime, setVisibleTime] = useState({
        start: undefined,
        end: undefined
    })

    //timeline item pop-over component
    const [mouseOverItem, setMouseOverItem] = useState()
    const [popPosition, setPopPosition] = useState({display: 'none'})
    const [summaryItem, setSummaryItem] = useState()

    //marker that also displays time information on thge group
    const [cursorTimeUTC, setCursorTimeUTC] = useState()
    const [cursorInformation, setCursorInformation] = useState({
        utc: undefined,
        lst: undefined,
    })

    //left-right panel elements TODO: combine states (into left + right sections)
    const [showLegendbar, setShowLegendbar] = useState(false)
    const [showSummary, setShowSummary] = useState(false)
    const [panelClassNames, setPanelClassNames] = useState({
        leftPanelClassName: "timeline-panel",
        middlePanelClassName: "timeline-panel",
        rightPanelClassName: "timeline-panel"
    })
    const [leftPanelSize, setLeftPanelSize] = useState(30) //30% of the screen

    //details summary
    const [summarySettings, setSummarySettings] = useState({
        isLoading: false,
        schedulingUnitItem: undefined,
        reservationItem: undefined,
        summaryConstraintTemplate: undefined
    })
    //TODO: needed for SU summary component; should be refactored into that component + updater functions
    const [onClose, setOnClose] = useState(false)
    const [cancelStatus, setCancelStatus] = useState(false)
    const [onSaveDisabled, setOnSaveDisabled] = useState(true)

    useEffect(() => { //TODO: extract
        const currentGroupIndex = groups.findIndex(group => group.title === mouseOverItem?.group)
        if (currentGroupIndex >= 0) {
            const currentGroupDate = moment(groups[currentGroupIndex].title, UIConstants.CALENDAR_GROUP_FORMAT).format(UIConstants.CALENDAR_DEFAULTDATE_FORMAT)
            getCursorDateTimes(cursorTimeUTC, currentGroupIndex, currentGroupDate, headerSettings.lstShiftInSeconds, setCursorInformation);
            let newGroups = groups.map(group => Object.assign({}, group));
            newGroups.forEach(group => delete group.cursorInfo)
            newGroups[currentGroupIndex].cursorInfo = cursorInformation
            setGroups(newGroups)
        }
    }, [cursorTimeUTC])


    useEffect(() => { //initializer
        setIsLoading(true)
        setGroups(createGroups(startTime))
        if (!Object.values(visibleTime).some(value => value === undefined)) { //TODO: extract
            const startDate = moment(startTime).format(UIConstants.CALENDAR_DEFAULTDATE_FORMAT)
            const visibleStartTime = moment(visibleTime.start).format(UIConstants.CALENDAR_TIME_FORMAT)
            const visibleEndTime = moment(visibleTime.end).format(UIConstants.CALENDAR_TIME_FORMAT)
            const startDateWithVisibleStartTime = moment(startDate + " " + visibleStartTime, UIConstants.CALENDAR_DATETIME_FORMAT);
            const startDateWithVisibleEndTime = moment(startDate + " " + visibleEndTime, UIConstants.CALENDAR_DATETIME_FORMAT);
            setVisibleTime({
                start: startDateWithVisibleStartTime,
                end: startDateWithVisibleEndTime
            })
        }

        const loadingStates = []
        loadingStates.push(fetchTimelineData(startTime, endTime, setData).catch(e => console.error("Couldn't get scheduling units and/or reservations", e)))
        loadingStates.push(fetchSunTimings(startTime, setItems).catch(e => console.error("Couldn't retrieve sun timings", e)))
        loadingStates.push(fetchLSTShift(startTime, setHeaderSettings).catch(e => console.error("Couldn't get current lst from the server", e)))
        if (!permissions) {
            loadingStates.push(fetchUserPermissions(setPermissions).catch(e => console.error("Couldn't retrieve the user permissions:", e)))
        }
        if (Object.values(schedulerSettings).some(value => value === undefined)) {
            loadingStates.push(fetchSchedulerStatuses(setSchedulerSettings).catch(e => console.error("Couldn't retrieve the scheduler's statuses (dynamic on, fixed time on, dynamic active:", e)))
        }
        Promise.all(loadingStates).then(() => setIsLoading(false))
    }, [startTime, endTime, refetchToggle])

    useEffect(() => { //resizing
        calculateSizeAndSetPanelClassName(showLegendbar, showSummary, leftPanelSize, setPanelClassNames)
    }, [showLegendbar, showSummary, leftPanelSize])

    useEffect(() => { //summary information
        if (summaryItem === undefined) {
            setShowSummary(false)
            setSummarySettings({
                isLoading: false,
                schedulingUnitItem: undefined,
                reservationItem: undefined,
                summaryConstraintTemplate: undefined
            })
        } else {
            setSummarySettings(prevState => ({
                ...prevState,
                isLoading: true,
            }))
            setShowSummary(true)
            if (summaryItem.type === "RESERVATION") {
                fetchReservationSummaryInformation(summaryItem.id, setSummarySettings).catch(e => console.error("Couldn't retrieve reservations details for id: ", summaryItem.id, e))
            } else { //a Scheduling Unit summary is selected
                fetchSUSummaryInformation(summaryItem.id, setSummarySettings).catch(e => console.error("Couldn't retrieve summary details for id: ", summaryItem.id, e))
            }
        }
    }, [summaryItem]);

    return <div>
        <div className={(isLoading ? "" : "hide-element")}>
            <ProgressBar className={isLoading ? "" : "hide-element"} mode="indeterminate"
                         style={{height: '0.25rem'}}/>
        </div>
        {!permissions ? <ProgressSpinner className="m-4" style={{color: "var(--primary-300)"}}/> :
            <TieredMenu className="app-header-menu" popup ref={actionsMenuRef}
                        model={getPageHeaderOptionsMenuItems(permissions)}/>}
        <PageHeader location={location} title={'Scheduling Units - Week View'}
                    actions={getSchedulerActions(schedulerSettings, setShowSchedulerSettingsDialog)
                        .concat(getHeaderActions(isLoading, setRefetchToggle, actionsMenuRef))}/>
        <div className="p-grid">
            <div className={panelClassNames.leftPanelClassName}>
                <Legendbar className={showLegendbar ? undefined : "hide-element"}/>
            </div>

            <div className={panelClassNames.middlePanelClassName}>
                <PanelButtons showLegendbar={showLegendbar}
                              setShowLegendbar={setShowLegendbar}
                              leftPanelSize={leftPanelSize}
                              setLeftPanelSize={setLeftPanelSize}/>

                <div className="timeline-tools">
                    <Filters timelineStore={timelineStore}
                             visibleStartTime={visibleTime.start}
                             setItems={setItems}
                             data={data}
                             setRefetchToggle={setRefetchToggle}

                    />
                    <DateTimeNavigator timelineStore={timelineStore}
                                       startTime={startTime}
                                       endTime={endTime}
                                       setStartTime={setStartTime}
                                       setEndTime={setEndTime}
                    />
                    <ZoomAndMove timelineStore={timelineStore}
                                 startTime={startTime}
                                 setVisibleTime={setVisibleTime}
                                 visibleTime={visibleTime}
                                 headerSettings={headerSettings}
                                 setHeaderSettings={setHeaderSettings}
                    />
                </div>
                {Object.values(visibleTime).some(value => value === undefined) ? <ProgressSpinner /> :
                    <Timeline
                        groups={groups}
                        items={[...items.sunTimingsItems, ...items.schedulingUnitItems, ...items.reservationItems]}
                        defaultTimeStart={startTime}
                        defaultTimeEnd={endTime}
                        visibleTimeStart={visibleTime.start.valueOf()}
                        visibleTimeEnd={visibleTime.end.valueOf()}
                        timeSteps={headerSettings.timeSteps.value}
                        buffer={2}
                        //general styling settings timeline
                        lineHeight={40}
                        itemHeightRatio={1}
                        resizeDetector={containerResizeDetector}
                        //general action settings timeline
                        canMove={false}
                        canResize={false}
                        stackItems={true}
                        //render settings
                        groupRenderer={({group}) => groupRenderer({group})}
                        itemRenderer={({
                                           item,
                                           timelineContext,
                                           itemContext,
                                           getItemProps,
                                           getResizeProps
                                       }) => itemRenderer({
                            item,
                            timelineContext,
                            itemContext,
                            getItemProps,
                            getResizeProps
                        }, setPopPosition, setMouseOverItem, setSummaryItem)}
                    >
                        {getTimelineHeaders(headerSettings)}
                        {getTimelineMarkers(setCursorTimeUTC)}
                    </Timeline>}
            </div>
            <div className={panelClassNames.rightPanelClassName + " summary"}>
                {showSummary &&
                    (summarySettings.isLoading ? <ProgressSpinner className="m-4" style={{color: "var(--primary-300)"}}/>
                            : (summaryItem.type === "RESERVATION" ?
                                (summarySettings.reservationItem === undefined ?
                                    <ProgressSpinner className="m-4" style={{color: "var(--primary-300)"}}/> :
                                    <ReservationSummary reservation={summarySettings.reservationItem}
                                                        viewInNewWindow
                                                        location={location}
                                                        closeCallback={() => setShowSummary(false)}>
                                    </ReservationSummary>) :
                                (summarySettings.schedulingUnitItem === undefined ?
                                    <ProgressSpinner className="m-4" style={{color: "var(--primary-300)"}}/> :
                                    <SchedulingUnitSummary
                                        schedulingUnit={summarySettings.schedulingUnitItem}
                                        viewInNewWindow
                                        constraintsTemplate={summarySettings.summaryConstraintTemplate}
                                        onUpdate={(suId, updatedConstraintTemplate) =>
                                            updateSUConstraintTemplate(suId, updatedConstraintTemplate, setOnClose, setOnSaveDisabled, setShowSummary)}
                                        newPopupvalue={() => {
                                        }}
                                        onClose={onClose}
                                        onSavedisable={onSaveDisabled}
                                        onCloseFn={(val) => editConstraintSettings(val, setOnClose, setOnSaveDisabled, setCancelStatus)}
                                        closeCallback={() => setShowSummary(false)}
                                        cancelCheckstatus={(val) => setCancelStatus(val)}
                                        cancelStatus={cancelStatus}
                                        location={location}>
                                    </SchedulingUnitSummary>))

                    )}
            </div>
        </div>
        {mouseOverItem && mouseOverItem.type !== "SUNTIME" ?
            <div className="p-overlaypanel p-component timeline-popover"
                 style={{...popPosition}}>
                <TimelineItemPopover mouseOverItem={mouseOverItem}/>
            </div> : null}
        <div className="p-grid" data-testid="confirm_dialog">
            <CustomDialog type="success"
                          visible={showSchedulerSettingsDialog}
                          showIcon={false}
                          header="Scheduling Settings"
                          message={<DynamicScheduler callBack={async (val, status) =>
                              await updateSchedulerStatuses(val, status, setSchedulerSettings)}
                          />}
                          onClose={() => setShowSchedulerSettingsDialog(false)}
            />
        </div>
    </div>
}