import _ from 'lodash';
import ScheduleService from '../../services/schedule.service';
import UtilService from '../../services/util.service';
import WorkflowService from '../../services/workflow.service';
import UnitConverter from '../../utils/unit.converter';

/**
 * Class with common functions used by both Timeline View and Week Overview components.
 */
class TimelineCommonUtils {

    // Constants used in timeline views
    ZOOM_LEVELS = [{name: '30 Minutes', value: 30 * 60},
                     {name: '1 Hour', value: 1 * 60 * 60},
                     {name: '3 Hours', value: 3 * 60 * 60},
                     {name: '6 Hours', value: 6 * 60 * 60},
                     {name: '12 Hour', value: 12 * 60 * 60},
                     {name: '1 Day', value: 24 * 60 * 60},
                     {name: '2 Days', value: 2 * 24 * 60 * 60},
                     {name: '3 Days', value: 3 * 24 * 60 * 60},
                     {name: '5 Days', value: 5 * 24 * 60 * 60},
                     {name: '1 Week', value: 7 * 24 * 60 * 60},
                     {name: '2 Weeks', value: 14 * 24  * 60 * 60},
                     {name: '4 Weeks', value: 28 * 24 * 60 * 60},
                     {name: 'Custom', value: 0 * 60 * 60}];
    DEFAULT_ZOOM_LEVEL = "2 Days";
    WORKFLOW_STATUS_MAP = {
        'wait scheduled': 'Waiting To Be Scheduled',
        'wait observing': 'Scheduled',
        // 'wait processed': 'Scheduled',
        'qa reporting to': 'QA Reporting (TO)',
        'qa reporting sos': 'QA Reporting (SDCO)',
        'pi verification': 'PI Verification',
        'decide acceptance': 'Decide Acceptance',
        'wait ingest done': 'Ingesting',
        'decide unpin data': 'Unpin Data',
        'end': 'Done'
     };

    constructor() {
        this.STATUS_BEFORE_SCHEDULED = ['defining', 'defined', 'schedulable'];  // Statuses before scheduled to get station_group
        this.loadMainStationGroups();
        // this.loadSUWorkflowTasks();
    }

    /**
     * To group the stations under CS,RS,IS to show the count in Popover
     */
    async loadMainStationGroups() {
        async function fetchMainStationGroups() {
            return await ScheduleService.getMainGroupStations()
        }

        this.mainStationGroups = await fetchMainStationGroups().catch(e => console.error("Couldn't get retrieve main station groups", e))
    }

    /**
     * TO get the main station groups (Core, Remote, International)
     * @returns Array - main station groups
     */
    getMainStationGroups() {
        return this.mainStationGroups;
    }

    /**
     * Function to fetch all Workflow processes and tasks to get the workflow status column
     * in the SU List table.
     */
     async loadSUWorkflowTasks() {
        this.worflowProcesses = await WorkflowService.getWorkflowProcesses();
        this.workflowTasks = await WorkflowService.getWorkflowTasks();
    }
                
    /**
     * Get all stations of the SU bleprint from the observation task or subtask based on the SU status.
     * @param {Object} suBlueprint
     */
     getSUStations(suBlueprint) {
        let stations = [];
        /* Get all observation tasks */
        let observationTasks = _.filter(suBlueprint.tasks, (task) => { return task.task_type.toLowerCase() === "observation" });
        if (observationTasks.length === 0) {
            observationTasks = _.filter(suBlueprint.task_blueprints, (task) => { return task.task_type.toLowerCase() === "observation" });
        }
        for (const observationTask of observationTasks) {
            /** If the status of SU is before scheduled, get all stations from the station_groups from the task specification_docs */
            if (this.STATUS_BEFORE_SCHEDULED.indexOf(suBlueprint.status.toLowerCase()) >= 0
                && observationTask.specifications_doc.station_configuration?.station_groups) {
                for (const grpStations of _.map(observationTask.specifications_doc.station_configuration?.station_groups, "stations")) {
                    stations = _.concat(stations, grpStations);
                }
            } else if (this.STATUS_BEFORE_SCHEDULED.indexOf(suBlueprint.status.toLowerCase()) < 0
                && observationTask.subtasks) {
                /** If the status of SU is scheduled or after get the stations from the subtask specification tasks */
                for (const subtask of observationTask.subtasks) {
                    if (subtask.specifications_doc.stations) {
                        stations = _.concat(stations, subtask.specifications_doc.stations.station_list);
                    }
                }
            }
        }
        return _.uniq(stations);
    }

    /**
     * Group the SU stations to main groups Core, Remote, International
     * @param {Object} stationList 
     */
    groupSUStations(stationList) {
        let suStationGroups = {};
        for (const group in this.mainStationGroups) {
            suStationGroups[group] = _.intersection(this.mainStationGroups[group], stationList);
        }
        return suStationGroups;
    }

    /**
     * This funtion gets the stations involved in a SUB, group them to the majour groups,
     * gets the count of stations in each group and format to 'CS/RS/IS' & their counts '12/13/14'
     * to display in SU list and SU block mouse over popup.
     * @param {Object} itemSU - SU Blueprint object
     * @returns Object - A JSON object with station groups and count as proerpty with formatted string to display
     */
    getSUStationGroupCount(itemStations) {
        const itemStationGroups = this.groupSUStations(itemStations);
        let suStationGroupCount = { groups: "", counts: "" };
        for (const stationgroup of _.keys(itemStationGroups)) {
            let groups = suStationGroupCount.groups;
            let counts = suStationGroupCount.counts;
            if (groups) {
                groups = groups.concat("/");
                counts = counts.concat("/");
            }
            // Get station group 1st character and append 'S' to get CS,RS,IS 
            groups = groups.concat(stationgroup.substring(0, 1).concat('S'));
            counts = counts.concat(itemStationGroups[stationgroup].length);
            suStationGroupCount.groups = groups;
            suStationGroupCount.counts = counts;
        }
        return suStationGroupCount;
    }

    /**
     * To get all workflow processes and tasks for a specific set of SUB Ids passed as array.
     * @param {Array} suIds - Array of scheduling unit blueprint id
     * @returns - Object with all workflow processes and tasks as properties. {processes: [], tasks:[]}
     */
    async getWorkflowsAndTasks(suIds) {
        let workflow = {};
        let processes = [];
        let tasks = [];
        if (suIds.length > 0) {
            processes = await WorkflowService.getSUWorkflowProcesses(suIds.join(","));
            const processIds = _.map(processes, 'id');
            if (processIds.length > 0) {
                tasks = await WorkflowService.getWorkflowProcessesTasks(processIds);
            }
        }
        workflow.processes = processes;
        workflow.tasks = tasks;
        return workflow;
    }

    /**
     * Function to get the current workflow status for the scheduling unit
     * @param {Array} workflowProcesses - List of workflow processes filtered for the SUBs in the table view
     * @param {Array} workflowTasks - List of workflow tasks filtered for the workflow processes
     * @param {Number} scheduleunitId - Scheduling Unit blueprint id for which the status to be found
     * @returns workflow status as String
     */
     getWorkflowStatus(workflows, scheduleunitId) {
        let workflowStatus = null;
        const suWorkflow = _.find(workflows.processes, ['su', scheduleunitId]);
        const suWorkflowTasks = suWorkflow?_.filter(workflows.tasks, ['process', suWorkflow.id]):[];
        const lastWorkflowTask = suWorkflowTasks.length > 0?(_.orderBy(suWorkflowTasks, ['id'], ['desc']))[0]:null;
        if (lastWorkflowTask && lastWorkflowTask.flow_task) {
            workflowStatus = this.WORKFLOW_STATUS_MAP[lastWorkflowTask.flow_task.toLowerCase()];
        }
        return workflowStatus;
    }

    /**
     * Function to get the current workflow details for the scheduling unit
     * @param {Array} workflowProcesses - List of workflow processes filtered for the SUBs in the table view
     * @param {Array} workflowTasks - List of workflow tasks filtered for the workflow processes
     * @param {Number} scheduleunitId - Scheduling Unit blueprint id for which the status to be found
     * @returns workflow status as Object with status, results_accepted.
     */
     getWorkflowDetails(workflows, scheduleunitId) {
        let workflowDetails = {};
        const suWorkflow = _.find(workflows.processes, ['su', scheduleunitId]);
        const suWorkflowTasks = suWorkflow?_.filter(workflows.tasks, ['process', suWorkflow.id]):[];
        const lastWorkflowTask = suWorkflowTasks.length > 0?(_.orderBy(suWorkflowTasks, ['id'], ['desc']))[0]:null;
        if (lastWorkflowTask && lastWorkflowTask.flow_task) {
            workflowDetails["dataAccepted"] = suWorkflow.results_accepted;  // TODO: replaced flag, now it's in the SUB model so update this ref.
            workflowDetails["status"] = this.WORKFLOW_STATUS_MAP[lastWorkflowTask.flow_task.toLowerCase()];
        }
        return workflowDetails;
    }

    /**
     * Format the SUB object to display in unschedulable tab list.
     * @param {Object} unschedulableBlueprint - Blueprint scheduling unit with status unschedulable
     * @param {Array} workflows - list of workflow objects
     * @returns Object - formatted unschedulable blueprint object
     */
    formatUnschedulableSUB(unschedulableBlueprint, workflows) {
        unschedulableBlueprint['actionpath'] = `/schedulingunit/view/blueprint/${unschedulableBlueprint.id}`;
        unschedulableBlueprint.suDraft = unschedulableBlueprint.draft;
        unschedulableBlueprint.project = unschedulableBlueprint.draft.scheduling_set.project.name;
        unschedulableBlueprint.project_rank = unschedulableBlueprint.draft.scheduling_set.project.rank;
        unschedulableBlueprint.suSet = unschedulableBlueprint.draft.scheduling_set;
        unschedulableBlueprint.durationInSec = unschedulableBlueprint.duration;
        unschedulableBlueprint.duration = UnitConverter.getSecsToHHmmss(unschedulableBlueprint.duration);
        unschedulableBlueprint.workflowStatus = this.getWorkflowStatus(workflows, unschedulableBlueprint.id);
        unschedulableBlueprint.task_content = "";
        unschedulableBlueprint.observ_template_name = unschedulableBlueprint.draft.observation_strategy_template?unschedulableBlueprint.draft.observation_strategy_template.name:null;
        unschedulableBlueprint.tasks = unschedulableBlueprint.task_blueprints;
        const itemStations = this.getSUStations(unschedulableBlueprint);
        unschedulableBlueprint.stationGroupCount = this.getSUStationGroupCount(itemStations).counts;
        for (let task of unschedulableBlueprint.tasks) {
            const controlTask = _.find(task.subtasks, subtask => { return subtask.primary});
            task.controlId = controlTask?controlTask.id:"";
            if (task.task_type === "observation") {
                task.antenna_set = task.specifications_doc.station_configuration?.antenna_set;
                task.band = task.specifications_doc.station_configuration?.filter;
            }
        }
        return unschedulableBlueprint;
    }

    /**
     * Function to store the Last UI option atrributes as preference to consider while loading the page.
     * @param {Object} timelineUIAttributes - JSON object with all options and its last selected values.
     */
    storeUIAttributes(timelineUIAttributes) {
        UtilService.localStore({ type: 'set', key: 'TIMELINE_UI_ATTR', value: timelineUIAttributes });
    }
    
}

export default TimelineCommonUtils;