import {InputSwitch} from "primereact/inputswitch";
import {MultiSelect} from "primereact/multiselect";
import React, {useEffect, useState} from "react";
import {updateStore} from "../../../../services/store.helper";
import UIConstants from "../../../../utils/ui.constants";
import {
    getReservationItem,
    getTimelineItem,
    splitObjectIfSpanIsMultipleDays
} from "../../helpers/toolbar/filters.helper";
import moment from "moment";

import { ProgressSpinner } from 'primereact/progressspinner';
        
import {fetchAllOptionsForMultiSelects} from "../../data/filters.data";

function Toggle(props) { //TODO: extract to separate class
    const {
        labelName,
        tooltipText,
        checkedValue,
        onChangeCallback,
    } = props

    if (checkedValue === undefined || onChangeCallback === undefined) {
        return null
    }

    return <div className={`toggle-container ${checkedValue ? "" : "disabled"}`}>
        <label>{labelName}</label>
        <InputSwitch checked={checkedValue}
                     tooltip={tooltipText}
                     onChange={(e) => onChangeCallback(e.value)
                     }/>
    </div>
}

function filterReservations(reservations, visibleStartTime, reservationFilter, projectFilter) {
    let reservationItems = reservations.flatMap(reservation => splitObjectIfSpanIsMultipleDays(reservation, false))
        .map((reservation, index) => getReservationItem(reservation, index, moment(visibleStartTime).format(UIConstants.CALENDAR_DEFAULTDATE_FORMAT)))
        .filter(reservationItem => reservationFilter.includes(reservationItem.reason))

    if (projectFilter.length) {
        reservationItems = reservationItems.filter(reservationItem => reservationItem.project === null || projectFilter.includes(reservationItem.project))
    } else { //only show items that don't have a project
        reservationItems = reservationItems.filter(reservationItem => reservationItem.project === null)
    }
    return reservationItems;
}


function filterSchedulingUnits(schedulingUnits, visibleStartTime, onSkyToggle, schedulingUnitFilter, projectFilter) {
    let schedulingUnitItems = schedulingUnits.flatMap(su => splitObjectIfSpanIsMultipleDays(su, onSkyToggle))
        .map(su => getTimelineItem(su, moment(visibleStartTime), onSkyToggle))
        .filter(suItem => schedulingUnitFilter.includes(suItem.status))

    if (projectFilter?.length) {
        schedulingUnitItems = schedulingUnitItems.filter(suItem => suItem.project === null || projectFilter.includes(suItem.project))
    } else {//only show items that don't have a project
        schedulingUnitItems = schedulingUnitItems.filter(suItem => suItem.project === null)
    }
    return schedulingUnitItems;
}

export default function Filters(props) {
    const {
        visibleStartTime,
        setItems,
        data,
        setRefetchToggle,
        timelineStore,
    } = props

    const [multiSelectAllOptions, setMultiSelectAllOptions] = useState({
        isLoading: false,
        reservationReasons: [],
        schedulingUnitStatuses: [],
        projectStatuses: [],
        projectNamesWithState: []
    })

    const [onSkyToggle, setOnSkyToggle] = useState(timelineStore.onSkyToggle ?? true)

    const [reservationsToggle, setReservationsToggle] = useState(timelineStore.reservationsToggle ?? false)
    const [reservationFilter, setReservationFilter] = useState(timelineStore.reservationFilter)

    const [schedulingUnitFilter, setSchedulingUnitFilter] = useState(timelineStore.schedulingUnitFilter)

    const [projectFilter, setProjectFilter] = useState(timelineStore.projectFilter)
    const [projectAllOptionsWithStatus, setProjectAllOptionsWithStatus] = useState([])
    const [projectStatusFilter, setProjectStatusFilter] = useState(timelineStore.projectStatusFilter ?? ["active"])


    useEffect(() => { //initializer
        fetchAllOptionsForMultiSelects(setMultiSelectAllOptions)
            .catch(e => console.error("Couldn't retrieve the multi select options for at least one multi select", e))
    }, [])

    useEffect(() => { //set default filter to all options if none in store
        if (!schedulingUnitFilter?.length) {
            setSchedulingUnitFilter(multiSelectAllOptions.schedulingUnitStatuses.map(status => status.name))
        }

        if (!reservationFilter?.length) {
            setReservationFilter(multiSelectAllOptions.reservationReasons.map(reason => reason.name))
        }

        const filteredProjectsOnState = multiSelectAllOptions.projectNamesWithState.filter(project => projectStatusFilter.includes(project.project_state_value))
        setProjectAllOptionsWithStatus(filteredProjectsOnState)
        setProjectFilter(filteredProjectsOnState.map(project => project.name))
    }, [multiSelectAllOptions])

    useEffect(() => { //the status filter is only for the project filter
        updateStore(UIConstants.STORE_KEY_TIMELINE, timelineStore, {
            ["projectStatusFilter"]: projectStatusFilter,
        })
        const filteredProjectsOnState = multiSelectAllOptions.projectNamesWithState.filter(project => projectStatusFilter.includes(project.project_state_value))
        setProjectAllOptionsWithStatus(filteredProjectsOnState)
        setProjectFilter(filteredProjectsOnState.map(project => project.name))
    }, [projectStatusFilter])

    useEffect(() => { //since reservations loading is slow and often off, only fetch them om toggle
        updateStore(UIConstants.STORE_KEY_TIMELINE, timelineStore, {
            ["reservationsToggle"]: reservationsToggle,
        })

        if (reservationsToggle) {
            setRefetchToggle(prevState => !prevState)
        } else {
            setItems(prevState => ({
                ...prevState,
                reservationItems: []
            }))
        }
    }, [reservationsToggle])

    useEffect(() => {
        updateStore(UIConstants.STORE_KEY_TIMELINE, timelineStore, {
            ["reservationFilter"]: reservationFilter,
            ["projectFilter"]: projectFilter,
            ["onSkyToggle"]: onSkyToggle,
        })

        let reservationsItems = []
        if (reservationsToggle) {
            reservationsItems = filterReservations(data.reservations, visibleStartTime, reservationFilter, projectFilter);
        }

        setItems(prevState => ({
            ...prevState,
            reservationItems: reservationsItems
        }))
    }, [data, reservationFilter, projectFilter])

    useEffect(() => {
        updateStore(UIConstants.STORE_KEY_TIMELINE, timelineStore, {
            ["schedulingUnitFilter"]: schedulingUnitFilter,
            ["projectFilter"]: projectFilter,
            ["onSkyToggle"]: onSkyToggle,
        })

        const schedulingUnitItems = filterSchedulingUnits(data.schedulingUnits, visibleStartTime, onSkyToggle, schedulingUnitFilter, projectFilter);

        setItems(prevState => ({
            ...prevState,
            schedulingUnitItems: schedulingUnitItems
        }))
    }, [data, schedulingUnitFilter, projectFilter, onSkyToggle])

    return <div className="p-grid timeline-view-toolbar">
        {multiSelectAllOptions.isLoading ? <ProgressSpinner  className="m-5" style={{color: "var(--primary-300)"}}/> :
            <div className="section">
                <div className="header">Filters</div>
                <div className="group">
                    <Toggle labelName="On Sky" tooltipText="Show on-sky system durations"
                            checkedValue={onSkyToggle} onChangeCallback={setOnSkyToggle}/>
                </div>
                <div className="group">
                    <Toggle labelName={"Reservations"} tooltipText={"Show station reservations"}
                            checkedValue={reservationsToggle} onChangeCallback={setReservationsToggle}/>
                    <div className="multiselect-container relative-position">
                        <span className="small-label">Reason:</span>
                        <MultiSelect
                            className={!reservationsToggle ? "disabled" : ""}
                            disabled={!reservationsToggle}
                            optionLabel="name"
                            optionValue="name"
                            tooltip="Select Reservation Reason(s)"
                            tooltipOptions={{position: 'left'}}
                            value={reservationFilter}
                            options={multiSelectAllOptions.reservationReasons}
                            maxSelectedLabels="1"
                            filter
                            filterBy="name"
                            placeholder="Reason"
                            onChange={(e) => setReservationFilter(e.value)}
                        />
                    </div>
                </div>
                <div className="group">
                    <label>Scheduling Units</label>
                    <div className="multiselect-container relative-position">
                        <span className="small-label">Status:</span>
                        <MultiSelect
                            optionLabel="name"
                            optionValue="name"
                            tooltip="Select Scheduling Unit Status(es)"
                            tooltipOptions={{position: 'left'}}
                            value={schedulingUnitFilter}
                            options={multiSelectAllOptions.schedulingUnitStatuses}
                            maxSelectedLabels="1"
                            filter
                            filterBy="name"
                            placeholder="Status"
                            onChange={(e) => setSchedulingUnitFilter(e.value)}
                        />
                    </div>
                </div>

                <div className="group">
                    <label>Projects</label>
                    <div className="multiselect-container relative-position">
                        <span className="small-label">Status:</span>
                        <MultiSelect
                            optionLabel="value"
                            optionValue="value"
                            tooltip="Select Project Status(es)"
                            tooltipOptions={{position: 'left'}}
                            value={projectStatusFilter}
                            options={multiSelectAllOptions.projectStatuses}
                            maxSelectedLabels="1"
                            filter
                            filterBy="value"
                            placeholder="Status"
                            onChange={(e) => setProjectStatusFilter(e.value)}
                        />
                    </div>
                    <div className="multiselect-container relative-position">
                        <span className="small-label">Value:</span>
                        <MultiSelect
                            optionLabel="name"
                            optionValue="name"
                            tooltip="Select Project Name(s)"
                            tooltipOptions={{position: 'left'}}
                            value={projectFilter}
                            options={projectAllOptionsWithStatus}
                            maxSelectedLabels="1"
                            filter
                            filterBy="name"
                            placeholder="Project"
                            onChange={(e) => setProjectFilter(e.value)}
                        />
                    </div>
                </div>
            </div>
        }
    </div>
}