import React, { useEffect, useState } from "react";
import { updateStore } from "../../../../services/store.helper";
import { ProgressSpinner } from 'primereact/progressspinner';
import UIConstants from "../../../../utils/ui.constants";
import { Button } from "primereact/button";
import { Dropdown } from "primereact/dropdown";
import {
    getMovePossibilities,
    getTimeStepsForZoom,
    getZoomPossibilities,
    getZoomTimes,
    moveTimeline
} from "../../helpers/toolbar/zoomandmove.helper";
import moment from "moment/moment";
import Flatpickr from "react-flatpickr";

function IconButton(props) {  //TODO: extract to separate class
    const {
        title,
        onClickCallback,
        iconClassName,
        disabled = false
    } = props
    let tooltipText = disabled ? "Maximum reached. Cannot " + title : title
    return <button className="p-link"
        title={tooltipText}
        onClick={onClickCallback}
        disabled={disabled}
        data-testid={title}>
        <i className={`pi ${iconClassName}`} />
    </button>
}

function ZoomAndMoveActions(props) {  //TODO: extract to separate class
    const {
        zoomLevelName,
        setZoomLevelName,
        visibleTime,
        setVisibleTime,
        timeSteps
    } = props

    const zoomLevelIndex = UIConstants.ALL_ZOOM_LEVELS.findIndex(level => level.name === zoomLevelName);
    const zoomPossibilities = getZoomPossibilities(zoomLevelIndex)
    const movePossibilities = getMovePossibilities(visibleTime)

    let timeStepMinuteAmount = 30
    let titleInfo = "30 minutes";
    if (timeSteps) {
        if (timeSteps.value.hour === 0) {
            timeStepMinuteAmount = timeSteps.value.minute
            titleInfo = timeStepMinuteAmount + " minute(s)"
        } else {
            const timeStepAmount = timeSteps.value.hour
            titleInfo = timeStepAmount + " hour"
            timeStepMinuteAmount = timeStepAmount * 60
        }
    }


    return <div className="move-container">
        <label>Move</label>
        <div>
            <IconButton title={"Move Left " + titleInfo}
                onClickCallback={() => moveTimeline(setVisibleTime, -1 * timeStepMinuteAmount)}
                iconClassName={"pi-angle-left"}
                disabled={!movePossibilities.moveLeft} />
            <IconButton title={"Zoom out"}
                onClickCallback={() => setZoomLevelName(UIConstants.ALL_ZOOM_LEVELS[zoomLevelIndex + 1].name)}
                iconClassName={"pi-minus-circle"}
                disabled={!zoomPossibilities.canZoomOut} />
            <IconButton title={"Zoom in"}
                onClickCallback={() => setZoomLevelName(UIConstants.ALL_ZOOM_LEVELS[zoomLevelIndex - 1].name)}
                iconClassName={"pi-plus-circle"}
                disabled={!zoomPossibilities.canZoomIn} />
            <IconButton title={"Move Right " + titleInfo}
                onClickCallback={() => moveTimeline(setVisibleTime, timeStepMinuteAmount)}
                iconClassName={"pi-angle-right"}
                disabled={!movePossibilities.moveRight} />
        </div>
    </div>
}

function ZoomSelect(props) {   //TODO: extract to separate class
    const {
        currentZoomLevelName,
        allOptions,
        setZoomLevelName
    } = props

    return <div className="zoom-selector-container" data-testid="zoom-select">
        <label title="Set the amount of time surrounding the current time">Span</label>
        <div>
            <Dropdown optionLabel="name" optionValue="name"
                value={currentZoomLevelName}
                options={allOptions}
                filter
                showClear={false}
                filterBy="name"
                onChange={(e) => setZoomLevelName(e.value)}
                placeholder="Zoom" />
        </div>
    </div>
}


function TimeStepsSetter(props) {  //TODO: extract to separate class
    const {
        currentTimeSteps,
        allOptions,
        setHeaderSettings,
        timelineStore
    } = props

    return <div className="timesteps-container" data-testid="time-steps-setter">
        <label title="Set the time steps for the header">Time steps</label>
        <div>
            <Dropdown optionLabel="name" optionValue="name"
                value={currentTimeSteps.name}
                options={allOptions}
                filter
                showClear={false}
                filterBy="name"
                onChange={(e) => {
                    const timeStepsValue = UIConstants.ALL_TIMESTEPS.filter(timeSteps => timeSteps.name === e.value)[0].value
                    const timeStepsObj = { name: e.value, value: timeStepsValue };
                    const unitForTimeSteps = timeStepsValue.hour === 0 || timeStepsValue.days === 0 ? "minute" : "hour";
                    if (setHeaderSettings !== undefined)
                        setHeaderSettings(prevState => ({
                            ...prevState,
                            timeSteps: timeStepsObj,
                            unit: unitForTimeSteps
                        }))
                    updateStore(UIConstants.STORE_KEY_TIMELINE, timelineStore, {
                        timeSteps: timeStepsObj,
                        headerUnit: unitForTimeSteps
                    })
                }}
                placeholder="Time Steps" />
        </div>
    </div>
}

function TimeSetter(props) {  //TODO: extract to separate class
    const {
        zoomTime,
        setSelectedTime,
        zoomLevelName
    } = props
    const zoomLevelIndex = UIConstants.ALL_ZOOM_LEVELS.findIndex(level => level.name === zoomLevelName);
    const isMinZoom = UIConstants.ALL_ZOOM_LEVELS_WEEK.length - 1 === zoomLevelIndex;
    const options = {
        "allowInput": true,
        "enableSeconds": false,
        "enableTime": true,
        "time_24hr": true,
        "noCalendar": true,
        "dateFormat": "H:i",
        "mode": "single",
        "minuteIncrement": 1,
        "defaultHour": 0,
        "wrap": true,
        "inlineHideInput": true,
    }
    return <div data-testid="zoom-time-selector">
        <label>Set time</label>
        <Flatpickr data-enable-time={true}
            data-input
            value={zoomTime}
            placeholder="hi"
            options={options}
            onClose={value => setSelectedTime(moment(value[0]).format("HH:mm"))}>
            <input type="text"
                data-input
                className="p-inputtext p-component time-input"
                placeholder="Input time"
                disabled={isMinZoom} />
            <Button className="p-button-icon-only calendar-button"
                data-testid="time-reset-button"
                disabled={isMinZoom}
                onClick={() => setSelectedTime(undefined)}
                title="Reset to the current time and default time steps for span">
                <i className="fas fa-undo"></i>
            </Button>
        </Flatpickr>
    </div>
}


export default function ZoomAndMove(props) {
    const {
        timelineStore,
        startTime,
        visibleTime,
        setVisibleTime,
        headerSettings,
        setHeaderSettings
    } = props

    const [zoomSettings, setZoomSettings] = useState({
        zoomLevelName: timelineStore.zoomLevelName ?? UIConstants.DEFAULT_ZOOM_LEVEL.name,
        selectedTime: timelineStore.selectedTime,
    });

    useEffect(() => { //initializer
        if (zoomSettings.zoomLevelName) {
            const zoomLevel = UIConstants.ALL_ZOOM_LEVELS_WEEK.find(zoomLevel => zoomLevel.name === zoomSettings.zoomLevelName)
            let zoomTimes = getZoomTimes(zoomLevel, zoomSettings.selectedTime, startTime);
            if (setVisibleTime !== undefined) {
                setVisibleTime({
                    start: zoomTimes.start,
                    end: zoomTimes.end,
                });
            }
        } else {
            setVisibleTime({
                start: moment(startTime).startOf('day'),
                end: moment(startTime).endOf('day')
            })
        }
    }, [])

    useEffect(() => {
        updateStore(UIConstants.STORE_KEY_TIMELINE, timelineStore, {
            ["zoomLevelName"]: zoomSettings.zoomLevelName,
            ["selectedTime"]: zoomSettings.selectedTime,
        });
        const selectedZoomLevel = UIConstants.ALL_ZOOM_LEVELS.find(
            (level) => level.name === zoomSettings.zoomLevelName
        );

        if (selectedZoomLevel && setVisibleTime !== undefined) {
            let zoomTimes = getZoomTimes(selectedZoomLevel, zoomSettings.selectedTime, startTime);
            setVisibleTime({
                start: zoomTimes.start,
                end: zoomTimes.end,
            });
            if (!zoomSettings.selectedTime) {
                const timeStepsForZoom = getTimeStepsForZoom(selectedZoomLevel);
                const unitForTimeSteps = timeStepsForZoom.value.hour === 0 || timeStepsForZoom.value.days === 0 ? "minute" : "hour";
                if (setHeaderSettings !== undefined)
                    setHeaderSettings(prevState => {
                        return ({
                            ...prevState,
                            timeSteps: timeStepsForZoom,
                            unit: unitForTimeSteps
                        });
                    })
                updateStore(UIConstants.STORE_KEY_TIMELINE, timelineStore, {
                    ["zoomLevelName"]: zoomSettings.zoomLevelName,
                    ["selectedTime"]: zoomSettings.selectedTime,
                    ["timeSteps"]: timeStepsForZoom,
                    ["headerUnit"]: unitForTimeSteps
                });
            }
        }
    }, [zoomSettings]);

    if (setVisibleTime === undefined || setHeaderSettings === undefined) {
        return <div className="group group--row">
            <ProgressSpinner className="m-4" style={{ color: "var(--primary-300)" }} data-testid="zoom-loading-spinner" />
        </div>
    }

    return <div className="timeline-zoom-and-move section">
        <div className="header">
            Zoom
            <Button icon="fas fa-undo"
                className="p-button p-button-primary reset-button"
                onClick={() => setZoomSettings({
                    zoomLevelName: UIConstants.DEFAULT_ZOOM_LEVEL.name,
                    selectedTime: undefined
                })}
                title="Reset span to 1 day, time steps to 1 hour and time to now."
                data-testid="zoom-reset-button"
            />
        </div>
        <div className="group group--row">
            <ZoomSelect currentZoomLevelName={zoomSettings.zoomLevelName}
                allOptions={UIConstants.ALL_ZOOM_LEVELS_WEEK}
                setZoomLevelName={(newZoomLevelName) => {
                    setZoomSettings((prevState) => ({
                        ...prevState,
                        zoomLevelName: newZoomLevelName,
                    }));
                }} />
            <TimeSetter zoomTime={zoomSettings.selectedTime}
                setSelectedTime={(newSelectedTime) => {
                    setZoomSettings((prevState) => ({
                        ...prevState,
                        selectedTime: newSelectedTime,
                    }))
                }}
                zoomLevelName={zoomSettings.zoomLevelName} />
        </div>
        <div className="group">
            <TimeStepsSetter currentTimeSteps={headerSettings.timeSteps}
                allOptions={UIConstants.ALL_TIMESTEPS}
                setHeaderSettings={setHeaderSettings}
                timelineStore={timelineStore} />
            <ZoomAndMoveActions zoomLevelName={zoomSettings.zoomLevelName}
                setZoomLevelName={(newZoomLevelName) => {
                    setZoomSettings((prevState) => ({
                        ...prevState,
                        zoomLevelName: newZoomLevelName,
                    }));
                }}
                visibleTime={visibleTime}
                setVisibleTime={setVisibleTime}
                timeSteps={headerSettings.timeSteps} />
        </div>
    </div>
}