import UIConstants from "../../../utils/ui.constants";
import ScheduleService from "../../../services/schedule.service";
import ReservationService from "../../../services/reservation.service";
import AuthUtil from "../../../utils/auth.util";
import UtilService from "../../../services/util.service";
import moment from "moment/moment";
import TaskService from "../../../services/task.service";
import {getSunTimingItems} from "../helpers/week.view.helper";
import TimelineCommonUtils from "../common.utils";
import {getStore} from "../../../services/store.helper";

export async function fetchTimelineData(startTime, endTime, setData) {
    const from = startTime.format(UIConstants.CALENDAR_DATETIME_FORMAT);
    const until = endTime.format(UIConstants.CALENDAR_DATETIME_FORMAT);

    const schedulingUnits = await ScheduleService.getTimelineSlimBlueprints(from, until);
    const reservations = await fetchReservations(from, until)

    setData({
        schedulingUnits: schedulingUnits,
        reservations: reservations
    })
}

export async function fetchReservations(formattedStartTime, formattedEndTime) {
    let reservations = []
    const shouldFetchReservations = getStore(UIConstants.STORE_KEY_TIMELINE).reservationsToggle;
    if (shouldFetchReservations) {
        reservations = await ReservationService.getTimelineReservations(formattedStartTime, formattedEndTime);
    }
    return reservations
}

export async function fetchUserPermissions(setPermissions) {
    const permission = await AuthUtil.getUserRolePermission();
    setPermissions({
        weekView: permission.userRolePermission.weekoverview,
        schedulingUnit: permission.userRolePermission.scheduleunit
    })
}

export async function fetchSchedulerStatuses(setSchedulerSettings) {
    let isDynamicSchedulerOn, isFixedTimeSchedulerOn, isDynamicSchedulerActive
    await UtilService.getDynamicSchedulerStatus()
        .then(response => isDynamicSchedulerOn = response ? response.value : false)
    await UtilService.getFixedTimeSchedulerStatus()
        .then(response => isFixedTimeSchedulerOn = response ? response.value : false)
    await UtilService.getSubSystemScheduler()
        .then(response => isDynamicSchedulerActive = response ? response.status_value === "active" : false)
    setSchedulerSettings({
        isDynamicSchedulerOn: isDynamicSchedulerOn,
        isFixedTimeSchedulerOn: isFixedTimeSchedulerOn,
        isDynamicSchedulerActive: isDynamicSchedulerActive
    })
}

export async function fetchSunTimings(startTime, setItems) {
    const sunTimingItems = await getSunTimingItems(startTime)
    setItems(prevState => ({
        ...prevState,
        sunTimingsItems: sunTimingItems
    }))
}

export async function fetchLSTShift(startTime, setHeaderSettings) {
    const UTCTimeNow = moment(startTime).format(UIConstants.UTC_DATE_TIME_FORMAT);
    let lst = await UtilService.getLST(UTCTimeNow)
    const hoursMinutesSeconds = lst.split(".")[0].split(":").map(stringTimeElement => parseInt(stringTimeElement))
    const LSTTimeNow = moment().set({
        hour: hoursMinutesSeconds[0],
        minute: hoursMinutesSeconds[1],
        second: hoursMinutesSeconds[2]
    })
    const shift = moment.duration(LSTTimeNow.diff(UTCTimeNow)).asSeconds();
    setHeaderSettings(prevState => ({
        ...prevState,
        lstShiftInSeconds: shift
    }))
}

async function addNestedBlueprintInformation(suBluePrint) {
    let missingStations = []
    for (let taskIndex = 0; taskIndex < suBluePrint.task_blueprints.length; taskIndex++) {
        const task = suBluePrint.task_blueprints[taskIndex]
        let subtasksSpecifications = []
        for (const subtaskId of task.subtasks_ids) {
            subtasksSpecifications.push(await TaskService.getSubtaskDetails(subtaskId, "specifications_doc"))
            const foundMissingStations = await TaskService.getSubtaskMissingStations(subtaskId);
            missingStations = missingStations.concat(foundMissingStations.filter((item) => missingStations.indexOf(item) < 0))
        }
        suBluePrint.task_blueprints[taskIndex].subtasks = subtasksSpecifications
    }
    const timelineCommonUtils = new TimelineCommonUtils();
    suBluePrint.stations = timelineCommonUtils.getSUStations(suBluePrint);
    suBluePrint.missingStations = missingStations

    const projectInfo = await ScheduleService.getSchedulingSet(suBluePrint.draft.scheduling_set_id, "project_id")
    suBluePrint.project = projectInfo.project_id;
}

export async function fetchSUSummaryInformation(id, setSummarySettings) {
    let suBluePrint = await ScheduleService.getBlueprintByschedulingUnitId(id, "task_blueprints,draft")
    await addNestedBlueprintInformation(suBluePrint);

    const suConstraintTemplate = await ScheduleService.getSchedulingConstraintTemplateById(suBluePrint.scheduling_constraints_template_id)

    setSummarySettings(prevState => ({
        ...prevState,
        isLoading: false,
        schedulingUnitItem: suBluePrint,
        summaryConstraintTemplate: suConstraintTemplate
    }))

}

export async function fetchReservationSummaryInformation(id, setSummarySettings) {
    let reservation = await ReservationService.getReservation(id)
    setSummarySettings(prevState => ({
        ...prevState,
        isLoading: false,
        reservationItem: reservation,
    }))
}