import moment from "moment";
import UIConstants from "../../../../utils/ui.constants";
import UnitConverter from "../../../../utils/unit.converter";

function determineNewStartAndEnd(startTime, day, spanInDays, endTime) {
    let start = moment(moment(startTime).add(day, "days"));
    let end = moment(start).endOf('day');
    if (day !== 0) { //not the actual start
        start = moment(start).startOf('day')
    }
    if (day === spanInDays) { //the actual end
        end = moment(endTime)
    }

    const formattedStart = start.format(UIConstants.UTC_DATE_TIME_FORMAT);
    const formattedEnd = end.format(UIConstants.UTC_DATE_TIME_FORMAT);
    return {formattedStart, formattedEnd};
}

function splitElementPerDay(element, spanInDays, shouldShowOnSkyTimes) {
    const newElements = []
    for (let day = 0; day <= spanInDays; day++) {
        let newElement = {...element}
        const startTime = (shouldShowOnSkyTimes ? (element.on_sky_start_time ?? element.process_start_time) : element.process_start_time) ?? element.start_time
        const endTime = (shouldShowOnSkyTimes ? (element.on_sky_stop_time ?? element.process_stop_time) : element.process_stop_time) ?? element.stop_time

        const {formattedStart, formattedEnd} = determineNewStartAndEnd(startTime, day, spanInDays, endTime);

        if (shouldShowOnSkyTimes) {
            newElement.on_sky_start_time = formattedStart
            newElement.on_sky_stop_time = formattedEnd
        } else {
            if (element.process_start_time) { //it is a SU unit with process start/stop times
                newElement.process_start_time = formattedStart
                newElement.process_stop_time = formattedEnd
            } else { //it is a reservation with only start/stop times
                newElement.start_time = formattedStart
                newElement.stop_time = formattedEnd
            }
        }

        newElements.push(newElement)
    }
    return newElements
}

/**
 * The timeline expects items per day but the data can span multiple days.
 * @param element
 * @param shouldShowOnSkyTimes determines whether to use the process times or on sky times
 * @return [elements] (nested thus use flatMap instead of map)
 */
export function splitObjectIfSpanIsMultipleDays(element, shouldShowOnSkyTimes) {
    const startTime = (shouldShowOnSkyTimes ? (element.on_sky_start_time ?? element.process_start_time) : element.process_start_time) ?? element.start_time
    if (!startTime) {
        throw new ReferenceError("(On Sky) start time is not defined for:\n" + JSON.stringify(element))
    }
    element.real_start_time = moment(startTime).format(UIConstants.CALENDAR_DATETIME_FORMAT)

    const stopTime = (shouldShowOnSkyTimes ? (element.on_sky_stop_time ?? element.process_stop_time) : element.process_stop_time) ?? element.stop_time
    element.real_end_time = "Unknown"

    let spanInDays = 7 //initial amount of days if stop time is unknown
    if (stopTime) {
        element.real_end_time = moment(stopTime).format(UIConstants.CALENDAR_DATETIME_FORMAT)
        spanInDays = moment(stopTime).startOf('day').diff(moment(startTime).startOf('day'), 'days')
    }
    if (isNaN(spanInDays)) {
        throw new TypeError("Start and/or stop times are not invalid for:\n" + JSON.stringify(element))
    }

    if (spanInDays) {
        return splitElementPerDay(element, spanInDays, shouldShowOnSkyTimes)
    }
    return [element]
}

function getGroupCount(stations) {
    const counts = stations.reduce((result, station) => {
        const stationPrefix = station.substring(0, 2);
        result[stationPrefix] = (result[stationPrefix] || 0) + 1;
        return result;
    }, {});
    return "CS/RS/IS : "
        + (counts["CS"] ?? "0") + "/"
        + (counts["RS"] ?? "0") + "/"
        + (counts["IS"] ?? "0")
}

function getStationsInfo(stations) {
    let stationsInfo = "-"
    if (stations?.length) {
        if (stations.length > 3) {
            stationsInfo = getGroupCount(stations);
        } else {
            stationsInfo = stations.toString()
        }
    }
    return stationsInfo
}

/**
 * @param suBlueprint
 * @param displayDate the selected date for which this SU has a start and end time
 * @param shouldShowOnSkyTimes start and end times differ for original and on sky system durations
 * @return {{on_sky_duration: string, placed: (boolean|*), end_time: string<{moment}>, project, suId, title: string,
 * type: string, scheduleMethod, duration: (string|string), start_time: string<{moment}>, bgColor: (*|string),
 * unschedulable_reason, antennaSet: string, name, id: string, band: (string|string), selectedBgColor: string,
 * group: string, status: string | undefined}}
 */
export function getTimelineItem(suBlueprint, displayDate, shouldShowOnSkyTimes = false) {
    const duration = `${(UnitConverter.getSecsToHrsWithFractionDigits(suBlueprint.duration))}Hrs`;
    const onSkyDuration = `${(UnitConverter.getSecsToHrsWithFractionDigits(suBlueprint.on_sky_duration))}Hrs`;
    const startTimeWithDisplayDate = moment(`${displayDate.format('YYYY-MM-DD')} ${suBlueprint.process_start_time.split('T')[1]}`);
    const endTimeWithDisplayDate = moment(`${displayDate.format('YYYY-MM-DD')} ${suBlueprint.process_stop_time.split('T')[1]}`)
    const onSkyStartTimeWithDisplayDate = moment(`${displayDate.format('YYYY-MM-DD')} ${suBlueprint.on_sky_start_time.split('T')[1]}`);
    const onSkyEndTimeWithDisplayDate = moment(`${displayDate.format('YYYY-MM-DD')} ${suBlueprint.on_sky_stop_time.split('T')[1]}`)
    const band = suBlueprint.antenna_set?.split("_")[0]  //Example: antenna sets LBA_OUTER -> band = LBA
    const currentGroup = moment(shouldShowOnSkyTimes ? suBlueprint.on_sky_start_time : suBlueprint.process_start_time).format(UIConstants.CALENDAR_GROUP_FORMAT)
    const stations = getStationsInfo(suBlueprint.stations);
    return {
        suId: suBlueprint.id,
        project: suBlueprint.project,
        name: suBlueprint.name,
        antenna_set: suBlueprint.antenna_set,
        scheduler: suBlueprint.scheduler,
        duration: duration,
        on_sky_duration: onSkyDuration,
        start_time: shouldShowOnSkyTimes ? onSkyStartTimeWithDisplayDate : startTimeWithDisplayDate,
        end_time: shouldShowOnSkyTimes ? onSkyEndTimeWithDisplayDate : endTimeWithDisplayDate,
        stations: stations,
        desc: suBlueprint.description,
        status: suBlueprint.status?.toLowerCase(),
        band: band,
        unschedulable_reason: suBlueprint.unschedulable_reason,
        id: `${suBlueprint.id}-${currentGroup}`,
        title: "",
        group: currentGroup,
        type: 'SCHEDULE',
        //for item pop-over
        real_start_time: suBlueprint.real_start_time,
        real_end_time: suBlueprint.real_end_time,
    };
}

/**
 * @param reservation note: if the reservation doesn't have duration, item end time should be the week's end time.
 * @param index when the data is split into multiple timeline items, add the index to the id (unique keys)
 * @param displayDate the selected date for which this reservation has a start and end time
 * @return {{color, end_time: moment.Moment, project, type: string, title: string,
 * duration: (string|*|string), start_time: moment.Moment, bgColor, name, actType: string,
 * id: string, selectedBgColor, group: string, desc}}
 */
export function getReservationItem(reservation, index, displayDate) {
    const specificationsDoc = reservation.specifications_doc;
    const startTime = moment(displayDate + ` ${moment(reservation.start_time).format('HH:mm:ss')}`);
    const endTime = moment(displayDate + ` ${moment(reservation.stop_time).format('HH:mm:ss')}`);
    const type = specificationsDoc?.activity?.type;
    const group = moment(reservation.start_time).format(UIConstants.CALENDAR_GROUP_FORMAT);
    const blockColor = getReservationTypeColorIndex(specificationsDoc?.schedulability);
    const duration = reservation.duration ? UnitConverter.getSecsToHHmmss(reservation.duration) : "Unknown";
    const stations = getStationsInfo(specificationsDoc?.resources?.stations);
    return {
        id: reservation.id + "~" + index,
        reservationId: reservation.id,
        start_time: startTime,
        end_time: endTime,
        group: group,
        name: reservation.name,
        project: reservation.project_id,
        type: 'RESERVATION',
        activity_type: `${type}${reservation.project_id ? (" - " + reservation.project_id) : ""}`,
        reason: type,
        title: '',
        stations: stations,
        desc: reservation.description,
        duration: duration,
        planned: specificationsDoc?.activity?.planned,
        //for item pop-over
        real_start_time: reservation.real_start_time ? reservation.real_start_time : moment(reservation.start_time).format(UIConstants.CALENDAR_DATETIME_FORMAT),
        real_end_time: reservation.real_end_time ? reservation.real_end_time : moment(reservation.stop_time).format(UIConstants.CALENDAR_DATETIME_FORMAT),
        bgColor: blockColor.bgColor,
        selectedBgColor: blockColor.bgColor,
        color: blockColor.color
    };
}


/**
 * Get the colors of the reservation blocks based on the schedulability's type
 * according to the type.
 * @param {Object} schedulability
 */
export function getReservationTypeColorIndex(schedulability) {
    const reservationColorLookup = {
        'fixed_time-dynamic': {bgColor: "lightgrey", color: "#585859"},
        'fixed_time-not_dynamic': {bgColor: '#585859', color: "white"},
        'not_fixed_time-dynamic': {bgColor: "#9b9999", color: "white"},
        'other': {bgColor: "black", color: "white"}
    };

    const key =
        (schedulability.fixed_time ? 'fixed_time' : 'not_fixed_time') +
        '-' +
        (schedulability.dynamic ? 'dynamic' : 'not_dynamic');

    return reservationColorLookup[key] || reservationColorLookup['other'];
}
