import moment from "moment/moment";
import UIConstants from "../../../utils/ui.constants";
import UtilService from "../../../services/util.service";

export function createGroups(startTime, amountOfGroups = 7) {
    const groups = []
    const startDate = moment(startTime).format(UIConstants.CALENDAR_DEFAULTDATE_FORMAT)
    for (let groupIndex = 0; groupIndex < amountOfGroups; groupIndex++) {
        const group = moment(startDate).add(groupIndex, "days");
        const groupTitle = group.format(UIConstants.CALENDAR_GROUP_FORMAT)
        groups.push({id: groupTitle, title: groupTitle})
    }
    return groups
}

const MAX_COLUMNS_WHOLE_PAGE = 12;

function getClassName(size) {
    if (size === 0) {
        return "hide-element"
    }
    if (size === MAX_COLUMNS_WHOLE_PAGE) { //max columns for a whole page; so no columns needs to be set
        return ""
    }
    return `col-lg-${size} col-md-${size} col-sm-${size}`
}

export function calculateSizeAndSetPanelClassName(showLeft, showRight, leftPanelSize, setPanelClassNames) {
    if (!showLeft) {
        leftPanelSize = 0
    }
    if (showRight) { //set it to a fixed column size (resizing is not possible)
        leftPanelSize = 10
    }

    const mapForSize = UIConstants.PANEL_SIZE_CLASS_MAP.find(panelMapper => panelMapper.size === leftPanelSize);
    const leftSize = showLeft ? mapForSize.left : 0;
    const rightSize = showRight ? mapForSize.right : 0;
    const middleSize = MAX_COLUMNS_WHOLE_PAGE - leftSize - rightSize
    setPanelClassNames({
        leftPanelClassName: "timeline-panel " + getClassName(leftSize),
        middlePanelClassName: "timeline-panel " + getClassName(middleSize),
        rightPanelClassName: "timeline-panel " + getClassName(rightSize),
    })
}

export async function updateSchedulerStatuses(switchValue, status, setSchedulerSettings) {
    if (status === "fixed") {
        setSchedulerSettings(prevState => ({
            ...prevState,
            isFixedTimeSchedulerOn: switchValue
        }))
    }
    if (status === "dynamic") {
        let isDynamicSchedulerActive = false
        if (switchValue) {
            await UtilService.getSubSystemScheduler()
                .then(response => isDynamicSchedulerActive = response ? response.status_value === "active" : false);
        }
        setSchedulerSettings(prevState => ({
            ...prevState,
            isDynamicSchedulerOn: switchValue,
            isDynamicSchedulerActive: isDynamicSchedulerActive
        }))
    }
}

const actionObject = (title, icon, classes, className, callbackProp, content, type, actOn) => ({
    title: title,
    icon: icon,
    classes: classes,
    className: className,
    content: content,
    type: type,
    actOn: actOn,
    props: callbackProp
})

const actionButtonObject = (title, icon, classes, callbackProp) => (
    actionObject(title, icon, classes, undefined, callbackProp, undefined, "button", "click")
)

const actionTagObject = (title, content, className) => (
    actionObject(title, undefined, undefined, className, undefined, content, "tag", "click")
)

export function getHeaderActions(isLoading, setRefetchToggle, actionsMenuRef) {
    return [
        actionButtonObject(
            isLoading ? "System is already reloading the timeline data" : "Reload the timeline data",
            "fa-" + (isLoading ? "hourglass fa-spin" : "sync-alt"),
            `subsystem subsystem--${isLoading ? "on" : "standard"}`,
            {callback: () => setRefetchToggle(prevState => !prevState)}
        ),
        actionButtonObject(
            "Options",
            "fa-bars",
            "subsystem subsystem--standard",
            {callback: (ev) => actionsMenuRef.current.toggle(ev)}
        ),
    ];
}


export function getSchedulerActions(schedulerSettings, setShowSchedulerSettingsDialog) {
    return [
        actionTagObject(
            schedulerSettings.isDynamicSchedulerOn ? "Dynamic Scheduling is On" : "Dynamic Scheduling is Off",
            "D",
            "tag tag--" + (schedulerSettings.isDynamicSchedulerOn ? "on" : "off")
        ),
        actionTagObject(
            schedulerSettings.isFixedTimeSchedulerOn ? "Fixed time Scheduling is On" : "Fixed time Scheduling is Off",
            "F",
            "tag tag--" + (schedulerSettings.isFixedTimeSchedulerOn ? "on" : "off")
        ),
        actionButtonObject(
            (schedulerSettings.isDynamicSchedulerActive ? "Dynamic Scheduler is Active" :
                (schedulerSettings.isDynamicSchedulerOn ? "Dynamic Scheduler is Idle" : "Dynamic Scheduler is stopped")) + ". Click to change the scheduler",
            "fa-" + (schedulerSettings.isDynamicSchedulerActive ? "play" : (schedulerSettings.isDynamicSchedulerOn ? "pause" : "stop")),
            "subsystem subsystem--" + (schedulerSettings.isDynamicSchedulerActive ? "on" : (schedulerSettings.isDynamicSchedulerOn ? "paused" : "off")),
            {callback: () => setShowSchedulerSettingsDialog(true)}
        )
    ];
}

export function getPageHeaderOptionsMenuItems(permissions) { //TODO factory method
    return [
        {
            label: 'Add Reservation',
            icon: "fa fa-plus",
            disabled: permissions==null||permissions.weekView==null||!permissions.weekView.addreservation,
            command: () => window.open('/reservation/create', '_blank')
        },
        {
            label: 'Reservation List',
            icon: "fa fa-bars",
            disabled:  permissions==null||permissions.weekView==null||!permissions.weekView.listreservation,
            command: () => window.open('/reservation/list', '_blank')

        },
        {
            separator: true
        },
        {
            label: 'System Issues',
            icon: "fa fa-bars",
            disabled:  permissions==null||permissions.weekView==null||!permissions.weekView.listsystemevent,
            command: () => window.open('/systemevent/list', '_blank')
        },
        {
            label: 'Add System Event',
            icon: "fa fa-plus",
            disabled:  permissions==null||permissions.weekView==null||!permissions.weekView.addsystemevent,
            command: () => window.open('/systemevent/create', '_blank')

        },
        {
            separator: true
        },
        {
            label: 'Add Scheduling Unit',
            icon: "fa fa-plus",
            disabled:  permissions==null||permissions.schedulingUnit==null||!permissions.schedulingUnit.create,
            command: () => window.open('/schedulingunit/create', '_blank')

        },
    ];
}

export function getCursorDateTimes(cursorTimeUTC, groupIndex, groupDate, lstShiftInSeconds, setCursorDateTimes) {
    const cursorTime = moment(cursorTimeUTC).format(UIConstants.CALENDAR_TIME_FORMAT);
    if (groupIndex >= 0) {
        const cursorDateTimeUTC = moment(groupDate + " " + cursorTime)
        const shiftForGroup = {'minute': 3 * groupIndex, 'second': 45 * groupIndex};
        const cursorDateTimeLST = moment(moment(cursorDateTimeUTC).add(lstShiftInSeconds, 'second')).add(shiftForGroup)
        setCursorDateTimes({
            utc: cursorDateTimeUTC.format("HH:mm"),
            lst: cursorDateTimeLST.format("HH:mm"),
        })
    }
}

function getSunTimingItem(groupId, idPrefix, color, startTime, sunTimeStart, sunTimeEnd, fullOpacity = false) {
    return {
        id: `${idPrefix}-${groupId}`,
        group: groupId,
        title: "",
        start_time: startTime.clone().hours(sunTimeStart.hours()).minutes(sunTimeStart.minutes()).seconds(sunTimeStart.seconds()),
        end_time: startTime.clone().hours(sunTimeEnd.hours()).minutes(sunTimeEnd.minutes()).seconds(sunTimeEnd.seconds()),
        bgColor: color,
        selectedBgColor: color,
        opacity: fullOpacity ? "0%" : "50%",
        type: "SUNTIME"
    };
}

export async function getSunTimingItems(startTime) {
    let sunTimingItems = []
    for (let day = 0; day <= UIConstants.WEEKVIEW_SHOW_NUMBER_OF_DAYS; day++) {
        const groupDay = moment(moment(startTime).add(day, 'days'))

        const timings = await UtilService.getSunTimings(groupDay.format("YYYY-MM-DD"));
        const sunriseStart = moment.utc(timings.sun_rise.start);
        const sunriseEnd = moment.utc(timings.sun_rise.end);
        const sunsetStart = moment.utc(timings.sun_set.start);
        const sunsetEnd = moment.utc(timings.sun_set.end);
        const startOfDay = startTime.clone().startOf('day')
        const endOfDay = startTime.clone().endOf('day')

        const timelineGroup = moment(groupDay).format(UIConstants.CALENDAR_GROUP_FORMAT)
        const nightItemBeforeSunrise = getSunTimingItem(timelineGroup, "bef-sunrise", "grey", startOfDay, startOfDay, sunriseStart)
        const sunriseItem = getSunTimingItem(timelineGroup, "sunrise", "yellow", startTime, sunriseStart, sunriseEnd)
        const sunsetItem = getSunTimingItem(timelineGroup, "sunset", "orange", startTime, sunsetStart, sunsetEnd)
        const dayItem = getSunTimingItem(timelineGroup, "day", "white", startTime, sunriseEnd, sunsetStart, true)
        const nightItemAfterSunset = getSunTimingItem(timelineGroup, "aft-sunrise", "grey", startTime, sunsetEnd, endOfDay)

        sunTimingItems.push(nightItemBeforeSunrise, sunriseItem, dayItem, sunsetItem, nightItemAfterSunset)
    }

    return sunTimingItems;
}