import React, { useEffect, useState } from 'react';
import {Steps} from 'primereact/steps';
import PageHeader from '../../layout/components/PageHeader';
import {Toast} from 'primereact/toast';
import { Link } from 'react-router-dom';
import _ from 'lodash';
import ScheduleService from '../../services/schedule.service';
import Scheduled from './Scheduled';
import ProcessingDone from './processing.done';
import QAreporting from './qa.reporting';
import QAsos from './qa.sos';
import PIverification from './pi.verification';
import DecideAcceptance from './decide.acceptance';
import Ingesting from './ingesting';
import Done from './done';
import DataProduct from './unpin.data';
import UnitConverter from '../../utils/unit.converter';
import AppLoader from '../../layout/components/AppLoader';
import WorkflowService from '../../services/workflow.service';
import DataProductService from '../../services/data.product.service';
import TaskService from '../../services/task.service';
import AuthUtil from '../../utils/auth.util';
import UtilService from '../../services/util.service';
import NotFound from '../../layout/components/NotFound';
import { appGrowl } from '../../layout/components/AppGrowl';
import Websocket from 'react-websocket';
/* eslint-disable react-hooks/exhaustive-deps */

const RedirectionMap = {
    'wait scheduled': {
        stepValue: 1,
    },
    'wait observing': {
        stepValue: 2,
    },
    'wait processed': {
        stepValue: 2,
    },
    'qa reporting to': {
        stepValue: 3,
        process: 'qa_reporting_to'
    },
    'qa reporting sos':{
        stepValue: 4,
        process: 'qa_reporting_sos'
    },
    'pi verification': {
        stepValue: 5,
        process: 'pi_verification'
    },
    'decide acceptance': {
        stepValue: 6,
        process: 'decide_acceptance'
    },
    'wait ingest done': {
        stepValue: 7,
        process: 'ingest'
    },
    'decide unpin data':{
        stepValue: 8,
        process: 'unpin_data'
    },
    'end': {
        stepValue: 9,
        process: 'finish'
    }
 };

//Workflow Page Title 
const stepItems = [
    {label: 'Wait Scheduled'},
    {label: 'Scheduled'},
    {label: 'QA Reporting(TO)'},
    {label: 'QA Reporting(SDCO)'},
    {label: 'PI Verification'},
    {label: 'Decide Acceptance'},
    {label: 'Ingesting'},
    {label: 'Unpin Data'},
    {label: 'Done'}
];
const ALLOWED_PROJECT_ROLES = {'qa reporting to': ["shared_support", "friend_of_project", "friend_of_project_primary"],
                                'qa reporting sos': ["shared_support", "friend_of_project", "friend_of_project_primary"],
                                'pi verification': ["pi", "contact", "shared_support", "friend_of_project", "friend_of_project_primary"],
                                'decide acceptance': ["friend_of_project", "friend_of_project_primary"],
                                'unpin data': ["shared_support", "friend_of_project", "friend_of_project_primary"]
                              };

const roleTitleMap = {pi: "PI", co_i: "Co-I", contact: "Contact Author", shared_support: "Shared Support User",
                        friend_of_project: "Friend of Project", friend_of_project_primary:"Friend of Project(Primary)"};
const WorkflowView = (props) => {
    let growl;
    // const [disableNextButton, setDisableNextButton] = useState(false);
    const [loader, setLoader] = useState(false);
    const [state, setState] = useState({});
    const [currentStep, setCurrentStep] = useState();
    const [tasks, setTasks] = useState([]);
    const [showIngestTab, setShowIngestTab] = useState(false);
    const [QASUProcess, setQASUProcess] = useState();
    const [workflowTasks, setWorkflowTasks] = useState([]);
    const [currentView, setCurrentView] = useState();
    const [schedulingUnit, setSchedulingUnit] = useState();
    const [userrole, setPermissions] = useState({})
    const [projectRoles, setProjectRoles] = useState();
    const [notFound, setNotFound] = useState();
    const [showAddSystemEvent, setShowAddSystemEvent] = useState(false);
    const [skippedSteps, setSkippedSteps] = useState([]);
    const [originPath, setOriginPath] = useState("/su/workflow");
    const [websocket, setWebsocket] = useState({});
    // const [ingestTask, setInjestTask] = useState({});
    // const [QASchedulingTask, setQASchdulingTask] = useState([]);

    useEffect(() => {
        setLoader(true);
        setOriginPath(props.location.state?props.location.state.origin:"/su/workflow");
        const promises = [
            ScheduleService.getSchedulingUnitExtended('blueprint', props.match.params.id),
            UtilService.getProjectRoles()
        ]
        
        Promise.all(promises).then(responses => {
            const SUB = responses[0];
            setSchedulingUnit(SUB);
            for (let projectRole of responses[1]) {
                projectRole.displayValue = roleTitleMap[projectRole.value] || projectRole.value.toUpperCase();
            }
            setProjectRoles(responses[1]);
            setTasks(SUB.task_blueprints);
            getStatusUpdate(SUB.task_blueprints);
            getPermissions();
            // setShowIngestTab(SUB.task_blueprints.filter(i => i.specifications_template.name === 'ingest').length > 0);
        });
    }, []);

    const getPermissions = async () => {
        const permissions = await AuthUtil.getUserPermissionByModule('workflow');
        setPermissions(permissions);
    }

    
    /**
     * Method to fetch data product for each sub task except ingest.
     * @param {*} taskItems List of tasks
     */
    const getDataProductDetails = async (taskItems, subtaskTemplates) => {
        // setLoader(true);
        taskItems = taskItems?taskItems:tasks;
        const taskList = [...taskItems];
        for (const task of taskList) {
            if (task.specifications_template.type_value === 'observation' || task.specifications_template.type_value === 'pipeline') {
                const promises = [];
                task.subtasks.map(subtask => {
                    const subtaskTemplate = subtaskTemplates.find(template => template.id===subtask.specifications_template_id);
                    if (subtaskTemplate.type_value.toLowerCase() === 'observation' || 
                            subtaskTemplate.type_value.toLowerCase() === 'pipeline') {
                        promises.push(DataProductService.getSubtaskOutputDataproduct(subtask.id));
                    }
                    return subtask;
                });
                const dataProducts = await Promise.all(promises);
                task['dataProducts'] = dataProducts.filter(product => product.data.length).map(product => product.data).flat();
                task.actionpath = `/task/view/blueprint/${task.id}/dataproducts`;
                task.totalDataSize = _.sumBy(task['dataProducts'], 'size');
                task.dataSizeNotDeleted = _.sumBy(task['dataProducts'], function(product) { return product.deletedSince?0:product.size});
                if (task.totalDataSize) {
                    task.totalDataSize = UnitConverter.getUIResourceUnit('bytes', (task.totalDataSize));
                }
                if (task.dataSizeNotDeleted) {
                    task.dataSizeNotDeleted = UnitConverter.getUIResourceUnit('bytes', (task.dataSizeNotDeleted));
                }
            }
        }
        // setInjestTask(taskList.find(task => task.specifications_template.type_value==='ingest'));
        setTasks(taskList);
        // setLoader(false);
    };

    /**
     * Method to fetch current step workflow details 
     * @param {*} taskList List of tasks
     */
    const getStatusUpdate = async(taskList) => {
        setLoader(true);
        const ingestTabVisible = taskList.filter(task => task.specifications_template.name === 'ingest').length > 0;
        setShowIngestTab(ingestTabVisible);
        const suQAProcess = await WorkflowService.getSUWorkflowProcess(props.match.params.id);
        setQASUProcess(suQAProcess);
        const suQAProcessTasks = await WorkflowService.getWorkflowProcessTasks(suQAProcess.id);
        setWorkflowTasks(suQAProcessTasks);
        let workflowLastTask = (_.orderBy(suQAProcessTasks, ['id'], ['desc']))[0];
        let skippedSteps = [];
        if(workflowLastTask && workflowLastTask.flow_task) {
            let currView = RedirectionMap[workflowLastTask.flow_task.toLowerCase()];
            if (currView) {
                currView = currView.stepValue;
            }   else {
                workflowLastTask = (_.filter(suQAProcessTasks, processTask => _.keys(RedirectionMap).includes(processTask.flow_task.toLowerCase())))[0];
                currView = RedirectionMap[workflowLastTask.flow_task.toLowerCase()].stepValue;
            }
            _.forOwn(RedirectionMap, function(value, key) {
                if(value.stepValue< currView) {
                    if( suQAProcess[value.process]!==undefined && value.process && !suQAProcess[value.process]) {
                        if (value.stepValue > 7 && !ingestTabVisible) {
                            skippedSteps.push(value.stepValue-1);
                        }   else {
                            skippedSteps.push(value.stepValue);
                        }
                    }   else if (suQAProcess[value.process]===undefined && skippedSteps.length>0) {
                        if (skippedSteps[skippedSteps.length-1] < value.stepValue) {
                            skippedSteps.push(value.stepValue);
                        }
                    }
                }
            })
            if (currView === 3) {
                setShowAddSystemEvent(true);
            }
            let currStep = RedirectionMap[workflowLastTask.flow_task.toLowerCase()].stepValue;
            // Need to cross check below if condition if it fails in next click
            if (workflowLastTask.status === 'NEW') {
                currView = RedirectionMap[workflowLastTask.flow_task.toLowerCase()].stepValue;
                currStep = RedirectionMap[workflowLastTask.flow_task.toLowerCase()].stepValue;
            } //else {
            //     setCurrentView(3);
            // }
            else if (workflowLastTask.status.toLowerCase() === 'done' || workflowLastTask.status.toLowerCase() === 'finished') {
                // setDisableNextButton(true);
                currView = 9;
                currStep = 9;
                // if (!(_.find(suQAProcessTasks, processTask => processTask.flow_task.toLowerCase()==="decide unpin data"))) {
                //     let unpinDataStepValue = RedirectionMap["decide unpin data"].stepValue;
                //     skippedSteps.push(showIngestTab?unpinDataStepValue:unpinDataStepValue-1);
                // }
            }
            if (currView > 7) {
                const subtaskTemplates = await TaskService.getSubtaskTemplates();
                await getDataProductDetails(taskList, subtaskTemplates);
                if (!ingestTabVisible) {
                    currView--;
                    currStep--;
                }
            }
            setSkippedSteps(skippedSteps);
            setCurrentView(currView);
            setCurrentStep(currStep);
            setLoader(false); 
        } else {
            setNotFound(true);
            setLoader(false);
        }
    }

    const qaReporting = () => {
        setCurrentView(5)
    };

    const getIngestTask = () => {
        return tasks.find(task => task.specifications_template.type_value==='ingest')
    }

    const getProcessPermission = async () => {
        const permissions = await WorkflowService.getWorkflowProcessPermission(QASUProcess.id);
        let processPermission = permissions?(_.includes(permissions, 'PUT', 'PATCH')):false;
        return processPermission;
    }

    const getCurrentTaskDetails = async () => {
        // const response = await WorkflowService.getCurrentTask(props.match.params.id);
        const currentTask = await WorkflowService.getCurrentTask(QASUProcess.id);
        if (currentTask) {
        const currentTaskpermissions = await WorkflowService.getWorkflowTaskPermission(currentTask.id);
            currentTask.owner_username = currentTaskpermissions.user_name
            currentTask.editPermissions = currentTaskpermissions?(_.includes(currentTaskpermissions.headers, 'PUT', 'PATCH')):false;
        }
        return currentTask;
    };

    const getProject = () => {
        return schedulingUnit.scheduling_set_object.project;
    };

    const getProjectRoles = (view) => {
        const allowedProjectRoles = ALLOWED_PROJECT_ROLES[view];
        return _.filter(projectRoles, projectRole => {
            return allowedProjectRoles.indexOf(projectRole.value)>=0;
        });
    }

   //Pages changes step by step
    const onNext = (content) => {
        setState({...state, ...content});
        getStatusUpdate(tasks);
    };

    const onCancel = () => {
        cancelView()
    }

    /**
     * Function to show different messages from child components
     * @param {String} type - Message type like 'error', 'warning', 'info', 'success'
     * @param {String} title - Summary of the message
     * @param {String} detail - Details of the message
     */
    const showMessage = (type, title, detail) => {
        // growl.show({severity: 'error', summary: 'Unable to proceed', detail: 'Please clear your browser cookies and try again'});
        growl.show({severity: type, summary: title, detail: detail});
    }
    
    const  getStepItems = () => {
        let stepItemsModel = _.cloneDeep(stepItems);
        if (!showIngestTab) {
            _.remove(stepItemsModel, function(stepItem) {return stepItem.label === 'Ingesting'});
        }
        for(const step of skippedSteps) {
            stepItemsModel[step-1].disabled = true;
        }
        return stepItemsModel;
    }

    const getTitle = () => {
        let title = getStepItems()[currentView - 1].label;
        return title;
    }
    const cancelView = () => {
        props.history.push(originPath);
    }
    
    /**
     * Enable/Disable the add System Event button in Qa Reporting To page
     */
    const updateShowSystemEventStatus = async () => {
        setShowAddSystemEvent(false);
    }

    /**
     * Function to call wnen websocket is connected
     */
    const onConnect = async () =>  {
        try{
            console.log("WS Opened");
            const userDets = localStorage.getItem("user");
            if (userDets) {
                websocket.sendMessage(JSON.stringify({"token": JSON.parse(userDets).websocket_token}));
                console.log("Auth token submitted");
            }
        }catch (err) {
            console.log('err',err)
        }
    }

    /**
     * Function to call when websocket is disconnected
     */
    const onDisconnect = async () =>  {
        console.log("WS Closed")
    }

    /**
     * Handles the message received through websocket
     * @param {String} data - String of JSON data
     */
    const handleData = async (data) =>  {
        if (data) {
            console.log('received websocket data:', data)
            const jsonData = JSON.parse(data);
            //check the WS SU id is matched with current SU id
            if (jsonData.object_type === 'scheduling_unit_blueprint' && (props.match.params.id+'') === (jsonData.object_details.id+'')) { 
                if (jsonData.action === 'update') {
                    ScheduleService.getSchedulingUnitExtended('blueprint', props.match.params.id).then ((sub) => {
                        setSchedulingUnit(sub);
                    });
                }   else if (jsonData.action === 'delete') {
                    appGrowl.show({ severity: 'info', summary: 'This Scheduling Unit is not available', detail: 'Scheduling Unit might have been deleted!' });
                    this.setState({redirect: '/su/workflow'});
                }   
            }  
        }
    }

    return (
        
        <>
            { notFound ?
                <NotFound/>
                :<>
                    <Toast ref={(el) => growl = el} />
                    {currentStep && 
                    <PageHeader location={props.location} title={getTitle()} 
                        actions={[{type:'ext_link', icon:'', label: 'SDC Helpdesk', title: 'Report major issues here', props: { pathname: 'https://support.astron.nl/sdchelpdesk' } },
                                    {icon: 'fa-window-close', title: 'Click to Close Workflow', type: 'button',  actOn: 'click', props:{ callback: cancelView }},
                                ]} />}
                    {loader && <AppLoader />}
                    {!loader && schedulingUnit &&
                        <>
                            <div className="p-fluid">
                                {currentView && <div className="p-field p-grid workflow-wrap">
                                    <label htmlFor="suName" className="col-lg-2 col-md-2 col-sm-12">Scheduling Unit</label>
                                    <div className="col-lg-3 col-md-3 col-sm-12">
                                        <Link to={{ pathname: `/schedulingunit/view/blueprint/${schedulingUnit.id}` }}>{schedulingUnit.name}</Link>
                                    </div>
                                    <div className="col-lg-1 col-md-1 col-sm-12"></div>
                                    <label htmlFor="suStatus" className="col-lg-2 col-md-2 col-sm-12">Scheduling Unit Status</label>
                                    <div className="col-lg-3 col-md-3 col-sm-12">
                                        <div>
                                            <span className={"tag-content p-chips-token-label su-"+schedulingUnit.status}>{schedulingUnit.status}</span>
                                        </div>
                                    </div>
                                    <label htmlFor="viewPlots" className="col-lg-2 col-md-2 col-sm-12">View Plots</label>
                                    <div className="col-lg-3 col-md-3 col-sm-12" style={{ paddingLeft: '2px' }}>
                                        <label className="col-sm-10 " >
                                            <a rel="noopener noreferrer" href="https://proxy.lofar.eu/inspect/HTML/" target="_blank">Inspection plots</a>
                                        </label>
                                        <label className="col-sm-10 ">
                                            <a rel="noopener noreferrer" href="https://proxy.lofar.eu/qa" target="_blank">Adder plots</a>
                                        </label>
                                        <label className="col-sm-10 ">
                                            <a href=" https://proxy.lofar.eu/lofmonitor/" target="_blank">Station Monitor</a>
                                        </label>
                                    </div> 
                                    { schedulingUnit.status === 'unschedulable' &&
                                        <>
                                            <div className="col-lg-1 col-md-1 col-sm-12"></div>
                                            <label htmlFor="suStatus" className="col-lg-2 col-md-2 col-sm-12">Unschedulable Reason</label>
                                            <div className="col-lg-3 col-md-3 col-sm-12">
                                                <span >{schedulingUnit.unschedulable_reason}</span>
                                            </div>
                                        
                                        </>
                                    }                                            
                                </div>}
                                <div className={`step-header-${currentStep}`}>
                                    
                                    <Steps model={getStepItems()} activeIndex={currentView - 1} readOnly={false} 
                                            onSelect={(e) => e.index<currentStep?setCurrentView(e.index+1):setCurrentView(currentView)} />
                                </div>
                                <div className="step-content">
                                    {currentView === 1 &&
                                    <Scheduled onNext={onNext} onCancel={onCancel} readOnly={ currentStep !== 1 }
                                                schedulingUnit={schedulingUnit} />
                                    }
                                    {currentView === 2 &&
                                    <ProcessingDone onNext={onNext} onCancel={onCancel} readOnly={ currentStep !== 2 }
                                            schedulingUnit={schedulingUnit}  />
                                    }
                                    {}
                                    {currentView === 3 && userrole.workflow.qa_reporting_to &&
                                    <QAreporting onNext={onNext} onCancel={onCancel} id={QASUProcess.id} readOnly={ currentStep !== 3 } 
                                            process={QASUProcess} getCurrentTaskDetails={getCurrentTaskDetails} processPermission={getProcessPermission}
                                            workflowTask={_.find(workflowTasks, ['flow_task', 'Qa Reporting To'])}
                                            onError={showMessage} projectRoles={getProjectRoles('qa reporting to')} schedulingUnit={schedulingUnit} 
                                            showAddSystemEvent={showAddSystemEvent} callback={updateShowSystemEventStatus} />
                                    }
                                    {currentView === 4 && userrole.workflow.qa_reporting_sos &&
                                    <QAsos onNext={onNext} onCancel={onCancel} id={QASUProcess.id} readOnly={ currentStep !== 4 }
                                            process={QASUProcess} getCurrentTaskDetails={getCurrentTaskDetails} processPermission={getProcessPermission}
                                            workflowTask={_.find(workflowTasks, ['flow_task', 'Qa Reporting Sos'])}
                                            onError={showMessage} projectRoles={getProjectRoles('qa reporting sos')} 
                                            schedulingUnit={schedulingUnit}/>
                                    }
                                    {currentView === 5 && userrole.workflow.pi_verification &&
                                    <PIverification onNext={onNext} onCancel={onCancel} id={QASUProcess.id} readOnly={ currentStep !== 5 } 
                                            process={QASUProcess} getCurrentTaskDetails={getCurrentTaskDetails} processPermission={getProcessPermission}
                                            workflowTask={_.find(workflowTasks, ['flow_task', 'Pi Verification'])}
                                            onError={showMessage} projectRoles={getProjectRoles('pi verification')} 
                                            schedulingUnit={schedulingUnit}/>
                                    }
                                    {currentView === 6 && userrole.workflow.decide_acceptance &&
                                    <DecideAcceptance onNext={onNext} onCancel={onCancel} id={QASUProcess.id} readOnly={ currentStep !== 6 }
                                            process={QASUProcess} getCurrentTaskDetails={getCurrentTaskDetails} processPermission={getProcessPermission}
                                            workflowTask={_.find(workflowTasks, ['flow_task', 'Decide Acceptance'])}
                                            onError={showMessage} project={getProject()}
                                            projectRoles={getProjectRoles('decide acceptance')} />
                                    }
                                    {(showIngestTab && currentView === 7) && 
                                    <Ingesting onNext={onNext} onCancel={onCancel} id={QASUProcess.id} readOnly={ currentStep !== 7 } 
                                            onError={showMessage} task={getIngestTask()} />
                                    }
                                    {currentView === (showIngestTab?8:7) && userrole.workflow.unpin_data &&
                                    <DataProduct onNext={onNext} onCancel={onCancel} id={QASUProcess.id} onError={showMessage} readOnly={ currentStep !== (showIngestTab?8:7) }  
                                            tasks={tasks} getCurrentTaskDetails={getCurrentTaskDetails} processPermission={getProcessPermission}
                                            schedulingUnit={schedulingUnit} projectRoles={getProjectRoles('unpin data')} showIngestProgress={showIngestTab}
                                            workflowTask={_.find(workflowTasks, ['flow_task', 'Decide Unpin Data'])} />
                                    }
                                    {currentView === (showIngestTab?9:8) &&
                                    <Done onNext={onNext} onCancel={onCancel} onError={showMessage} 
                                            reportingPage={qaReporting} readOnly={ currentStep !== 9 } />
                                    }
                                </div>
                            </div>
                            {!loader &&
                                <Websocket ref={websocket => setWebsocket(websocket)} url={process.env.REACT_APP_WEBSOCKET_URL} 
                                    onOpen={onConnect} onMessage={handleData} onClose={onDisconnect} />}
                        </>
                    }
                </>
            }
        </>
    )
};

export default WorkflowView;